package org.processmining.acceptingpetrinet.algorithms;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinet.parameters.PackPetrinetParameters;
import org.processmining.acceptingpetrinet.utils.PetriNetUtils;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

public class PackPetrinetAlgorithm {

	public AcceptingPetriNet apply(PluginContext context, Petrinet net, PackPetrinetParameters parameters) {
		Map<Place, Place> placeMap = new HashMap<Place, Place>(net.getPlaces().size());
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>(net.getTransitions().size());
		Map<Arc, Arc> arcMap = new HashMap<Arc, Arc>(net.getEdges().size());
		Petrinet clonedNet = PetriNetUtils.cloneNet(net, placeMap, transitionMap, arcMap);
		AcceptingPetriNet apn = AcceptingPetriNetFactory.createAcceptingPetriNet();
		apn.init(clonedNet);

		if (parameters.isTryConnections()) {
			/*
			 * Check whether the initial marking is known.
			 */
			try {
				Collection<InitialMarkingConnection> connections = context.getConnectionManager().getConnections(
						InitialMarkingConnection.class, context, net);
				if (connections.size() == 1) {
					/*
					 * Found unique initial marking, use it.
					 */
					apn.setInitialMarking(PetriNetUtils.apply((Marking) connections.iterator().next()
							.getObjectWithRole(InitialMarkingConnection.MARKING), placeMap));
				}
			} catch (ConnectionCannotBeObtained e) {
			}

			/*
			 * Check existing final markings.
			 */
			try {
				Collection<FinalMarkingConnection> connections = context.getConnectionManager().getConnections(
						FinalMarkingConnection.class, context, net);
				Set<Marking> finalMarkings = new HashSet<Marking>();
				for (FinalMarkingConnection connection : connections) {
					finalMarkings.add(PetriNetUtils.apply(
							(Marking) connection.getObjectWithRole(FinalMarkingConnection.MARKING), placeMap));
				}
				apn.setFinalMarkings(finalMarkings);
			} catch (ConnectionCannotBeObtained e) {
			}
			
			/*
			 * Copy layout.
			 */
			GraphLayoutConnection clonedLayoutConnection = new GraphLayoutConnection(apn.getNet());
			try {
				GraphLayoutConnection layoutConnection = context.getConnectionManager().getFirstConnection(GraphLayoutConnection.class, context, net);
				for (Transition transition : transitionMap.keySet()) {
					clonedLayoutConnection.setSize(transitionMap.get(transition), layoutConnection.getSize(transition));
					clonedLayoutConnection.setPosition(transitionMap.get(transition), layoutConnection.getPosition(transition));
				}
				for (Place place : placeMap.keySet()) {
					clonedLayoutConnection.setSize(placeMap.get(place), layoutConnection.getSize(place));
					clonedLayoutConnection.setPosition(placeMap.get(place), layoutConnection.getPosition(place));
				}
				for (Arc arc : arcMap.keySet()) {
					clonedLayoutConnection.setEdgePoints(arcMap.get(arc), layoutConnection.getEdgePoints(arc));
				}
				clonedLayoutConnection.setLayedOut(true);
			} catch (ConnectionCannotBeObtained e) {
				// Never mind.
			}
			context.getConnectionManager().addConnection(clonedLayoutConnection);

		}
		return apn;
	}
}
