package org.processmining.acceptingpetrinet.algorithms;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.parameters.UnpackPetrinetParameters;
import org.processmining.acceptingpetrinet.utils.PetriNetUtils;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

public class UnpackPetrinetAlgorithm {

	public Petrinet apply(PluginContext context, AcceptingPetriNet apn, UnpackPetrinetParameters parameters) {
		Map<Place, Place> placeMap = new HashMap<Place, Place>(apn.getNet().getPlaces().size());
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>(
				apn.getNet().getTransitions().size());
		Map<Arc, Arc> arcMap = new HashMap(apn.getNet().getEdges().size());
		Petrinet net = PetriNetUtils.cloneNet(apn.getNet(), placeMap, transitionMap, arcMap);
		Marking initialMarking = PetriNetUtils.apply(apn.getInitialMarking(), placeMap);
		Set<Marking> finalMarkings = PetriNetUtils.apply(apn.getFinalMarkings(), placeMap);

		if (parameters.isTryConnections()) {
			context.getProvidedObjectManager().createProvidedObject("Initial marking for " + net.getLabel(),
					initialMarking, Marking.class, context);
			context.addConnection(new InitialMarkingConnection(net, initialMarking));

			for (Marking finalMarking : finalMarkings) {
				context.getProvidedObjectManager().createProvidedObject("Final marking for " + net.getLabel(),
						finalMarking, Marking.class, context);
				context.addConnection(new FinalMarkingConnection(net, finalMarking));
			}

			/*
			 * Copy layout.
			 */
			GraphLayoutConnection clonedLayoutConnection = new GraphLayoutConnection(net);
			try {
				GraphLayoutConnection layoutConnection = context.getConnectionManager()
						.getFirstConnection(GraphLayoutConnection.class, context, apn.getNet());
				for (Transition transition : transitionMap.keySet()) {
					clonedLayoutConnection.setSize(transitionMap.get(transition), layoutConnection.getSize(transition));
					clonedLayoutConnection.setPosition(transitionMap.get(transition),
							layoutConnection.getPosition(transition));
				}
				for (Place place : placeMap.keySet()) {
					clonedLayoutConnection.setSize(placeMap.get(place), layoutConnection.getSize(place));
					clonedLayoutConnection.setPosition(placeMap.get(place), layoutConnection.getPosition(place));
				}
				for (Arc arc : arcMap.keySet()) {
					clonedLayoutConnection.setEdgePoints(arcMap.get(arc), layoutConnection.getEdgePoints(arc));
				}
				clonedLayoutConnection.setLayedOut(true);
			} catch (ConnectionCannotBeObtained e) {
				// Never mind.
			}
			context.getConnectionManager().addConnection(clonedLayoutConnection);
		}

		return net;
	}
}
