package org.processmining.acceptingpetrinet.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.algorithms.MergeAcceptingPetriNetArrayIntoAcceptingPetriNetAlgorithm;
import org.processmining.acceptingpetrinet.connections.MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection;
import org.processmining.acceptingpetrinet.help.MergeAcceptingPetriNetArrayIntoAcceptingPetriNetHelp;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.parameters.MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Merge Accepting Petri Nets", parameterLabels = { "Accepting Petri Net Array", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = MergeAcceptingPetriNetArrayIntoAcceptingPetriNetHelp.TEXT)
public class MergeAcceptingPetriNetArrayIntoAcceptingPetriNetPlugin extends MergeAcceptingPetriNetArrayIntoAcceptingPetriNetAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Merge Accepting Petri Nets, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, AcceptingPetriNetArray nets) {
		MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters parameters = new MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters();
		return runConnection(context, nets, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet mergeDefault(PluginContext context, AcceptingPetriNetArray nets) {
		MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters parameters = new MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters();
		return runConnection(context, nets, parameters);
	}

	@PluginVariant(variantLabel = "Merge Accepting Petri Nets, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, AcceptingPetriNetArray nets,
			MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters parameters) {
		return runConnection(context, nets, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet mergeParameters(PluginContext context, AcceptingPetriNetArray nets,
			MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters parameters) {
		return run(context, nets, parameters);
	}

	private AcceptingPetriNet runConnection(PluginContext context, AcceptingPetriNetArray nets,
			MergeAcceptingPetriNetArrayIntoAcceptingPetriNetParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection.class, context, nets);
				for (MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection connection : connections) {
					if (connection.getObjectWithRole(MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection.NETS)
							.equals(nets) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection.NET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet markedNet = apply(context, nets, parameters);
		if (parameters.isTryConnections()) {
			context.getProvidedObjectManager().createProvidedObject(
					"Initial marking for " + markedNet.getNet().getLabel(), markedNet.getInitialMarking(),
					Marking.class, context);
			context.addConnection(new InitialMarkingConnection(markedNet.getNet(), markedNet.getInitialMarking()));
			context.getConnectionManager().addConnection(
					new MergeAcceptingPetriNetArrayIntoAcceptingPetriNetConnection(nets, markedNet, parameters));
		}
		return markedNet;
	}

}
