/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.framework.connections.Connection;
import org.processmining.models.connections.petrinets.behavioral.BehavioralAnalysisInformationConnection;
import org.processmining.models.connections.petrinets.behavioral.BoundednessInfoConnection;
import org.processmining.models.connections.petrinets.behavioral.DeadTransitionsConnection;
import org.processmining.models.connections.petrinets.behavioral.HomeMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.LivenessInfoConnection;
import org.processmining.models.connections.petrinets.behavioral.MarkingsetNetConnection;
import org.processmining.models.connections.petrinets.behavioral.NonLiveTransitionsConnection;
import org.processmining.models.connections.petrinets.behavioral.NonRelaxedSoundTransitionsConnection;
import org.processmining.models.connections.petrinets.behavioral.RelaxedSoundessInfoConnection;
import org.processmining.models.connections.petrinets.behavioral.UnboundedPlacesConnection;
import org.processmining.models.connections.petrinets.behavioral.UnboundedSequencesConnection;
import org.processmining.models.connections.petrinets.structural.AbstractComponentSetConnection;
import org.processmining.models.connections.petrinets.structural.AbstractInvariantMarkingConnection;
import org.processmining.models.connections.petrinets.structural.AbstractNetHandleConnection;
import org.processmining.models.connections.petrinets.structural.AbstractStructuralAnalysisInformationConnection;
import org.processmining.models.connections.petrinets.structural.ExtFreeChoiceInfoConnection;
import org.processmining.models.connections.petrinets.structural.FreeChoiceInfoConnection;
import org.processmining.models.connections.petrinets.structural.NonExtendedFreeChoiceClustersConnection;
import org.processmining.models.connections.petrinets.structural.NonFreeChoiceClustersConnection;
import org.processmining.models.connections.petrinets.structural.PTHandleConnection;
import org.processmining.models.connections.petrinets.structural.PlaceInvariantConnection;
import org.processmining.models.connections.petrinets.structural.SComponentConnection;
import org.processmining.models.connections.petrinets.structural.SiphonConnection;
import org.processmining.models.connections.petrinets.structural.TComponentConnection;
import org.processmining.models.connections.petrinets.structural.TPHandleConnection;
import org.processmining.models.connections.petrinets.structural.TransitionInvariantConnection;
import org.processmining.models.connections.petrinets.structural.TrapConnection;
import org.processmining.models.graphbased.directed.petrinet.analysis.DeadTransitionsSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.HomeMarkingSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NetAnalysisInformation;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonExtendedFreeChoiceClustersSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonFreeChoiceClustersSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonLiveTransitionsSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonRelaxedSoundTransitionsSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.PTHandles;
import org.processmining.models.graphbased.directed.petrinet.analysis.PlaceInvariantSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.SComponentSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.SiphonSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.TComponentSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.TPHandles;
import org.processmining.models.graphbased.directed.petrinet.analysis.TransitionInvariantSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.TrapSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.UnboundedPlacesSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.UnboundedSequences;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version May 31, 2010
 */
public class PNPluginSelectionPanel extends JPanel {
	private static final long serialVersionUID = 8854230239880636749L;

	public static enum AnalysisTypes {
		DEADTRANSITIONS("Dead Transitions", true, DeadTransitionsConnection.class, DeadTransitionsSet.class,
				DeadTransitionsConnection.TRANSITIONS), //
		UNBOUNDEDPLACES("Unbounded Places", true, //
				UnboundedPlacesConnection.class, UnboundedPlacesSet.class, UnboundedPlacesConnection.PLACES), //
		BOUNDEDNESSSEQ("Unbounded Sequences", true, //
				UnboundedSequencesConnection.class, UnboundedSequences.class, UnboundedSequencesConnection.SEQUENCES), //
		BOUNDEDNESSINFO("Unbounded Info",
				true, //
				BoundednessInfoConnection.class, NetAnalysisInformation.BOUNDEDNESS.class,
				BehavioralAnalysisInformationConnection.NETANALYSISINFORMATION), //
		HOMEMARKINGSET("Home Marking", true, //
				HomeMarkingConnection.class, HomeMarkingSet.class, MarkingsetNetConnection.MARKINGS), //
		LIVENESSINFO("Liveness Info",
				true, //
				LivenessInfoConnection.class, NetAnalysisInformation.LIVENESS.class,
				BehavioralAnalysisInformationConnection.NETANALYSISINFORMATION), //
		LIVENESSTRANS("Non Live Transitions",
				true, //
				NonLiveTransitionsConnection.class, NonLiveTransitionsSet.class,
				NonLiveTransitionsConnection.TRANSITIONS), //
		PLACEINVARIANT("Place Invariants", false, PlaceInvariantConnection.class, PlaceInvariantSet.class,
				AbstractInvariantMarkingConnection.INVARIANTMARKING), //
		TRANSITIONINVARIANT("Transition Invariants", false, TransitionInvariantConnection.class,
				TransitionInvariantSet.class, AbstractInvariantMarkingConnection.INVARIANTMARKING), //
		TPHANDLES("T-P Handles", false, TPHandleConnection.class, TPHandles.class, AbstractNetHandleConnection.HANDLES), //
		PTHANDLES("P-T Handles", false, PTHandleConnection.class, PTHandles.class, AbstractNetHandleConnection.HANDLES), SCOMPONENTSET(
				"S-Components", false, SComponentConnection.class, SComponentSet.class,
				AbstractComponentSetConnection.COMPONENTSET), //
		TCOMPONENTSET("T-Components", false, TComponentConnection.class, TComponentSet.class,
				AbstractComponentSetConnection.COMPONENTSET), //
		SIPHONSET("Siphons", false, SiphonConnection.class, SiphonSet.class,
				AbstractComponentSetConnection.COMPONENTSET), //
		TRAPSET("Traps", false, TrapConnection.class, TrapSet.class, AbstractComponentSetConnection.COMPONENTSET), //
		FREECHOICE("Free-choice info", false, FreeChoiceInfoConnection.class, NetAnalysisInformation.FREECHOICE.class,
				AbstractStructuralAnalysisInformationConnection.NETANALYSISINFORMATION), //
		EXTFREECHOICE("Extended-free-choice info", false, ExtFreeChoiceInfoConnection.class, NetAnalysisInformation.EXTFREECHOICE.class,
				AbstractStructuralAnalysisInformationConnection.NETANALYSISINFORMATION), //
		NONFREECHOICECLUS("Non-free-choice clusters", false, NonFreeChoiceClustersConnection.class,
				NonFreeChoiceClustersSet.class, NonFreeChoiceClustersConnection.CLUSTERS), //
		NONFREEEXTCHOICECLUS("Non-extended-free-choice clusters", false, NonExtendedFreeChoiceClustersConnection.class,
				NonExtendedFreeChoiceClustersSet.class, NonExtendedFreeChoiceClustersConnection.CLUSTERS),
		RELAXEDSOUNDESS("Relaxed soundness info (using LoLA for Windows)", true, RelaxedSoundessInfoConnection.class, NetAnalysisInformation.RELAXEDSOUND.class, BehavioralAnalysisInformationConnection.NETANALYSISINFORMATION), 
		UNFIREABLETRANSITIONRELAXEDSOUNDNESS("Non-relaxed sound transitions (using LoLA for Windows)", true, NonRelaxedSoundTransitionsConnection.class, NonRelaxedSoundTransitionsSet.class, NonRelaxedSoundTransitionsConnection.TRANSITIONS);

		private final String name;
		private final Class<? extends Connection> connection;
		private final boolean isBehavioral;
		private final String role;
		private final Class<?> objType;

		private AnalysisTypes(String name, boolean isBehavioral, Class<? extends Connection> connection,
				Class<?> objType, String role) {
			this.name = name;
			this.connection = connection;
			this.isBehavioral = isBehavioral;
			this.objType = objType;
			this.role = role;

		}

		public boolean isBehavioral() {
			return isBehavioral;
		}

		public String getName() {
			return name;
		}

		public Class<? extends Connection> getConnectionType() {
			return connection;
		}

		public Class<?> getObjectType() {
			return objType;
		}

		public String getRole() {
			return role;
		}

	}

	SlickerPluginDependencyConfiguration plugConfig = null;

	public PNPluginSelectionPanel(Set<AnalysisTypes> calculatedProp,
			Map<AnalysisTypes, Set<AnalysisTypes>> dependencies, String header, String description) {
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		JLabel headerLabel = new JLabel(header);
		headerLabel.setOpaque(false);
		headerLabel.setFont(headerLabel.getFont().deriveFont(14f));
		add(packLeftAligned(headerLabel));

		JLabel descriptionLabel = new JLabel("<html>" + description + "</html>");
		descriptionLabel.setOpaque(false);
		descriptionLabel.setFont(descriptionLabel.getFont().deriveFont(11f));
		add(packLeftAligned(descriptionLabel));

		plugConfig = new SlickerPluginDependencyConfiguration(calculatedProp, dependencies);
		add(plugConfig);
	}

	public Set<AnalysisTypes> getPropertiesToBeCalculated() {
		return plugConfig.getPluginsToBeCalculated();
	}

	protected JComponent packLeftAligned(JComponent component) {
		JPanel packed = new JPanel();
		packed.setOpaque(false);
		packed.setBorder(BorderFactory.createEmptyBorder());
		packed.setLayout(new BoxLayout(packed, BoxLayout.X_AXIS));
		packed.add(component);
		packed.add(Box.createHorizontalGlue());
		return packed;
	}
}
