package org.processmining.plugins.properties.auxiliary;

import java.util.Collection;

import org.apache.commons.math3.distribution.AbstractRealDistribution;
import org.apache.commons.math3.distribution.ExponentialDistribution;
import org.apache.commons.math3.distribution.GammaDistribution;
import org.apache.commons.math3.distribution.NormalDistribution;
import org.apache.commons.math3.distribution.WeibullDistribution;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.impl.Distribution;

/**
 * 
 * @author DSchunse
 *
 */
public class HelperFunctions {
	public static <K> boolean overlap(Collection<K> elements, Collection<K> elementsp){
		if(elements == null || elementsp == null){
			return false;
		}
		for(K element: elements){
			if(elementsp.contains(element)){
				return true;
			}
		}
		return false;
	}
	
	public static <K> boolean disjoint(Collection<K> elements, Collection<K> elementsp){
		return !overlap(elements, elementsp);
	}
	
	/**
	 * 
	 * @param distr the distribution we want to have the minimum and maximum of
	 * @param probMin cut-off point for the minimal value, i.e., if the probability of values come below this probability they are no longer taken into account
	 * @param probMax cut-off point for the maximal value, i.e., if the probability of values come below this probability they are no longer taken into account
	 * @return
	 * @throws MathException
	 */
	public static Pair<Double, Double> minMax(Distribution distr, Double probMin, Double probMax){
		// let's first cast the distr to the right distribution
		AbstractRealDistribution abstrDistr = null;
		Pair<Double, Double> ret = null;
		switch(distr.getDistributionType()){
			case Beta:
				// by definition beta is defined on this interval
				ret = new Pair<Double, Double>(0d, 1d);
				break;
			case Constant:
				ret = new Pair<Double, Double>(distr.getMean(), distr.getMean());
				break;
			case Erlang:
				break;
			case Gamma:
				abstrDistr = new GammaDistribution(distr.getShape(), distr.getScale());
				break;
			case LogNormal:
				break;
			case NegativeExponential:
				abstrDistr = new ExponentialDistribution(distr.getMean());
				break;
			case Normal:
				abstrDistr = new NormalDistribution(distr.getMean(), Math.sqrt(distr.getVariance()));
				break;
			case Uniform:
				ret =  new Pair<Double, Double>(distr.getMinimum(), distr.getMaximum());
				break;
			case Weibull:
				abstrDistr = new WeibullDistribution(distr.getScale(), distr.getShape());
				break;
			default:		
		}
		// now we have the abstract distribution
		if(abstrDistr == null && ret == null){
			throw new NullPointerException("Distribution not supported");
		}
		if(ret != null){
			return new Pair<Double, Double>(distr.getTranslationConstant() + distr.getScaleFactor()*ret.getFirst(), distr.getTranslationConstant() + distr.getScaleFactor()*ret.getSecond());
		}
		return new Pair<Double, Double>(distr.getTranslationConstant() + distr.getScaleFactor()*abstrDistr.inverseCumulativeProbability(probMin), distr.getTranslationConstant() + distr.getScaleFactor()*abstrDistr.inverseCumulativeProbability(1 - probMax));
	}
}
