package org.processmining.plugins.properties.processmodel;

import java.io.Serializable;
import java.util.Collection;
import java.util.Set;

import javax.swing.ImageIcon;

import org.processmining.plugins.properties.enumerations.Constructs;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
public interface Property<K> extends Serializable {
	/**
	 * @return the unique ID for this property for import and export functionality
	 */
	public Long getID();
	
	/**
	 * @return the print friendly name of this property
	 */
	public String getName();
	
	/**
	 * @return the cloned version of this property
	 */
	public K clone(Object element);
	
	/**
	 * @param o1 a value for a property
	 * @param o2 another value for a property
	 * @return > 0 if and only if o1 is better than o2 
	 */
	public int compare(Object o1, Object o2);
	
	
	/**
	 * @return The dependencies of this property, some properties can only be set when certain other properties have been set
	 */
	public Collection<Class<? extends Property<?>>> getDependencies();
	
	/**
	 * @return the set of constructs under which this property is monotone
	 */
	public Set<Constructs> monotonicity();
	
	/**
	 * @return returns the type of objects for which it makes sense to have this property
	 */
	public Set<Constructs> getMeaningfulTypeConstructs();
	
	/**
	 * We want to have a default value for a property s.t., we can reason on it
	 * @return The default value of a property
	 */
	public K getDefaultValue();
	
	/**
	 * @param value The value one wants to have a single double of
	 * @return
	 */
	public double getValueDouble(Object value);

	/**
	 * @return the icon to be used in the visualisation (note this can be null)
	 */
	public ImageIcon getIcon();
	
	/**
	 * @return the GUI to set this property
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	public GUIProperty<K> getGUIProperty() throws InstantiationException, IllegalAccessException;
	
	/**
	 * @return denotes if the ordering matters (e.g., distribution does not have an ordering)
	 */
	public boolean hasOrdering();
	
	/**
	 * @return denotes if a higher value is better or not
	 */
	public boolean higherBetter();
	
	/**
	 * @param value the value of the object to be written
	 * @return the xml string of the object
	 */
	public String marshall(Object value);
	
	/**
	 * @param xml the xml fragment to be parsed
	 * @return the initialised data-structure
	 */
	public Object unmarshall(String xml);
	
}
