package org.processmining.plugins.properties.processmodel.abstractproperty;

import org.processmining.plugins.properties.enumerations.TimeOptions;
import org.processmining.plugins.properties.processmodel.TimeValue;

/**
 * 
 * @author DSchunse
 *
 */
public class TimeValueImpl implements TimeValue{
	TimeOptions timeOption;
	Double value;
	
	public TimeValueImpl(){
		this(0d);
	};
	
	public TimeValueImpl(Double value){
		this(TimeOptions.millisecond, value);
	}
	
	public TimeValueImpl(TimeOptions timeOption, Double value){
		this.timeOption = timeOption;
		this.value = value;
	}
	
	public TimeValueImpl(TimeValue tv){
		this(tv.getTimeGranularity(), new Double(tv.getValue()));
	}
	
	public TimeOptions getTimeGranularity() {
		return timeOption;
	}
	public void setTimeGranularity(TimeOptions option) {
		if(option != null){
			this.timeOption = option;
		}
		else{
			this.timeOption = TimeOptions.millisecond;
		}
	}
	
	public Double getValue() {
		return value;
	}
	public void setValue(Double value) {
		this.value = value;
	}
	
	public Double toMilliseconds() {
		// this is our basis, i.e., everything will come from this
		Double result = value;
		result = timeOption.compareTo(TimeOptions.second) >= 0 ? result * 1000 : result;
		result = timeOption.compareTo(TimeOptions.minute) >= 0 ? result * 60 : result;
		result = timeOption.compareTo(TimeOptions.hour) >= 0 ? result * 60 : result;
		result = timeOption.compareTo(TimeOptions.day) >= 0 ? result * 24 : result;
		result = timeOption.compareTo(TimeOptions.week) >= 0 ? result * 7 : result;
		result = timeOption.compareTo(TimeOptions.month) >= 0 ? result * 4 : result;
		result = timeOption.compareTo(TimeOptions.year) >= 0 ? result * 12 : result;
		
		return result;
	}
	
	public Double toSeconds() {
		return toMilliseconds() / 1000;
	}
	
	public Double toMinutes() {
		return toSeconds() / 60;
	}
	
	public Double toHours() {
		return toMinutes() / 60;
	}
	
	public Double toDays() {
		return toHours() / 24;
	}
	
	public Double toWeeks() {
		return toDays() / 7;
	}
	
	public Double toMonths() {
		return toWeeks() / 4;
	}
	
	public Double toYears() {
		return toMonths() / 12;
	}
	
	public Double toTimeGranularity(TimeOptions timeOption){
		switch(timeOption){
			case millisecond:
				return toMilliseconds();
			case second:
				return toSeconds();
			case minute:
				return toMinutes();
			case hour:
				return toHours();
			case day:
				return toDays();
			case week:
				return toWeeks();
			case month:
				return toMonths();
			case year:
				return toYears();
			default: return null;
		}
	}
	
	public String toString(){
		return "timeOption = " + timeOption + " value = " + value;
	}
	
	public void valueOf(String value) {
		// we have two options for backwards compatibility
		if(value.contains("timeOption")){
			this.timeOption = TimeOptions.valueOf(value.split("timeOption = ")[1].split(" value = ")[0]);
			this.value = Double.valueOf(value.split("value = ")[1]);
		}
		else{
			// we have a double
			this.timeOption = TimeOptions.millisecond;
			this.value = Double.valueOf(value);
		}
	}
	
	
}
