package org.processmining.plugins.properties.processmodel.configurable.abstractproperty;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurableProperty;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurablePropertyConfiguration;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurablePropertyIterator;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
public class ConfigurablePropertyIteratorImpl<K> implements ConfigurablePropertyIterator<K>{

	private ConfigurableProperty<K> configurableProperty;
	private Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties = null;
	private BitSet currentConfiguration;
	private List<Object> removable;
	private Pair<K, K> values;
	private BitSet lastReturnedConfiguration = null;
	private int bitLength;
	private BitSet minConfiguration = null;
	private BitSet maxConfiguration = null;
	
	@Deprecated
	public ConfigurablePropertyIteratorImpl(ConfigurableProperty<K> configurableProperty, Pair<K, K> values){
		this(configurableProperty, values, null, null, null);
	}
	
	public ConfigurablePropertyIteratorImpl(ConfigurableProperty<K> configurableProperty, Pair<K, K> values, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties){
		this(configurableProperty, values, null, null, dependentProperties);
	}
	
	public ConfigurablePropertyIteratorImpl(ConfigurableProperty<K> configurableProperty, Pair<K, K> values, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties, Object minConfiguration, Object maxConfiguration){
		this.configurableProperty = configurableProperty;
		this.values = values;
		this.dependentProperties = dependentProperties;
		this.minConfiguration = (BitSet) minConfiguration;
		this.maxConfiguration = (BitSet) maxConfiguration;
		fillRemovable(values.getSecond());
		this.bitLength = getBitLength();
		sort();
		currentConfiguration = new BitSet(bitLength + 2); // last 2 bits is for denoting termination, and fixing the size
		currentConfiguration.clear();
		currentConfiguration.set(bitLength + 1, true);
		if(this.minConfiguration != null){
			currentConfiguration = (BitSet) this.minConfiguration.clone();
		}
	}
	
	private void fillRemovable(K values){
		removable = new ArrayList<Object>();
		if(values instanceof List<?>){
			List<?> valuesL = (List<?>)values;
			for(Object e: valuesL){
				removable.add(e);
			}
		}
		else{
			throw new NotImplementedException();
		}
	}
	
	private int getBitLength(){
		int bitLength = removable.size();
		return bitLength;
	}
	
	private void sort(){
		//Collections.sort(removable);
	}
	
	// so we have a list of total values, and removable values
	public ConfigurablePropertyIterator<K> createIterator(ConfigurableProperty<K> configurableProperty, Pair<K, K> values) {
		return new ConfigurablePropertyIteratorImpl<K>(configurableProperty, values);
	}
	
	public ConfigurablePropertyIterator<K> createIterator(ConfigurableProperty<K> configurableProperty, Pair<K, K> values, boolean total) {
		return new ConfigurablePropertyIteratorImpl<K>(configurableProperty, values);
	}
	
	public ConfigurablePropertyIterator<K> createIterator(ConfigurableProperty<K> configurableProperty, Pair<K, K> values, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties) {
		return new ConfigurablePropertyIteratorImpl<K>(configurableProperty, values, dependentProperties);
	}
	
	public ConfigurablePropertyIterator<K> createIterator(ConfigurableProperty<K> configurableProperty, Pair<K, K> values, boolean total, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties) {
		return new ConfigurablePropertyIteratorImpl<K>(configurableProperty, values, dependentProperties);
	}

	public boolean hasNext() throws Exception {
		BitSet currentC = (BitSet) currentConfiguration.clone();
		BitSet lastRetConf = null;
		if(lastReturnedConfiguration != null){
			lastRetConf = (BitSet) lastReturnedConfiguration.clone();
		}
		if(nextConfiguration() != null){
			currentConfiguration = currentC;
			lastReturnedConfiguration = lastRetConf;
			return true;
		}
		else{
			return false;
		}
	}

	public ConfigurablePropertyConfiguration<K> getNext() throws Exception {
		if(hasNext()){
			return nextConfiguration();
		}
		throw new NullPointerException();
	}
	
	public void reset(){
		currentConfiguration = new BitSet(bitLength + 2); // last 2 bits is for denoting termination, and fixing the size
		currentConfiguration.clear();
		currentConfiguration.set(bitLength + 1, true);
		if(this.minConfiguration != null){
			currentConfiguration = (BitSet) this.minConfiguration.clone();
		}
		lastReturnedConfiguration = null;
	}

	public List<Pair<Object, Object>> getSubdivisions(int divisions) throws Exception {
		// TODO Auto-generated method stub
		return null;
	}
	
	private ConfigurablePropertyConfiguration<K> nextConfiguration() throws Exception{
		ConfigurablePropertyConfiguration<K> conf = new ConfigurablePropertyConfiguration<K>();
		if(lastReturnedConfiguration == null){
			fillConfiguration(conf);
			configurableProperty.makeTotalConfiguration(values, conf);
		}
		if(currentConfiguration.equals(lastReturnedConfiguration) && !currentConfiguration.get(currentConfiguration.length() - 2)){
			nextBitSequece(currentConfiguration);
			fillConfiguration(conf);
			configurableProperty.makeTotalConfiguration(values, conf);
		}
		while((!isConsistentComplete(conf) && !currentConfiguration.get(currentConfiguration.length() - 2) && smaller(currentConfiguration, maxConfiguration))){
			nextBitSequece(currentConfiguration);
			conf = new ConfigurablePropertyConfiguration<K>();
			fillConfiguration(conf);
			configurableProperty.makeTotalConfiguration(values, conf);
		}
		if(isConsistentComplete(conf) && !currentConfiguration.get(currentConfiguration.length() - 2) && !(currentConfiguration.equals(lastReturnedConfiguration)) && smaller(currentConfiguration, maxConfiguration)){
			lastReturnedConfiguration = (BitSet) currentConfiguration.clone();
			return conf;
		}
		else{
			return null;
		}
	}
	
	private static void nextBitSequece(BitSet currentConfiguration){
		// we have to add one to the bitsequence
		boolean carry = true;
		for(int i = 0; i < currentConfiguration.length() - 1; i++){
			boolean tempcarry = currentConfiguration.get(i) && carry;
			currentConfiguration.set(i, (currentConfiguration.get(i) || carry) && !tempcarry);
			carry = tempcarry;
		}
	}
	
	private static boolean smaller(BitSet left, BitSet right) throws Exception{
		if(right == null){
			// special case, this way we do not need to explicitly model the maximal bitsequence possible
			return true;
		}
		if(left.size() != right.size()){
			// bit weird... let's throw an exception
			throw new Exception("Bit Sequences cannot be different in size\n");
		}
		else{
			for(int i = left.size() - 1; i >= 0; i--){
				if(left.get(i) && !right.get(i)){
					return false;
				}
				if(!left.get(i) && right.get(i)){
					return true;
				}
			}
			// they are the same, so it is not smaller
			return false;
		}
	}
	
	@SuppressWarnings("unchecked")
	private void fillConfiguration(ConfigurablePropertyConfiguration<K> conf){
		// note the -2 due to the control bits
		// we only need to take care we are working on lists
		List<Object> removed = new ArrayList<Object>();
		for(int i = 0; i < currentConfiguration.length() - 2; i++){
			if(currentConfiguration.get(i)){
				// we only have a single list, so we do not need to do any hassle :-D
				removed.add(removable.get(i));
			}
		}
		conf.setRemoved((K) removed);
		conf.setNonRemoved((K) new ArrayList<Object>());
	}
	
	private boolean isConsistentComplete(ConfigurablePropertyConfiguration<K> conf){
		return configurableProperty.isValidConfiguration(values, conf, dependentProperties) && configurableProperty.isTotalConfiguration(values, conf);
	}

}
