package org.processmining.plugins.properties.processmodel.configurable.abstractproperty;

import java.util.ArrayList;
import java.util.List;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.auxiliary.XMLHelper;
import org.processmining.plugins.properties.processmodel.impl.Distribution;

/**
 * 
 * @author DSchunse
 *
 */
@SuppressWarnings("serial")
public abstract class ConfigurablePropertyListList extends ConfigurablePropertyList<List<Object>> {

	@SuppressWarnings("unchecked")
	public String marshall(Object value){
		return this.marshall((Pair<List<List<Object>>, List<List<Object>>>)value);
	}
	
	public Pair<List<List<Object>>, List<List<Object>>> getDefaultValue() {
		return new Pair<List<List<Object>>, List<List<Object>>>(new ArrayList<List<Object>>(), new ArrayList<List<Object>>());
	}
		
	public String marshall(Pair<List<List<Object>>, List<List<Object>>> elements){
		String ret = "<valueEntryElements>\n";
		ret += marshall(elements.getFirst());
		ret += "</valueEntryElements>\n";
		ret += "<valueEntryRemovable>\n";
		ret += marshall(elements.getSecond());
		ret += "</valueEntryRemovable>\n";
		return ret;
	}
	
	public Object unmarshall(String xml){
		String first = xml.substring(xml.indexOf("<valueEntryElements>") + "<valueEntryElements>".length(), xml.indexOf("</valueEntryElements>"));
		String second = xml.substring(xml.indexOf("<valueEntryRemovable>") + "<valueEntryRemovable>".length(), xml.indexOf("</valueEntryRemovable>"));
		return new Pair<Object, Object>(unmarshallPrivate(first), unmarshallPrivate(second));
	}
	
	public String marshall(List<List<Object>> elements){
		String ret = "";
		for(List<Object> element: elements){
			ret += "<elementValue ";
			int index = 0;
			for(Object o: element){
				if(o instanceof String){
					ret += "elementString_" + index + " = \"" + XMLHelper.escape((String)o) + "\" ";
				}
				else if(o instanceof Double){
					ret += "elementDouble_" + index + " = \"" + XMLHelper.escape(((Double)o).doubleValue() + "") + "\" ";
				}
				else if(o instanceof Integer){
					ret += "elementInteger_" + index + " = \"" + XMLHelper.escape(((Integer)o).intValue() + "") + "\" ";
				}
				else if(o instanceof Boolean){
					ret += "elementBoolean_" + index + " = \"" + XMLHelper.escape(((Boolean)o).booleanValue() + "") + "\" ";
				}
				else if(o instanceof Distribution){
					ret += "elementDistribution_" + index + " = \"" + XMLHelper.escape(((Distribution)o).distributionValue()) + "\"";
				}
				else{
					throw new NullPointerException("Conversion for type: " + o.getClass().getName() + " undefined");
				}
				index++;
			}
			ret += "/>\n";
		}
		return ret;
	}
	
	private List<List<Object>> unmarshallPrivate(String xml){
		// we have a list of elementValues
		List<List<Object>> ret = new ArrayList<List<Object>>();
		String[] elements = xml.split("<elementValue");
		for(String element: elements){
			List<Object> retElem = new ArrayList<Object>();
			// lets split this on the element thing
			String[] elem = element.split("element");
			// now we need to parse the whole lot
			int index = 0;
			for(String s: elem){
				Object o = null;
				String type = s.split("_")[0];
				String indexS = s.split("_")[1].split(" = ")[0];
				String value = s.split("\"")[1];
				int indexInt = Integer.parseInt(indexS);
				if(type.equalsIgnoreCase("String")){
					o = new String(XMLHelper.unescape(value));
				}
				else if(type.equalsIgnoreCase("Double")){
					o = new Double(XMLHelper.unescape(value));
				}
				else if(type.equalsIgnoreCase("Integer")){
					o = new Integer(XMLHelper.unescape(value));
				}
				else if(type.equalsIgnoreCase("Boolean")){
					o = new Boolean(XMLHelper.unescape(value));
				}
				else if(type.equalsIgnoreCase("Distribution")){
					o = new Distribution(XMLHelper.unescape(value));
				}
				else{
					throw new NullPointerException("Conversion for type: " + type + " undefined");
				}
				if(index == indexInt){
					retElem.add(o);
				}
				else{
					throw new NullPointerException("We have an index inconsistency");
				}
				index++;
			}
			ret.add(retElem);
		}
		return ret;
	}
}
