package org.processmining.plugins.properties.processmodel.gui.abstractguiproperty;

import info.clearthought.layout.TableLayout;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
public abstract class GUIPropertyListOfElements<K> implements GUIProperty<K> {

	public <E extends PropertableElement, C extends Property<?>> void initialiseGUIProperty(UIPluginContext context, E model, Collection<E> elements, C property, HashMap<E, Object> mapping, boolean independent) throws InstantiationException, IllegalAccessException {
		HashMap<E, List<List<List<Object>>>> elementLines = new HashMap<E, List<List<List<Object>>>>();
		JComponent tabPaneSuper = initialiseTabPane(elements, property, elementLines, true, independent);
		context.showConfiguration("Configuring " + property.getName(), tabPaneSuper);
		for(E element: elements){
			List<List<List<Object>>> lines = elementLines.get(element);
			if(independent){
				element.setIndependentProperty(property, parseComponents(property, lines));
			}
			else{
				element.setDependentProperty(property, parseComponents(property, lines));
			}
			if(mapping != null){
				mapping.put(element, parseComponents(property, lines));
			}
		}
	}
	
	public <E extends PropertableElement> List<List<JComponent>> getFooterRows(E element) {
		return new ArrayList<List<JComponent>>();
	}

	public <E extends PropertableElement, C extends Property<?>> JComponent initialiseTabPane(Collection<E> elements, C property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException{
		JTabbedPane tabPaneSuper = new JTabbedPane();
		tabPaneSuper.setTabLayoutPolicy(JTabbedPane.WRAP_TAB_LAYOUT);
		for(E element: elements){
			tabPaneSuper.addTab(element.getName(), initialiseTabPane(element, property, elementLines, editable, independent));
		}
		return tabPaneSuper;
	}
	
	@SuppressWarnings("unchecked")
	public <E extends PropertableElement, P extends Property<?>> JComponent initialiseTabPane(E element, P property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException{
		List<List<List<Object>>> lines = new ArrayList<List<List<Object>>>();
		// get the header and data
		List<List<JComponent>> headerRow = getHeaderRows(element);
		List<List<JComponent>> dataRows = getDataRow(element);
		List<List<JComponent>> footerRow = getFooterRows(element);
		// now we can determine the width of the thing
		int width = Math.max(3, dataRows.size());
		for(Collection<JComponent> comps: headerRow){
			width = Math.max(width, comps.size());
		}
		for(Collection<JComponent> comps: dataRows){
			width = Math.max(width, comps.size() + 1);
		}
		for(Collection<JComponent> comps: footerRow){
			width = Math.max(width, comps.size());
		}
		// we can also determine the minimal height
		int height = headerRow.size();
		SlickerTabbedPane tabPane = SlickerFactory.instance().createTabbedPane(element.getName());
		double[][] layout = new double[2][Math.max(width, height)];
		for(int i = 0; i < width; i++){
			layout[0][i] = 100;
		}
		for(int i = 0; i < height; i++){
			layout[1][i] = 30;
		}
		tabPane.setLayout(new TableLayout(layout));
		int x = 0;
		int y = 0;
		for(Collection<JComponent> comps: headerRow){
			x = 0;
			for(JComponent comp: comps){
				tabPane.add(comp, "" + x + ", " + y + ", " + x + ", " + y + ", l, c");
				x++;
			}
			y++;
		}
		// we have to do some cleaning
		if(width > height){
			int size = ((TableLayout)tabPane.getLayout()).getNumRow();
			for(int i = 0; i < width - height; i++){
				((TableLayout)tabPane.getLayout()).deleteRow(size - 1);
				size -= 1;
			}
		}
		
		JButton addRow = SlickerFactory.instance().createButton("Add");
		HashMap<DeleteButton, List<Integer>> indices = new HashMap<DeleteButton, List<Integer>>();
		AddButton addButton = new AddButton(element, this, property, independent, tabPane, dataRows, lines, indices, headerRow.size() - 1 + dataRows.size(), editable);
		addRow.addActionListener(addButton);
		for(Collection<JComponent> dataRow: dataRows){
			x = 0;
			int size = ((TableLayout)tabPane.getLayout()).getNumRow();
			y = size;
			((TableLayout)tabPane.getLayout()).insertRow(size, 30);
			for(JComponent comp: dataRow){
				tabPane.add(comp, "" + x + ", " + size + ", " + x + ", " + size + ", l, c");
				x++;
			}
		}
		tabPane.add(addRow, "" + x + ", " + y + ", " + x + ", " + y + ", l, c");
		fillGUI(property, element, addButton, dataRows, independent);
		addRow.setEnabled(editable);
		if(elementLines != null){
			((HashMap<E, List<List<List<Object>>>>)elementLines).put(element, lines);
		}
		JScrollPane scroll = new JScrollPane(tabPane);
		return scroll;
	}
}
