package org.processmining.plugins.properties.processmodel.gui.abstractguiproperty;

import info.clearthought.layout.TableLayout;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.impl.GUIHelperFunction;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
public abstract class GUIPropertySinglePage<K> implements GUIProperty<K> {
	
	public <E extends PropertableElement, P extends Property<?>> void initialiseGUIProperty(UIPluginContext context, E model, Collection<E> elements, P property, HashMap<E, Object> mapping, boolean independent) throws InstantiationException, IllegalAccessException{
		// here we just want to have the individual elements below eachother, so header, data, footer, header, data, footer, etc
		HashMap<E, List<List<JComponent>>> elementDataRows = new HashMap<E, List<List<JComponent>>>();
		JComponent tabPane = initialiseTabPane(elements, property, elementDataRows, true, independent);
		context.showConfiguration(property.getName() + " initialisation", tabPane);
		for(E element: elements){
			List<List<List<Object>>> lines = new ArrayList<List<List<Object>>>();
			List<List<Object>> lineGroup = new ArrayList<List<Object>>();
			for(List<JComponent> components: elementDataRows.get(element)){
				List<Object> line = new ArrayList<Object>();
				for(JComponent comp: components){
					line.add(GUIHelperFunction.getValue(comp));
				}
				lineGroup.add(line);
			}
			lines.add(lineGroup);
			if(independent){
				element.setIndependentProperty(property, parseComponents(property, lines));
			}
			else{
				element.setDependentProperty(property, parseComponents(property, lines));
			}
			if(mapping != null){
				mapping.put(element, parseComponents(property, lines));
			}
		}
	}
	
	public <E extends PropertableElement> List<List<JComponent>> getHeaderRows(E element) {
		ArrayList<List<JComponent>> header = new ArrayList<List<JComponent>>();
		JLabel lname = SlickerFactory.instance().createLabel(element.getName());
		ArrayList<JComponent> head = new ArrayList<JComponent>();
		head.add(lname);
		header.add(head);
		return header;
	}
	
	public <E extends PropertableElement> List<List<JComponent>> getFooterRows(E element) {
		return new ArrayList<List<JComponent>>();
	}

	@SuppressWarnings("unchecked")
	public <E extends PropertableElement, P extends Property<?>> JComponent initialiseTabPane(Collection<E> elements, P property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException{
		// here we just want to have the individual elements below eachother, so header, data, footer, header, data, footer, etc
		if(elementLines == null){
			// we need to have this as a placeholder
			elementLines = new HashMap<E, List<List<JComponent>>>();
		}
		int width = 3;
		int height = 0;
		for(E element: elements){
			// get the header and data
			List<List<JComponent>> headerRow = getHeaderRows(element);
			List<List<JComponent>> dataRows = getDataRow(element);
			List<List<JComponent>> footerRow = getFooterRows(element);
			((HashMap<E, List<List<JComponent>>>)elementLines).put(element, dataRows);
			for(Collection<JComponent> comps: headerRow){
				width = Math.max(width, comps.size());
			}
			for(Collection<JComponent> comps: dataRows){
				width = Math.max(width, comps.size() + 1);
			}
			for(Collection<JComponent> comps: footerRow){
				width = Math.max(width, comps.size());
			}
			height += headerRow.size();
			height += dataRows.size();
			height += footerRow.size();
		}
		JPanel panel = new JPanel();
		double[][] layout = new double[2][Math.max(width, height)];
		for(int i = 0; i < width; i++){
			layout[0][i] = 100;
		}
		for(int i = 0; i < height; i++){
			layout[1][i] = 30;
		}
		panel.setLayout(new TableLayout(layout));
		int x = 0;
		int y = 0;
		for(E element: elements){
			List<List<JComponent>> headerRow = getHeaderRows(element);
			for(Collection<JComponent> comps: headerRow){
				x = 0;
				for(JComponent comp: comps){
					panel.add(comp, "" + x + ", " + y + ", " + x + ", " + y + ", l, c");
					x++;
				}
				y++;
			}
			// we have to do some cleaning
			if(width > height){
				int size = ((TableLayout)panel.getLayout()).getNumRow();
				for(int i = 0; i < height - width; i++){
					((TableLayout)panel.getLayout()).deleteRow(size - 1);
					size -= 1;
				}
			}
			// of course we need to add a single dataRow and the button to the right of it
			List<List<JComponent>> dataRows = ((HashMap<E, List<List<JComponent>>>)elementLines).get(element);
			for(Collection<JComponent> dataRow: dataRows){
				x = 0;
				for(JComponent comp: dataRow){
					comp.addKeyListener(new SinglePageElementKeyListener(element, this, property, independent, dataRows));
					comp.addMouseListener(new SinglePageElementMouseListener(element, this, property, independent, dataRows));
					panel.add(comp, "" + x + ", " + y + ", " + x + ", " + y + ", l, c");
					x++;
				}
				y++;
			}
			fillGUI(property, element, null, dataRows, independent);
		}
		return panel;
	}
	
	public <E extends PropertableElement, P extends Property<?>> JComponent initialiseTabPane(E element, P property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException{
		List<E> elements = new ArrayList<E>();
		elements.add(element);
		return initialiseTabPane(elements, property, elementLines, editable, independent);
	}
	
	@SuppressWarnings("unchecked")
	public <E extends PropertableElement, P extends Property<?>> void fillGUI(P prop, E element, AddButton button, List<List<JComponent>> dataRows, boolean independent) throws InstantiationException, IllegalAccessException{
		if(independent){
			if(element.getIndependentProperty(prop) != null){
				((JTextField)dataRows.get(0).get(0)).setText(((K)element.getIndependentProperty(prop)).toString());
			}
		}
		else{
			if(element.getDependentProperty(prop) != null){
				((JTextField)dataRows.get(0).get(0)).setText(((K)element.getDependentProperty(prop)).toString());
			}
		}
	}
	
	public <E extends PropertableElement> List<List<JComponent>> getDataRow(E element) throws InstantiationException, IllegalAccessException{
		ArrayList<List<JComponent>> dataRows = new ArrayList<List<JComponent>>();
		ArrayList<JComponent> dataRow = new ArrayList<JComponent>();
		dataRow.add(new JTextField(60));
		dataRows.add(dataRow);
		return dataRows;
	}
}
