package org.processmining.plugins.properties.processmodel.gui.impl;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurableProperty;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.AddButton;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.GUIPropertyListOfElements;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
public class GUIConfigurablePropertyList<K> extends GUIPropertyListOfElements<Pair<List<K>, List<K>>>{

	ConfigurableProperty<K> confProp;
	
	@SuppressWarnings("unchecked")
	public <E extends PropertableElement, P extends Property<?>> JComponent initialiseTabPane(E element, P property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException{
		confProp = (ConfigurableProperty<K>) property;
		return super.initialiseTabPane(element, property, elementLines, editable, independent);
	}
	
	@SuppressWarnings("unchecked")
	public <E extends PropertableElement, P extends Property<?>> void fillGUI(P prop, E element, AddButton button,
			List<List<JComponent>> dataRows, boolean independent) throws InstantiationException, IllegalAccessException {
		try{
			// we have to fill the element with each value in the list
			Pair<List<K>, List<K>> values = (Pair<List<K>, List<K>>) element.getIndependentProperty(prop);
			if(values == null || values.getFirst() == null || values.getSecond() == null){
				return;
			}
			Property<?> nonConfProp = confProp.getNonConfigurableVariant().newInstance();
			// we have to store this value from E
			Object valueProp = null;
			if(independent){
				valueProp = element.getIndependentProperty(nonConfProp);
			}
			else{
				valueProp = element.getDependentProperty(nonConfProp);
			}
			for(K elem: values.getFirst()){
				boolean configurable = false;
				if(values.getSecond().contains(elem)){
					configurable = true;
				}
				Pair<List<K>, List<K>> skeletonStructure = new Pair<List<K>, List<K>>(new ArrayList<K>(), new ArrayList<K>());
				skeletonStructure.getFirst().add(elem);
				Object rawValue = confProp.makeNonConfigurable(skeletonStructure); 

				// now we can parse it, add it to E
				if(independent){
					element.setIndependentProperty(nonConfProp, rawValue);
				}
				else{
					element.setDependentProperty(nonConfProp, rawValue);
				}
				// now pre-process the data rows
				List<JComponent> comps = dataRows.get(dataRows.size() - 1);
				((JCheckBox)comps.get(comps.size() - 1)).setSelected(configurable);
				button.setEnabled(false);
				nonConfProp.getGUIProperty().fillGUI(nonConfProp, element, button, dataRows, independent);
				button.setEnabled(true);
				button.actionPerformed(null);
			}
			// now set the values of E back if it exists
			if(independent){
				if(valueProp != null){
					element.setIndependentProperty(nonConfProp, valueProp);
				}
				else{
					element.removeIndependentProperty(nonConfProp);
				}
			}
			else{
				if(valueProp != null){
					element.setDependentProperty(nonConfProp, valueProp);
				}
				else{
					element.removeDependentProperty(nonConfProp);
				}
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	public <E extends PropertableElement> List<List<JComponent>> getHeaderRows(E element) {
		try{
			Property<?> prop = confProp.getNonConfigurableVariant().newInstance();
			GUIProperty<?> guiProp = prop.getGUIProperty();
			List<List<JComponent>> headerRows = guiProp.getHeaderRows(element);
			List<JComponent> comps = headerRows.get(headerRows.size() - 1);
			comps.add(SlickerFactory.instance().createLabel("Removable"));
			return headerRows;
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return new ArrayList<List<JComponent>>();
	}
	
	public <E extends PropertableElement> List<List<JComponent>> getDataRow(E element) throws InstantiationException,
			IllegalAccessException {
		try{
			Property<?> prop = confProp.getNonConfigurableVariant().newInstance();
			GUIProperty<?> guiProp = prop.getGUIProperty();
			List<List<JComponent>> dataRow = guiProp.getDataRow(element);
			List<JComponent> comps = dataRow.get(dataRow.size() - 1);
			comps.add(SlickerFactory.instance().createCheckBox("", false));
			return dataRow;
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return new ArrayList<List<JComponent>>();
	}
	
	@SuppressWarnings("unchecked")
	public <P extends Property<?>> Pair<List<K>, List<K>> parseComponents(P property, List<List<List<Object>>> lines)
			throws InstantiationException, IllegalAccessException {
		try{
			Property<?> prop = confProp.getNonConfigurableVariant().newInstance();
			GUIProperty<?> guiProp = prop.getGUIProperty();
			List<K> values = new ArrayList<>();
			List<K> removable = new ArrayList<>();
			for(List<List<Object>> line: lines){
				List<List<List<Object>>> linesp = new ArrayList<>();
				linesp.add(line);
				Object rawValue = guiProp.parseComponents(prop, linesp);
				Pair<List<K>, List<K>> value = (Pair<List<K>, List<K>>) confProp.makeConfigurable(rawValue);
				values.addAll(value.getFirst());
				// now check if it should be removable
				List<Object> lineWithCheckbox = line.get(line.size() - 1);
				if((Boolean)lineWithCheckbox.get(lineWithCheckbox.size() - 1)){
					removable.addAll(value.getFirst());
				}
			}
			return new Pair<>(values, removable);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return new Pair<>(null, null);
	}
	
	

}
