package org.processmining.plugins.properties.processmodel.gui.impl;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

import org.processmining.plugins.properties.enumerations.DistributionType;
import org.processmining.plugins.properties.enumerations.TimeOptions;
import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.AddButton;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.GUIPropertySinglePage;
import org.processmining.plugins.properties.processmodel.impl.ArrivalProcess;
import org.processmining.plugins.properties.processmodel.impl.Distribution;
import org.processmining.plugins.properties.processmodel.impl.EventDistribution;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author DSchunse
 *
 */
public class GUIPropertyDistribution extends GUIPropertySinglePage<Distribution>{

	public <E extends PropertableElement, P extends Property<?>> void fillGUI(P prop, E element, AddButton button, List<List<JComponent>> dataRows, boolean independent) throws InstantiationException, IllegalAccessException{
		Distribution distr = (Distribution) element.getIndependentProperty(prop);
		if(distr != null){
			((JComboBox)dataRows.get(0).get(0)).setSelectedItem(distr.distributionType);
			((JComboBox)dataRows.get(0).get(1)).setSelectedItem(distr.getTimeGranularity());
			// we need to parse for null pointers
			((JTextField)dataRows.get(0).get(2)).setText(distr.mean == null ? "" : distr.mean.toString());
			((JTextField)dataRows.get(0).get(3)).setText(distr.variance == null ? "" : distr.variance.toString());
			((JTextField)dataRows.get(0).get(4)).setText(distr.minimum == null ? "" : distr.minimum.toString());
			((JTextField)dataRows.get(0).get(5)).setText(distr.maximum == null ? "" : distr.maximum.toString());
			((JTextField)dataRows.get(0).get(6)).setText(distr.number == null ? "" : distr.number.toString());
		}
	}

	public <E extends PropertableElement> List<List<JComponent>> getHeaderRows(E element){
		ArrayList<List<JComponent>> header = new ArrayList<List<JComponent>>();
		JLabel lact = SlickerFactory.instance().createLabel("Element: ");
		JLabel lname = SlickerFactory.instance().createLabel(element.getName());
		ArrayList<JComponent> head = new ArrayList<JComponent>();
		head.add(lact);
		head.add(lname);
		header.add(head);
		head = new ArrayList<JComponent>();
		head.add(SlickerFactory.instance().createLabel("Distribution"));
		head.add(SlickerFactory.instance().createLabel("Time Granularity"));
		head.add(SlickerFactory.instance().createLabel("Mean"));
		head.add(SlickerFactory.instance().createLabel("Variance"));
		head.add(SlickerFactory.instance().createLabel("Minimum"));
		head.add(SlickerFactory.instance().createLabel("Maximum"));
		head.add(SlickerFactory.instance().createLabel("Number"));
		header.add(head);
		return header;
	}
	
	public <E extends PropertableElement> List<List<JComponent>> getDataRow(E element) throws InstantiationException, IllegalAccessException{
		ArrayList<List<JComponent>> dataRows = new ArrayList<List<JComponent>>();
		ArrayList<JComponent> data = new ArrayList<JComponent>();
		data.add(SlickerFactory.instance().createComboBox(DistributionType.values()));
		data.add(SlickerFactory.instance().createComboBox(TimeOptions.values()));
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(new JTextField(60));

		dataRows.add(data);
		return dataRows;
	}

	public <P extends Property<?>> Distribution parseComponents(P property, List<List<List<Object>>> lines) throws InstantiationException, IllegalAccessException{
		Distribution distribution = new Distribution();
		distribution.distributionType = (DistributionType)lines.get(0).get(0).get(0);
		distribution.timeGranularity = (TimeOptions)lines.get(0).get(0).get(1);
		distribution.mean = ((String)lines.get(0).get(0).get(2)).isEmpty() ? null : Double.valueOf((String)lines.get(0).get(0).get(2));
		distribution.variance = ((String)lines.get(0).get(0).get(3)).isEmpty() ? null : Double.valueOf((String)lines.get(0).get(0).get(3));
		distribution.minimum = ((String)lines.get(0).get(0).get(4)).isEmpty() ? null : Double.valueOf((String)lines.get(0).get(0).get(4));
		distribution.maximum = ((String)lines.get(0).get(0).get(5)).isEmpty() ? null : Double.valueOf((String)lines.get(0).get(0).get(5));
		distribution.number = ((String)lines.get(0).get(0).get(6)).isEmpty() ? null : Long.valueOf((String)lines.get(0).get(0).get(6));
		if(property.equals(ArrivalProcess.class)){
			return new ArrivalProcess(distribution);
		}
		if(property.equals(EventDistribution.class)){
			return new EventDistribution(distribution);
		}
		return distribution;
	}

}
