package org.processmining.plugins.properties.processmodel.gui.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.enumerations.DistributionType;
import org.processmining.plugins.properties.processmodel.ManualTask;
import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.AddButton;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.GUIPropertyListOfElements;
import org.processmining.plugins.properties.processmodel.impl.Distribution;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * 
 * @author DSchunse
 *
 */
public class GUIPropertyYerkesDodsonInterval extends GUIPropertyListOfElements<HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>> {

	@SuppressWarnings("unchecked")
	public <E extends PropertableElement, P extends Property<?>> void fillGUI(P prop, E element, AddButton button, List<List<JComponent>> dataRows, boolean independent) throws InstantiationException, IllegalAccessException{
		HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> property = null;
		if(independent){
			if(element.getIndependentProperty(prop) != null){
				property = (HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>)element.getIndependentProperty(prop);
			}
		}
		else{
			if(element.getDependentProperty(prop) != null){
				property = (HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>)element.getDependentProperty(prop);
			}
		}
		if(property != null){
			for(UUID id: property.keySet()){
				// get the originator from the manual task
				PropertableElement orig = null;
				for(PropertableElement org: ((ManualTask)element).getOriginators()){
					if(org.getID().equals(id)){
						orig = org;
					}
				}
				// now make the row
				if(orig != null){
					for(Pair<Double, Double> key: property.get(id).keySet()){
						((JComboBox)dataRows.get(0).get(0)).setSelectedItem(orig);
						((JTextField)dataRows.get(0).get(1)).setText(key.getFirst().toString());
						((JTextField)dataRows.get(0).get(2)).setText(key.getSecond().toString());
						Distribution distr = property.get(id).get(key);
						((JComboBox)dataRows.get(0).get(3)).setSelectedItem(distr.distributionType);
						// we need to parse for null pointers
						((JTextField)dataRows.get(0).get(4)).setText(distr.mean == null ? "" : distr.mean.toString());
						((JTextField)dataRows.get(0).get(5)).setText(distr.variance == null ? "" : distr.variance.toString());
						((JTextField)dataRows.get(0).get(6)).setText(distr.minimum == null ? "" : distr.minimum.toString());
						((JTextField)dataRows.get(0).get(7)).setText(distr.maximum == null ? "" : distr.maximum.toString());
						((JTextField)dataRows.get(0).get(8)).setText(distr.number == null ? "" : distr.number.toString());
						button.actionPerformed(null);
					}
				}
			}
		}
	}

	public <E extends PropertableElement> List<List<JComponent>> getHeaderRows(E element){
		ArrayList<List<JComponent>> header = new ArrayList<List<JComponent>>();
		JLabel lact = SlickerFactory.instance().createLabel("Activity: ");
		JLabel lname = SlickerFactory.instance().createLabel(element.getName());
		ArrayList<JComponent> head = new ArrayList<JComponent>();
		head.add(lact);
		head.add(lname);
		header.add(head);
		head = new ArrayList<JComponent>();
		head.add(SlickerFactory.instance().createLabel("Originator"));
		head.add(SlickerFactory.instance().createLabel("Lower"));
		head.add(SlickerFactory.instance().createLabel("Upper"));
		head.add(SlickerFactory.instance().createLabel("Distribution"));
		head.add(SlickerFactory.instance().createLabel("Mean"));
		head.add(SlickerFactory.instance().createLabel("Variance"));
		head.add(SlickerFactory.instance().createLabel("Minimum"));
		head.add(SlickerFactory.instance().createLabel("Maximum"));
		head.add(SlickerFactory.instance().createLabel("Number"));
		header.add(head);
		return header;
	}

	public <E extends PropertableElement> List<List<JComponent>> getDataRow(E element) throws InstantiationException, IllegalAccessException{
		ArrayList<List<JComponent>> dataRows = new ArrayList<List<JComponent>>();
		ArrayList<JComponent> data = new ArrayList<JComponent>();
		if(!((ManualTask)element).getOriginators().isEmpty()){
			data.add(SlickerFactory.instance().createComboBox(((ManualTask)element).getOriginators().toArray()));
		}
		else{
			data.add(SlickerFactory.instance().createComboBox(new String[]{}));
		}
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(SlickerFactory.instance().createComboBox(DistributionType.values()));
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(new JTextField(60));
		data.add(new JTextField(60));

		dataRows.add(data);
		return dataRows;
	}

	public <P extends Property<?>> HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> parseComponents(P property, List<List<List<Object>>> lines) throws InstantiationException, IllegalAccessException{
		HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> yerkdod = new HashMap<UUID, HashMap<Pair<Double,Double>,Distribution>>();
		for(List<List<Object>> lineGroup: lines){
			for(List<Object> line: lineGroup){
				// we are now at the right level
				UUID id = ((PropertableElement)line.get(0)).getID();
				Double low = Double.valueOf((String)line.get(1));
				Double high = Double.valueOf((String)line.get(2));
				Distribution distribution = new Distribution();
				distribution.distributionType = (DistributionType)line.get(3);
				distribution.mean = ((String)line.get(4)).isEmpty() ? null : Double.valueOf((String)line.get(4));
				distribution.variance = ((String)line.get(5)).isEmpty() ? null : Double.valueOf((String)line.get(5));
				distribution.minimum = ((String)line.get(6)).isEmpty() ? null : Double.valueOf((String)line.get(6));
				distribution.maximum = ((String)line.get(7)).isEmpty() ? null : Double.valueOf((String)line.get(7));
				distribution.number = ((String)line.get(8)).isEmpty() ? null : Long.valueOf((String)line.get(8));
				HashMap<Pair<Double, Double>, Distribution> element = yerkdod.get(id);
				if(element == null){
					element = new HashMap<Pair<Double,Double>, Distribution>();
					yerkdod.put(id, element);
				}
				element.put(new Pair<Double, Double>(low, high), distribution);
			}
		}
		return yerkdod;
	}

}
