package org.processmining.plugins.properties.processmodel.impl;

import java.util.Set;

import javax.swing.ImageIcon;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.auxiliary.HelperFunctions;
import org.processmining.plugins.properties.enumerations.Constructs;
import org.processmining.plugins.properties.enumerations.DistributionType;
import org.processmining.plugins.properties.enumerations.TimeOptions;
import org.processmining.plugins.properties.processmodel.TimeValue;
import org.processmining.plugins.properties.processmodel.abstractproperty.PropertyDistribution;
import org.processmining.plugins.properties.processmodel.abstractproperty.TimeValueImpl;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.impl.GUIPropertyDistribution;

/**
 * 
 * @author DSchunse
 *
 */
@SuppressWarnings("serial")
public class Distribution extends PropertyDistribution{
	
	public DistributionType distributionType;
	public TimeOptions timeGranularity;
	public Double mean;
	public Double variance;
	public Double minimum;
	public Double maximum;
	public Long number;
	public Double shape;
	public Double scale;
	public Double probability;
	public Long trials;
	public Double mode;
	public Double scaleFactor;
	public Double translationConstant;
	
	public Distribution(){
	}
	
	public Distribution(Distribution distr){
		this.distributionType = distr.distributionType;
		this.timeGranularity = distr.timeGranularity;
		this.mean = distr.mean != null ? new Double(distr.mean) : null;
		this.variance = distr.variance != null ? new Double(distr.variance) : null;
		this.minimum = distr.minimum != null ? new Double(distr.minimum) : null;
		this.maximum = distr.maximum != null ? new Double(distr.maximum) : null;
		this.number = distr.number != null ? new Long(distr.number) : null;
		this.shape = distr.shape != null ? new Double(distr.shape) : null;
		this.scale = distr.scale != null ? new Double(distr.scale) : null;
		this.probability = distr.probability != null ? new Double(distr.probability) : null;
		this.trials = distr.trials != null ? new Long(distr.trials) : null;
		this.mode = distr.mode != null ? new Double(distr.mode) : null;
		this.scaleFactor = distr.scaleFactor != null ? new Double(distr.scaleFactor) : null;
		this.translationConstant = distr.translationConstant != null ? new Double(distr.translationConstant) : null;
	}
	
	public Distribution(DistributionType type, Double mean, Double variance, Double minimum, Double maximum, Double number){
		this(type, TimeOptions.millisecond, mean, variance, minimum, maximum, number);
	}
	
	public Distribution(String value){
		this.setDistributionType(DistributionType.valueOf(value.split("distributionType=\"")[1].split("\"")[0]));
		this.setTimeGranularity(TimeOptions.valueOf(value.split("timeGranularity=\"")[1].split("\"")[0]));
		if(!value.split("mean=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setMean(Double.valueOf(value.split("mean=\"")[1].split("\"")[0]));
		}
		if(!value.split("variance=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setVariance(Double.valueOf(value.split("variance=\"")[1].split("\"")[0]));
		}
		if(!value.split("minimum=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setMinimum(Double.valueOf(value.split("minimum=\"")[1].split("\"")[0]));
		}
		if(!value.split("maximum=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setMaximum(Double.valueOf(value.split("maximum=\"")[1].split("\"")[0]));
		}
		if(!value.split("number=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setNumber(Long.valueOf(value.split("number=\"")[1].split("\"")[0]));
		}
		if(!value.split("shape=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setShape(Double.valueOf(value.split("shape=\"")[1].split("\"")[0]));
		}
		if(!value.split("scale=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setScale(Double.valueOf(value.split("scale=\"")[1].split("\"")[0]));
		}
		if(!value.split("probability=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setProbability(Double.valueOf(value.split("probability=\"")[1].split("\"")[0]));
		}
		if(!value.split("trials=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setTrials(Long.valueOf(value.split("trials=\"")[1].split("\"")[0]));
		}
		if(!value.split("mode=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setMode(Double.valueOf(value.split("mode=\"")[1].split("\"")[0]));
		}
		if(!value.split("scaleFactor=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setScaleFactor(Double.valueOf(value.split("scaleFactor=\"")[1].split("\"")[0]));
		}
		if(!value.split("translationConstant=\"")[1].split("\"")[0].equalsIgnoreCase("null")){
			this.setTranslationConstant(Double.valueOf(value.split("translationConstant=\"")[1].split("\"")[0]));
		}
	}
	
	public Distribution(DistributionType type, TimeOptions timeOpt, Double mean, Double variance, Double minimum, Double maximum, Double number){
		this.distributionType = type;
		this.timeGranularity = timeOpt;
		this.mean = mean;
		this.variance = variance;
		this.minimum = minimum;
		this.maximum = maximum;
		if(number != null){
			this.number = number.longValue();
		}
	}
		
	public DistributionType getDistributionType() {
		return distributionType;
	}

	public void setDistributionType(DistributionType distributionType) {
		this.distributionType = distributionType;
	}

	public TimeOptions getTimeGranularity() {
		return timeGranularity == null ? TimeOptions.millisecond : timeGranularity;
	}

	public void setTimeGranularity(TimeOptions timeGranularity) {
		this.timeGranularity = timeGranularity;
	}

	public Double getMean() {
		return mean;
	}

	public void setMean(Double mean) {
		this.mean = mean;
	}

	public Double getVariance() {
		return variance;
	}

	public void setVariance(Double variance) {
		this.variance = variance;
	}

	public Double getMinimum() {
		return minimum;
	}

	public void setMinimum(Double minimum) {
		this.minimum = minimum;
	}

	public Double getMaximum() {
		return maximum;
	}

	public void setMaximum(Double maximum) {
		this.maximum = maximum;
	}

	public Long getNumber() {
		return number;
	}

	public void setNumber(Long number) {
		this.number = number;
	}

	public Double getShape() {
		return shape;
	}

	public void setShape(Double shape) {
		this.shape = shape;
	}

	public Double getScale() {
		return scale;
	}

	public void setScale(Double scale) {
		this.scale = scale;
	}

	public Double getProbability() {
		return probability;
	}

	public void setProbability(Double probability) {
		this.probability = probability;
	}

	public Long getTrials() {
		return trials;
	}

	public void setTrials(Long trials) {
		this.trials = trials;
	}

	public Double getMode() {
		return mode;
	}

	public void setMode(Double mode) {
		this.mode = mode;
	}

	public Double getScaleFactor() {
		return scaleFactor == null ? 1d : scaleFactor;
	}

	public void setScaleFactor(Double scaleFactor) {
		this.scaleFactor = scaleFactor;
	}

	public Double getTranslationConstant() {
		return translationConstant == null ? 0d : translationConstant;
	}

	public void setTranslationConstant(Double translationConstant) {
		this.translationConstant = translationConstant;
	}
	
	@Override
	public boolean equals(Object o){
		if(o instanceof Distribution){
			return this.distributionValue().equalsIgnoreCase(((Distribution)o).distributionValue());
		}
		else{
			return false;
		}
	}

	public Long getID() {
		return null;
	}

	public String getName() {
		return "Distribution";
	}
	
	public int compare(Object o1, Object o2) {
		if(o1 instanceof Distribution && o2 instanceof Distribution){
			Pair<Double, Double> minMaxO1 = HelperFunctions.minMax((Distribution) o1, 0.025, 0.025);
			Pair<Double, Double> minMaxO2 = HelperFunctions.minMax((Distribution) o2, 0.025, 0.025);
			// now do case distinction on the values
			TimeValue tvO1Lower = new TimeValueImpl(((Distribution)o1).getTimeGranularity(), minMaxO1.getFirst());
			TimeValue tvO1Upper = new TimeValueImpl(((Distribution)o1).getTimeGranularity(), minMaxO1.getSecond());
			TimeValue tvO2Lower = new TimeValueImpl(((Distribution)o2).getTimeGranularity(), minMaxO2.getFirst());
			TimeValue tvO2Upper = new TimeValueImpl(((Distribution)o2).getTimeGranularity(), minMaxO2.getSecond());
			if(tvO1Upper.toHours() < tvO2Lower.toHours()){
				return 1;
			}
			if(tvO1Lower.toHours() > tvO2Upper.toHours()){
				return -1;
			}
			return 0;			
		}
		
		return 0;
	}

	public double getValueDouble(Object value) {
		return ((Distribution)value).mean;
	}

	public ImageIcon getIcon() {
		// fill it in when we have an image
		return null;
	}

	public boolean higherBetter() {
		return false;
	}

	public boolean hasOrdering() {
		return false;
	}
	
	public String distributionValue(){
		return "distributionType=\"" + this.getDistributionType() + "\" " +
				"timeGranularity=\"" + this.getTimeGranularity() + "\" " +
				"mean=\"" + this.getMean() + "\" " +
				"variance=\"" + this.getVariance() + "\" " +
				"minimum=\"" + this.getMinimum() + "\" " +
				"maximum=\"" + this.getMaximum() + "\" " +
				"number=\"" + this.getNumber() + "\" " +
				"shape=\"" + this.getShape() + "\" " +
				"scale=\"" + this.getScale() + "\" " +
				"probability=\"" + this.getProbability() + "\" " +
				"trials=\"" + this.getTrials() + "\" " +
				"mode=\"" + this.getMode() + "\" " +
				"scaleFactor=\"" + this.getScaleFactor() + "\" " +
				"translationConstant=\"" + this.getTranslationConstant() + "\"";
	}

	public Distribution getDefaultValue() {
		Distribution ret = new Distribution();
		ret.distributionType = DistributionType.Constant;
		ret.timeGranularity = TimeOptions.millisecond;
		ret.mean = 0d;
		return ret;
	}

	@Deprecated
	public GUIProperty<Distribution> getGUIProperty() throws InstantiationException, IllegalAccessException {
		return GUIPropertyDistribution.class.newInstance();
	}

	public Set<Constructs> monotonicity() {
		// TODO Auto-generated method stub
		return null;
	}

	public Set<Constructs> getMeaningfulTypeConstructs() {
		// TODO Auto-generated method stub
		return null;
	}
}
