package org.processmining.plugins.properties.processmodel.visualisation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;

import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;

/**
 * 
 * @author DSchunse
 *
 */
public class VisualiseProperties {

	public static <E extends PropertableElement> List<JComponent> visualiseProperties(E element) throws InstantiationException, IllegalAccessException{
		return visualiseProperties(element, false);
	}
	
	public static <E extends PropertableElement> List<JComponent> visualiseProperties(E element, boolean editable) throws InstantiationException, IllegalAccessException{
		List<JComponent> lcomp = new ArrayList<JComponent>();
		if(element != null){			
			if(element.getIndependentProperties() != null){
				// first do the independent properties
				// HACK for the configurable properties
				Set<Property<?>> props = new HashSet<Property<?>>(element.getIndependentProperties().keySet());
				//for(Property<?> prop: element.getIndependentProperties().keySet()){
				for(Property<?> prop: props){
					GUIProperty<?> gprop = prop.getGUIProperty();
					if(gprop != null){
						JComponent component = gprop.initialiseTabPane(element, prop, null, editable, true);
						component.setName("Ind " + prop.getName());
						lcomp.add(component);
					}
					else{
						System.out.println("The GUI property for property " + prop.getName() + " has not (yet) been implemented");
					}
				}
			}
			
			// do the dependent properties
			if(element.getDependentProperties() != null){
				for(Property<?> prop: element.getDependentProperties().keySet()){
					GUIProperty<?> gprop = prop.getGUIProperty();
					if(gprop != null){
						JComponent component = gprop.initialiseTabPane(element, prop, null, editable, false);
						component.setName("Dep " + prop.getName());
						lcomp.add(component);
					}
					else{
						System.out.println("The GUI for property " + prop.getName() + " has not (yet) been implemented");
					}
				}
			}
		}
		return lcomp;
	}
}
