package org.processmining.pnanalysis.models;

import java.util.Collection;

import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.HTMLToString;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.pnanalysis.metrics.PetriNetMetric;
import org.processmining.pnanalysis.metrics.PetriNetMetricManager;

public class PetriNetMetrics implements HTMLToString {

	private Petrinet net;
	private Marking marking;
	private StringBuffer buffer;

	public PetriNetMetrics(PluginContext context, Petrinet net) {
		this(context, net, null);
	}

	public PetriNetMetrics(PluginContext context, Petrinet net, Marking marking) {
		this.net = net;
		this.marking = marking;
		init(context);
		
	}

	public void init(PluginContext context) {
		if (marking == null) {
			Collection<InitialMarkingConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(InitialMarkingConnection.class, context,
						net);
				for (InitialMarkingConnection connection : connections) {
					if (connection.getObjectWithRole(InitialMarkingConnection.NET).equals(net)) {
						marking = connection.getObjectWithRole(InitialMarkingConnection.MARKING);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		if (marking == null) {
			marking = new Marking();
			for (Place place : net.getPlaces()) {
				if (net.getInEdges(place).isEmpty()) {
					marking.add(place);
				}
			}
		}

		buffer = new StringBuffer();
		buffer.append("<h1>Metrics for net " + net.getLabel() + "</h1>");
		for (PetriNetMetric metric : PetriNetMetricManager.getInstance().getMetrics()) {
			buffer.append("<h2>" + metric.getName() + "</h2>");
			buffer.append("<p>" + metric.compute(context, net, marking) + "</p>");
			buffer.append("<h3>Details</h3><pre>" + metric.getLog() + "</pre>");
		}
	}

	public String toHTMLString(boolean includeHTMLTags) {

		StringBuffer buf = new StringBuffer();
		if (includeHTMLTags) {
			buf.append("<html>");
		}
		buf.append(buffer);
		if (includeHTMLTags) {
			buf.append("</html>");
		}
		return buf.toString();
	}

}
