package org.processmining.processtree.configuration.data.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.processmining.processtree.configuration.data.DataConfiguration;

public class DataConfigurationImpl implements DataConfiguration{
	
	Map<UUID, UUID> selectedExpression;
	Map<UUID, Set<UUID>> nonSelectedExpressions;
	Map<UUID, Set<UUID>> removedRead;
	Map<UUID, Set<UUID>> nonRemovedRead;
	Map<UUID, Set<UUID>> removedWritten;
	Map<UUID, Set<UUID>> nonRemovedWritten;

	public DataConfigurationImpl(){
		this.selectedExpression = new HashMap<UUID, UUID>();
		this.nonSelectedExpressions = new HashMap<UUID, Set<UUID>>();
		this.removedRead = new HashMap<UUID, Set<UUID>>();
		this.nonRemovedRead = new HashMap<UUID, Set<UUID>>();
		this.removedWritten = new HashMap<UUID, Set<UUID>>();
		this.nonRemovedWritten = new HashMap<UUID, Set<UUID>>();
	}
	
	public void setSelectedExpression(UUID edge, UUID expression) {
		selectedExpression.put(edge, expression);
	}

	public void addNonSelectedExpression(UUID edge, UUID expression) {
		addElement(edge, expression, nonSelectedExpressions);
	}

	public void addRemovedRead(UUID node, UUID variable) {
		addElement(node, variable, removedRead);
	}

	public void addNonRemovedRead(UUID node, UUID variable) {
		addElement(node, variable, nonRemovedRead);
		
	}

	public void addRemovedWritten(UUID node, UUID variable) {
		addElement(node, variable, removedWritten);
	}

	public void addNonRemovedWritten(UUID node, UUID variable) {
		addElement(node, variable, nonRemovedWritten);
	}

	public UUID getSelectedExpression(UUID edge) {
		return selectedExpression.get(edge);
	}

	public Set<UUID> getNonSelectedExpression(UUID edge) {
		return getElement(edge, nonSelectedExpressions);
	}

	public Set<UUID> getRemovedRead(UUID node) {
		return getElement(node, removedRead);
	}

	public Set<UUID> getNonRemovedRead(UUID node) {
		return getElement(node, nonRemovedRead);
	}

	public Set<UUID> getRemovedWritten(UUID node) {
		return getElement(node, removedWritten);
	}

	public Set<UUID> getNonRemovedWritten(UUID node) {
		return getElement(node, nonRemovedWritten);
	}

	public Map<UUID, UUID> getSelectedExpressions() {
		return selectedExpression;
	}

	public Map<UUID, Set<UUID>> getNonSelectedExpressions() {
		return nonSelectedExpressions;
	}

	public Map<UUID, Set<UUID>> getRemovedRead() {
		return removedRead;
	}

	public Map<UUID, Set<UUID>> getNonRemovedRead() {
		return nonRemovedRead;
	}

	public Map<UUID, Set<UUID>> getRemovedWritten() {
		return removedWritten;
	}

	public Map<UUID, Set<UUID>> getNonRemovedWritten() {
		return nonRemovedWritten;
	}
	
	private void addElement(UUID key, UUID element, Map<UUID, Set<UUID>> datastructure){
		Set<UUID> elements = datastructure.get(key);
		if(elements == null){
			elements = new HashSet<UUID>();
		}
		elements.add(element);
		datastructure.put(key, elements);
	}
	
	private Set<UUID> getElement(UUID key, Map<UUID, Set<UUID>> datastructure){
		if(datastructure.get(key) != null){
			return datastructure.get(key);
		}
		else{
			// lets return an empty structure
			return new HashSet<UUID>();
		}
	}

}
