package org.processmining.processtree.conversion.yawl;

import java.awt.Rectangle;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.Pair;
import org.processmining.processtree.Block;
import org.processmining.processtree.Block.PlaceHolder;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task;
import org.yawlfoundation.yawl.editor.elements.model.EngineIdentifier;
import org.yawlfoundation.yawl.editor.elements.model.VertexContainer;
import org.yawlfoundation.yawl.editor.net.NetGraph;
import org.yawlfoundation.yawl.editor.net.NetGraphModel;
import org.yawlfoundation.yawl.editor.specification.SpecificationModel;
import org.yawlfoundation.yawl.editor.swing.net.YAWLEditorNetPanel;

/**
 * 
 * @author DSchunse
 * @see PTConversion package
 */
@Plugin(
		name = "Convert Process Tree to YAWL model", 
		parameterLabels = { "Process Tree" }, 
		returnLabels = { "YAWL Model" }, 
		returnTypes = { SpecificationModel.class }, 
		userAccessible = true)
@Deprecated
public class ProcessTree2Yawl {
	
	/*
	 * Hold different constructs which YAWL expects, but are not that relevant for us
	 */
	private SpecificationModel specification;
	private YAWLEditorNetPanel panel;
	private NetGraph netGraph;
	private NetGraphModel netGraphModel;
	
	/*
	 * Hold different converters.
	 */
	private YawlActivityConverter activityConverter;
	private Map<ProcessTree.Type, YawlConverter> operatorConverters;
	private YawlProxyConverter proxyConverter;
	private YawlConversionType conversionType = YawlConversionType.SHARED;

	/*
	 * To generate new ids.
	 */
	private int id = 100;

	/*
	 * To store how many parents a node has.
	 */
	//private Map<Node, Integer> parentCountMap;

	//private Stack<Map<Node, Integer>> parentCountMapStack;
	
	/**
	 * Generates a new, fresh, id to be used for Yawl nodes.
	 * 
	 * @return A new and fresh id.
	 */
	public String newId() {
		return "_" + id++ + "";
	}

	public boolean hasMultipleParents(Node node) {
		return node.getIncomingEdges().size() > 1;
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (copied)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertByUnfolding(PluginContext context, ProcessTree tree) throws InstantiationException, IllegalAccessException {
		conversionType = YawlConversionType.PRIVATE;
		return convert(tree, false, null);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (shared, synchronized)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertBySharing(PluginContext context, ProcessTree tree) throws InstantiationException, IllegalAccessException {
		conversionType = YawlConversionType.SHARED;
		return convert(tree, false, null);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (shared, decomposed)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertByDecomposition(PluginContext context, ProcessTree tree) throws InstantiationException, IllegalAccessException {
		conversionType = YawlConversionType.COMPOSITE;
		return convert(tree, false, null);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (shared, synchronized, decomposed)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertByComSharing(PluginContext context, ProcessTree tree) throws InstantiationException, IllegalAccessException {
		conversionType = YawlConversionType.COMSHARED;
		return convert(tree, false, null);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (shared)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertBySimpleSharing(PluginContext context, ProcessTree tree) throws InstantiationException, IllegalAccessException {
		conversionType = YawlConversionType.SIMPLE;
		return convert(tree, false, null);
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (minimal)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertMinimal(PluginContext context, ProcessTree tree) throws InstantiationException, IllegalAccessException {
		conversionType = YawlConversionType.SHARED;
		return convert(tree, true, null);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to Yawl (configurable)")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public SpecificationModel convertConfigurableModel(ProcessTree tree, Map<Edge, Pair<EngineIdentifier, EngineIdentifier>> mappingEdgeIds) throws InstantiationException, IllegalAccessException{
		conversionType = YawlConversionType.COMSHARED;
		SpecificationModel specification = convert(tree, false, mappingEdgeIds);
		return specification;
	}
	
	private SpecificationModel convert(ProcessTree tree, boolean minimal, Map<Edge, Pair<EngineIdentifier, EngineIdentifier>> mappingEdgeIds) throws InstantiationException, IllegalAccessException {
		/*
		 * Check for configurability, if so, we have to use subnets
		 */
		if(tree.isConfigurable() && !minimal){
			//conversionType = YawlConversionType.COMSHARED;
		}
		if(mappingEdgeIds == null){
			mappingEdgeIds = new HashMap<Edge, Pair<EngineIdentifier,EngineIdentifier>>();
		}
		/*
		 * Initialize converters.
		 */
		activityConverter = new YawlActivityConverter(true);
		operatorConverters = new HashMap<ProcessTree.Type, YawlConverter>();
		operatorConverters.put(ProcessTree.Type.AND, new YawlAndConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.DEF, new YawlDefConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.LOOPXOR, new YawlLoopXorConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.LOOPDEF, new YawlLoopDefConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.OR, new YawlOrConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.SEQ, new YawlSeqConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.XOR, new YawlXorConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.PLACEHOLDER, new YawlXorConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.TIMEOUT, new YawlEventConverter(conversionType));
		operatorConverters.put(ProcessTree.Type.MESSAGE, new YawlEventConverter(conversionType));

		proxyConverter = new YawlProxyConverter(conversionType);
		/*
		 * Initialize parent counts.
		 */
		//parentCountMap = buildParentCount(tree);
		//parentCountMapStack = new Stack<Map<Node,Integer>>();
		
		/*
		 * Initialise the specification
		 */
		specification = SpecificationModel.getInstance();
		specification.reset();
		specification.setAuthor("The ProM ProcessTree2YAWL converter");
		specification.setName(tree.getName());
		specification.setId("ID" + tree.getName());
		specification.setDescription("Conversion of Process Tree to YAWL of model: '" + tree.getName() + "'.\n \t\t\t\tFor comments on the conversion, please contact D.M.M. Schunselaar: D.M.M.Schunselaar@tue.nl.\n\t\t\t");
		specification.setDataTypeDefinition(YAWLUtil.createDataTypeDefinition(tree));
		netGraph = new NetGraph();
		panel = new YAWLEditorNetPanel(new Rectangle(2, 25, 1767, 878), netGraph);
		netGraph = panel.getNet();
		netGraph.setName(tree.getName());
		netGraphModel = netGraph.getNetModel();
		/*
		 * Create a decomposition for this top net
		 */
		netGraphModel.setDecomposition(YAWLUtil.createDecomposition(tree));

		/*
		 * Initialize the id counter.
		 */
		id = 100;

		/*
		 * Create an input condition.
		 */
		VertexContainer inputCondition = YAWLUtil.createInputCondition(netGraphModel, "input");

		/*
		 * Create an output condition.
		 */
		VertexContainer outputCondition = YAWLUtil.createOutputCondition(netGraphModel, "output");

		/*
		 * Create a Yawl fragment for the root node of the ProcessTree. Returns a pair
		 * containing the alpha task and the omega task.
		 */
		Pair<VertexContainer, VertexContainer> fragment = convert(tree, tree.getRoot(), netGraphModel, outputCondition, this, mappingEdgeIds);

		/*
		 * Connect the input condition to the alpha task.
		 */
		YAWLUtil.createEdge(netGraphModel, inputCondition, fragment.getFirst());

		/*
		 * Connect the omega task to the output condition.
		 */
		YAWLUtil.createEdge(netGraphModel, fragment.getSecond(), outputCondition);
		
		/*
		 * Some standard YAWL specific stuff
		 */
		netGraph.setBounds(0, 0, 1764, 875);
		netGraph.doLayout();
		netGraph.validate();
		
		/*
		 * TODO: for testing
		 */
		//specification.setFileName("D:/Temp/CaseStudyMerge/AutoGenOutput.yawl");
		//SpecificationArchiveHandler.getInstance().saveUpdatingGUI(specification);
		//specification.setFileName("D:/Temp/PT2YAWL/AutoGenOutput.yawl");
		//SpecificationArchiveHandler.getInstance().saveUpdatingGUI(specification);
		
		/*
		 * Return the model.
		 */
		return specification;
	}

	/**
	 * Counts the number of parents of any node.
	 * 
	 * @param dag
	 *            The FlatDAG to count the number of parents for.
	 * @return A map that maps every node to its parent count.
	 */
	@SuppressWarnings("unused")
	private Map<Node, Integer> buildParentCount(ProcessTree tree) {
		Map<Node, Integer> pcMap = new HashMap<Node, Integer>();
		for (Node node : tree.getNodes()) {
			pcMap.put(node, 0);
		}
		for (Node node : tree.getNodes()) {
			if(node instanceof Block){
				for (Node childNode : ((Block)node).getChildren()) {
					if ((conversionType != YawlConversionType.SHARED)
							|| (!(node instanceof Block.And) && !(node instanceof Block.Or))) {
						pcMap.put(childNode, pcMap.get(childNode) + 1);
					}
				}
			}
		}
		return pcMap;
	}

	//public void pushNode(ProcessTree tree, Node node) {
	//	parentCountMapStack.push(parentCountMap);
	//	parentCountMap = buildParentCount(tree, node);
	//}
	
	//public void popNode() {
	//	parentCountMap = parentCountMapStack.pop();
	//}
	
	@SuppressWarnings("unused")
	private Map<Node, Integer> buildParentCount(ProcessTree tree, Node root) {
		Set<Node> nodes = getNodes(tree, root);
		Map<Node, Integer> pcMap = new HashMap<Node, Integer>();
		for (Node node : nodes) {
			pcMap.put(node, 0);
		}
		for (Node node : tree.getNodes()) {
			if (node instanceof Block && !(node instanceof Block.And) && !(node instanceof Block.Or) && nodes.contains(node)) {
				for (Node childNode : ((Block)node).getChildren()) {
					pcMap.put(childNode, pcMap.get(childNode) + 1);
				}
			}
		}
		return pcMap;
	}

	private Set<Node> getNodes(ProcessTree tree, Node node) {
		Set<Node> nodes = new HashSet<Node>();
		getNodes(tree, node, nodes);
		return nodes;
	}
	
	private void getNodes(ProcessTree tree, Node node, Set<Node> nodes) {
		nodes.add(node);
		if(node instanceof Block) {
			if (!(node instanceof Block.And) && !(node instanceof Block.Or)) {
				nodes.add(node);
				for (Node child: ((Block)node).getChildren()) {
					getNodes(tree, child, nodes);
				}
			}
			return;
		}
	}
	
	public Pair<VertexContainer, VertexContainer> convert(ProcessTree tree, Node node, NetGraphModel netGraphModel,
			VertexContainer exitNode, ProcessTree2Yawl pt2y, Map<Edge, Pair<EngineIdentifier, EngineIdentifier>> mappingEdgeIds) {
		YawlConverter converter = null;

		if (node instanceof Task) {
			converter = activityConverter;
		} else if (node instanceof Block && !(node instanceof PlaceHolder)) {
			converter = operatorConverters.get(ProcessTree.Type.getTypeOf(node));
		} else if (node instanceof PlaceHolder) {
			converter = proxyConverter;
		}

		if (converter != null) {
			return converter.convert(tree, node, netGraphModel, exitNode, pt2y, mappingEdgeIds);
		}
		return null;
	}
}
