package org.processmining.processtree.conversion.yawl;

import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.GraphConstants;
import org.processmining.plugins.properties.processmodel.impl.DataValues;
import org.processmining.plugins.properties.processmodel.impl.InitialDataValue;
import org.processmining.processtree.Expression;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Variable;
import org.yawlfoundation.yawl.editor.data.DataVariable;
import org.yawlfoundation.yawl.editor.data.Decomposition;
import org.yawlfoundation.yawl.editor.data.ParameterList;
import org.yawlfoundation.yawl.editor.data.ParameterLists;
import org.yawlfoundation.yawl.editor.elements.model.AtomicTask;
import org.yawlfoundation.yawl.editor.elements.model.CompositeTask;
import org.yawlfoundation.yawl.editor.elements.model.Condition;
import org.yawlfoundation.yawl.editor.elements.model.Decorator;
import org.yawlfoundation.yawl.editor.elements.model.InputCondition;
import org.yawlfoundation.yawl.editor.elements.model.JoinDecorator;
import org.yawlfoundation.yawl.editor.elements.model.OutputCondition;
import org.yawlfoundation.yawl.editor.elements.model.SplitDecorator;
import org.yawlfoundation.yawl.editor.elements.model.VertexContainer;
import org.yawlfoundation.yawl.editor.elements.model.VertexLabel;
import org.yawlfoundation.yawl.editor.elements.model.YAWLFlowRelation;
import org.yawlfoundation.yawl.editor.elements.model.YAWLPort;
import org.yawlfoundation.yawl.editor.elements.model.YAWLTask;
import org.yawlfoundation.yawl.editor.net.NetGraph;
import org.yawlfoundation.yawl.editor.net.NetGraphModel;
import org.yawlfoundation.yawl.editor.swing.net.YAWLEditorNetPanel;

/**
 * This is a wrapperclass to ease the creation of new YAWL constructs based on the Editor version of YAWL
 * @author DSchunse
 *
 */
public class YAWLUtil {
		
	public static VertexContainer createAtomicTask(NetGraphModel netGraphModel, String label){
		return createAtomicTask(netGraphModel, label, false);
	}
	
	@SuppressWarnings("unchecked")
	public static VertexContainer createAtomicTask(NetGraphModel netGraphModel, String label, boolean split){
		label = "lab" + label;
		VertexContainer vertexCont = new VertexContainer();
		AtomicTask task = new AtomicTask(new Point2D.Double(10, 10));
		vertexCont.add(task);
		task.setParent(vertexCont);
		VertexLabel vLab = new VertexLabel(task, label);
		GraphConstants.setBounds(vLab.getAttributes(), new AttributeMap.SerializableRectangle2D(68.0, 132.0, 96.0, 20.0));
		vertexCont.add(vLab);
		vLab.setParent(vertexCont);
		task.setEngineLabel(label);
		netGraphModel.getGraph().addElement(vertexCont.getVertex());
		netGraphModel.getRoots().add(vertexCont);
		return vertexCont;
	}
	
	@SuppressWarnings("unchecked")
	public static VertexContainer createCompositeTask(NetGraphModel netGraphModel, String label){
		label = "lab" + label;
		VertexContainer vertexCont = new VertexContainer();
		CompositeTask task = new CompositeTask(new Point2D.Double(10, 10));
		vertexCont.add(task);
		task.setParent(vertexCont);
		VertexLabel vLab = new VertexLabel(task, label);
		GraphConstants.setBounds(vLab.getAttributes(), new AttributeMap.SerializableRectangle2D(68.0, 132.0, 96.0, 20.0));
		vertexCont.add(vLab);
		vLab.setParent(vertexCont);
		task.setEngineLabel(label);
		netGraphModel.getGraph().addElement(vertexCont.getVertex());
		netGraphModel.getRoots().add(vertexCont);
		return vertexCont;
	}
	
	@SuppressWarnings("unchecked")
	public static VertexContainer createCondition(NetGraphModel netGraphModel, String label){
		label = "lab" + label;
		VertexContainer vertexCont = new VertexContainer();
		Condition cond = new Condition(new Point2D.Double(10, 10));
		vertexCont.add(cond);
		cond.setParent(vertexCont);
		VertexLabel vLab = new VertexLabel(cond, label);
		GraphConstants.setBounds(vLab.getAttributes(), new AttributeMap.SerializableRectangle2D(68.0, 132.0, 96.0, 20.0));
		vertexCont.add(vLab);
		vLab.setParent(vertexCont);
		cond.setEngineLabel(label);
		netGraphModel.getGraph().addElement(vertexCont.getVertex());
		netGraphModel.getRoots().add(vertexCont);
		return vertexCont;
	}
	
	public static VertexContainer createInputCondition(NetGraphModel netGraphModel){
		return createInputCondition(netGraphModel, "input");
	}
	
	@SuppressWarnings("unchecked")
	public static VertexContainer createInputCondition(NetGraphModel netGraphModel, String label){
		label = "lab" + label;
		VertexContainer vertexCont = new VertexContainer();
		InputCondition cond = new InputCondition(new Point2D.Double(10, 10));
		vertexCont.add(cond);
		cond.setParent(vertexCont);
		VertexLabel vLab = new VertexLabel(cond, label);
		GraphConstants.setBounds(vLab.getAttributes(), new AttributeMap.SerializableRectangle2D(68.0, 132.0, 96.0, 20.0));
		vertexCont.add(vLab);
		vLab.setParent(vertexCont);
		cond.setEngineLabel(label);
		netGraphModel.getGraph().addElement(vertexCont.getVertex());
		netGraphModel.getRoots().add(vertexCont);
		return vertexCont;
	}
	
	public static VertexContainer createOutputCondition(NetGraphModel netGraphModel){
		return createOutputCondition(netGraphModel, "output");
	}
	
	@SuppressWarnings("unchecked")
	public static VertexContainer createOutputCondition(NetGraphModel netGraphModel, String label){
		label = "lab" + label;
		VertexContainer vertexCont = new VertexContainer();
		OutputCondition cond = new OutputCondition(new Point2D.Double(10, 10));
		vertexCont.add(cond);
		cond.setParent(vertexCont);
		VertexLabel vLab = new VertexLabel(cond, label);
		GraphConstants.setBounds(vLab.getAttributes(), new AttributeMap.SerializableRectangle2D(68.0, 132.0, 96.0, 20.0));
		vertexCont.add(vLab);
		vLab.setParent(vertexCont);
		cond.setEngineLabel(label);
		netGraphModel.getGraph().addElement(vertexCont.getVertex());
		netGraphModel.getRoots().add(vertexCont);
		return vertexCont;
	}
	
	public static void setJoinDecorator(VertexContainer container, int decorator){
		Decorator join = ((YAWLTask)container.getVertex()).getJoinDecorator();
		if(join == null){
			join = new JoinDecorator((YAWLTask)container.getVertex(), decorator, Decorator.LEFT);
			container.add(join);
			join.setParent(container);
		}
		else{
			join.setType(decorator);
		}
	}
	
	public static void setSplitDecorator(VertexContainer container, int decorator){
		Decorator split = ((YAWLTask)container.getVertex()).getSplitDecorator();
		if(split == null){
			split = new SplitDecorator((YAWLTask)container.getVertex(), decorator, Decorator.RIGHT);
			container.add(split);
			split.setParent(container);
		}
		else{
			split.setType(decorator);
		}
	}
	
	public static YAWLFlowRelation createEdge(NetGraphModel netGraphModel, VertexContainer source, VertexContainer target){
		return createEdge(netGraphModel, source, target, false, false);
	}
	
	public static YAWLFlowRelation createEdge(NetGraphModel netGraphModel, VertexContainer source, VertexContainer target, boolean hideable, boolean blockable){
		return createEdge(netGraphModel, source, target, "true()", hideable, blockable);
	}
	
	public static YAWLFlowRelation createEdge(NetGraphModel netGraphModel, VertexContainer source, VertexContainer target, Expression expression){
		return createEdge(netGraphModel, source, target, expression, false, false);
	}
	
	public static YAWLFlowRelation createEdge(NetGraphModel netGraphModel, VertexContainer source, VertexContainer target, Expression expression, boolean hideable, boolean blockable){
		// first we need to parse the expression
		String expr = "";
		if(expression != null){
			expr = expression.getExpression();
		}
		if(!expr.equals("")){
			// we need to replace all of the variables to their corresponding path
			String varPath = "/" + netGraphModel.getDecomposition().getLabel() + "/";
			String varPathPost = "/text()";
			for(Variable var: expression.getVariables()){
				// lets hope there are no collisions in the names
				expr = expr.replaceAll(var.getName(), varPath + var.getName() + varPathPost);
			}
			// now we can replace the different constructs
			expr = expr.replaceAll("\"", "'").replaceAll("\\|\\|", "or").replaceAll("\\&\\&", "and").replaceAll("\\=\\=", "=");
		}
		else{
			expr = "true()";
		}
		return createEdge(netGraphModel, source, target, expr, hideable, blockable);
	}
	
	public static YAWLFlowRelation createEdge(NetGraphModel netGraphModel, VertexContainer source, VertexContainer target, String predicate, boolean hideable, boolean blockable){
		// we cannot add the ports on the vertexcontainer, so we need to get the vertex from the vertex container
		// TODO: I think there is some logic on the order of ports w.r.t. a node, check this and update
		// TODO: blockable does not work on places, so we just put them on both
		YAWLPort sourcePort;
		YAWLPort targetPort;		
		if(source.getVertex() instanceof YAWLTask && ((YAWLTask)source.getVertex()).getSplitDecorator() != null && ((YAWLTask)source.getVertex()).getSplitDecorator().getPorts()[0] == null){
			sourcePort = new YAWLPort();
			((YAWLTask)source.getVertex()).getSplitDecorator().add(sourcePort);
		}
		else if(source.getVertex() instanceof YAWLTask && ((YAWLTask)source.getVertex()).getSplitDecorator() != null){
			sourcePort = ((YAWLTask)source.getVertex()).getSplitDecorator().getPorts()[0];
		}
		else if(source.getVertex().getPorts()[0] == null){
			sourcePort = new YAWLPort();
			source.getVertex().add(sourcePort);
		}
		else{
			sourcePort = source.getVertex().getPorts()[0];
		}
		if(target.getVertex() instanceof YAWLTask && ((YAWLTask)target.getVertex()).getJoinDecorator() != null && ((YAWLTask)target.getVertex()).getJoinDecorator().getPorts()[0] == null){
			targetPort = new YAWLPort();
			((YAWLTask)target.getVertex()).getJoinDecorator().add(sourcePort);
		}
		else if(target.getVertex() instanceof YAWLTask && ((YAWLTask)target.getVertex()).getJoinDecorator() != null){
			targetPort = ((YAWLTask)target.getVertex()).getJoinDecorator().getPorts()[0];
		}
		else if(target.getVertex().getPorts()[0] == null){
			targetPort = new YAWLPort();
			target.getVertex().add(targetPort);
		}
		else{
			targetPort = target.getVertex().getPorts()[0];
		}
		YAWLFlowRelation flow;// = new YAWLFlowRelation();
		//sourcePort.addEdge(flow);
		//targetPort.addEdge(flow);
		//flow.setSource(sourcePort);
		//flow.setTarget(targetPort);
		//flow.setPredicate(predicate);
		flow = netGraphModel.getGraph().connect(source.getVertex(), target.getVertex());
		//netGraphModel.getRoots().add(flow);
		if(blockable && source.getVertex() instanceof YAWLTask){
			((YAWLTask)source.getVertex()).setConfigurable(true);
			((YAWLTask)source.getVertex()).configureReset();
		}
		if((hideable || blockable) && target.getVertex() instanceof YAWLTask){
			((YAWLTask)target.getVertex()).setConfigurable(true);
			((YAWLTask)target.getVertex()).configureReset();
		}
		return flow;
		
	}
	
	public static NetGraphModel createSubnet(Node node, String label){
		NetGraph netGraph = new NetGraph();
		YAWLEditorNetPanel panel = new YAWLEditorNetPanel(new Rectangle(2, 25, 1767, 878), netGraph);
		netGraph = panel.getNet();
		netGraph.setName(label);
		netGraph.getNetModel().setName(label);
		Decomposition decomp = new Decomposition();
		decomp.setLabel("Decomp" + label);
		Set<Variable> usedVariables = new HashSet<Variable>();
		usedVariables.addAll(node.getReadVariablesRecursive());
		usedVariables.addAll(node.getWrittenVariablesRecursive());
		for(Variable var: usedVariables){
			int usage;
			if(node.getReadVariablesRecursive().contains(var) && node.getWrittenVariablesRecursive().contains(var)){
				usage = DataVariable.USAGE_INPUT_AND_OUTPUT;
			}
			else if(!node.getReadVariablesRecursive().contains(var)){
				usage = DataVariable.USAGE_OUTPUT_ONLY;
			}
			else{
				usage = DataVariable.USAGE_INPUT_ONLY;
			}
			// TODO: hack to make things work nicer
			usage = DataVariable.USAGE_INPUT_AND_OUTPUT;
			DataVariable dataVar = YAWLUtil.createDataVariable(var, usage, false);
			decomp.addVariable(dataVar);
		}
		netGraph.getNetModel().setDecomposition(decomp);
		return netGraph.getNetModel();
	}
	
	public static void setVariables(NetGraphModel netGraphModel, VertexContainer vertexCont, Collection<Variable> input, Collection<Variable> output, boolean task){
		Decomposition decomposition = ((YAWLTask)vertexCont.getVertex()).getDecomposition();
		Set<Variable> usedVariables = new HashSet<Variable>();
		usedVariables.addAll(input);
		usedVariables.addAll(output);
		ParameterList inputParameters = new ParameterList();
		ParameterList outputParameters = new ParameterList();
		for(Variable var: usedVariables){
			int usage;
			int mine;
			if(input.contains(var) && output.contains(var)){
				usage = DataVariable.USAGE_INPUT_AND_OUTPUT;
				mine = 0;
			}
			else if(!input.contains(var)){
				usage = DataVariable.USAGE_OUTPUT_ONLY;
				mine = 1;
			}
			else{
				usage = DataVariable.USAGE_INPUT_ONLY;
				mine = 2;
			}
			if(!task){
				// TODO: dirty hack
				usage = DataVariable.USAGE_INPUT_AND_OUTPUT;
				mine = 0;
			}
			DataVariable dataVar = YAWLUtil.createDataVariable(var, usage, false);
			decomposition.addVariable(dataVar);
			if(mine == 0 || mine == 2){
				// we have to create an input parameter
				inputParameters.addParameterPair(dataVar, "{/" + netGraphModel.getDecomposition().getLabel() + "/" + var.getName() + "/text()}");
				
			}
			if(mine == 0 || mine == 1){
				// we have to create an output parameter
				outputParameters.addParameterPair(netGraphModel.getVariableSet().getVariableWithName(var.getName()), "{/" + decomposition.getLabel() + "/" + var.getName() + "/text()}");
			}
		}
		ParameterLists paramLists = new ParameterLists();
		paramLists.setInputParameters(inputParameters);
		paramLists.setOutputParameters(outputParameters);
		((YAWLTask)vertexCont.getVertex()).setParameterLists(paramLists);
	}
	
	public static void setDecomposition(VertexContainer container, NetGraphModel netGraphModel){
		((YAWLTask)container.getVertex()).setDecomposition(netGraphModel.getDecomposition());
	}
	
	public static Decomposition createDecomposition(ProcessTree tree){
		Decomposition decomp = new Decomposition();
		decomp.setLabel("TreeDecomposition");
		for(Variable var: tree.getVariables()){
			decomp.addVariable(createDataVariable(var, DataVariable.USAGE_LOCAL, true));
		}
		return decomp;
	}
	
	public static DataVariable createDataVariable(Variable var, int usage, boolean initValue){
		try {
			if(var.getIndependentProperty(DataValues.class) != null){
				// we need to do this, else we might not have the proper type set
				// lets get the initial value
				if(initValue){
					String initialVal = (String)var.getIndependentProperty(InitialDataValue.class);
					return new DataVariable(var.getName(), "T" + var.getName(), initialVal, usage);
				}
				else{
					return new DataVariable(var.getName(), "T" + var.getName(), "", usage);
				}
			}
			else{
				return new DataVariable(var.getName(), "string", "", usage);
			}
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return new DataVariable(var.getName(), "string", "", usage);
	}
	
	@SuppressWarnings("unchecked")
	public static String createDataTypeDefinition(ProcessTree tree) throws InstantiationException, IllegalAccessException{
		//lets make enumerations of the data types, and make separate types of each of them
		String XML = "";
		XML += "<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">\n";
		for(Variable var: tree.getVariables()){
			if(var.getIndependentProperty(DataValues.class) != null){
				XML += "\t<xs:simpleType name=\"T" + var.getName() + "\">\n";
				XML += "\t\t<xs:restriction base=\"xs:string\">\n";
				List<String> dataValues = (List<String>) var.getIndependentProperty(DataValues.class);
				for(String value: dataValues){
					XML += "\t\t\t<xs:enumeration value=\"" + value.replace("\"", "") + "\"/>\n";
				}
				XML += "\t\t</xs:restriction>\n";
				XML += "\t</xs:simpleType>\n";
			}
		}
		XML += "</xs:schema>\n";
		return XML;
	}
}
