package org.processmining.processtree.conversion.yawl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.Originator.Role;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task.Manual;
import org.yawlfoundation.yawl.editor.data.WebServiceDecomposition;
import org.yawlfoundation.yawl.editor.elements.model.EngineIdentifier;
import org.yawlfoundation.yawl.editor.elements.model.VertexContainer;
import org.yawlfoundation.yawl.editor.elements.model.YAWLTask;
import org.yawlfoundation.yawl.editor.net.NetGraphModel;
import org.yawlfoundation.yawl.editor.resourcing.ResourceMapping;
import org.yawlfoundation.yawl.editor.resourcing.ResourcingFilter;
import org.yawlfoundation.yawl.editor.resourcing.ResourcingRole;

public class YawlActivityConverter implements YawlConverter {

	public YawlActivityConverter(boolean doShare) {
	}

	public Pair<VertexContainer, VertexContainer> convert(ProcessTree tree, Node node, NetGraphModel netGraphModel,
			VertexContainer exitNode, ProcessTree2Yawl pt2y, Map<Edge, Pair<EngineIdentifier, EngineIdentifier>> mappingEdgeIds) {

		VertexContainer task = YAWLUtil.createAtomicTask(netGraphModel, node.getName());
		WebServiceDecomposition decomp = new WebServiceDecomposition();
		String labelOfDecomposition = "DecompOf" + node.getName();
		decomp.setLabel(labelOfDecomposition);
		/*
		Set<Variable> usedVariables = new HashSet<Variable>();
		usedVariables.addAll(node.getReadVariables());
		usedVariables.addAll(node.getWrittenVariables());
		for(Variable var: usedVariables){
			int usage;
			if(node.getReadVariables().contains(var) && node.getWrittenVariables().contains(var)){
				usage = DataVariable.USAGE_INPUT_AND_OUTPUT;
			}
			else if(!node.getReadVariables().contains(var)){
				usage = DataVariable.USAGE_OUTPUT_ONLY;
			}
			else{
				usage = DataVariable.USAGE_INPUT_ONLY;
			}
			DataVariable dataVar = YAWLUtil.createDataVariable(var, usage);
			decomp.addVariable(dataVar);
		}
		*/
		((YAWLTask)task.getVertex()).setDecomposition(decomp);
		YAWLUtil.setVariables(netGraphModel, task, node.getReadVariablesRecursive(), node.getWrittenVariablesRecursive(), true);
		if(node instanceof Manual){
			// now lets do the resourcing
			ResourceMapping resMap = new ResourceMapping();
			List<ResourcingRole> resources = new ArrayList<ResourcingRole>();
			for(Originator org: ((Manual)node).getOriginators()){
				if(org instanceof Role){
					resources.add(new ResourcingRole("RO-" + org.getID().toString(), org.getName()));
				}
			}
			resMap.setOfferInteractionPoint(ResourceMapping.SYSTEM_INTERACTION_POINT);
			resMap.setBaseRoleDistributionList(resources);
			resMap.setResourcingFilters(new ArrayList<ResourcingFilter>());
			resMap.setAllocateInteractionPoint(ResourceMapping.USER_INTERACTION_POINT);
			resMap.setStartInteractionPoint(ResourceMapping.USER_INTERACTION_POINT);
			((YAWLTask)task.getVertex()).setResourceMapping(resMap);
		}		
		return new Pair<VertexContainer, VertexContainer>(task, task);
	}

}
