package org.processmining.processtree.editor;

import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.editor.editorpanel.EditorPanel;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;
import org.processmining.processtree.editor.propertypanel.PropertyPanel;
import org.processmining.processtree.impl.ProcessTreeImpl;

public class Editor {
	@Plugin(
		name = "Start Process Tree Editor",
		parameterLabels = { },
		returnLabels = { "Process Tree Editor" },
		returnTypes = { EditorObject.class },
		userAccessible = true,
		help = "Start up editor"
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "D.M.M. Schunselaar",
		email = "D.M.M.Schunselaar@tue.nl"
	)
	public EditorObject generateEmptyTree(PluginContext context){
		return new EditorObject(new ProcessTreeImpl("Fill-in name"));
	}
	
	@Plugin(
		name = "Start Process Tree Editor Beginner",
		parameterLabels = { },
		returnLabels = { "Process Tree Editor Beginner" },
		returnTypes = { EditorObjectBeginner.class },
		userAccessible = true,
		help = "Start up editor beginner"
	)
	@UITopiaVariant(
		affiliation = UITopiaVariant.EHV,
		author = "D.M.M. Schunselaar",
		email = "D.M.M.Schunselaar@tue.nl"
	)
	public EditorObjectBeginner generateEmptyTreeBeginner(PluginContext context){
		return new EditorObjectBeginner(new ProcessTreeImpl("Fill-in name"));
	}
	
	@Plugin(name = "XEditor Process Tree", 
			returnLabels = { "Editor" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"Process tree" }, 
			userAccessible = true)
	@Visualizer
	public JComponent edit(UIPluginContext context, final EditorObject input){
		context.getGlobalContext().getResourceManager().getResourceForInstance(input).destroy();
		ProcessTree tree = input.tree;
		return edit(context, tree);
	}
	
	@Plugin(name = "XEditor Process Tree Beginner", 
			returnLabels = { "Editor" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"Process tree" }, 
			userAccessible = true)
	@Visualizer
	public JComponent editBeginner(UIPluginContext context, final EditorObjectBeginner input){
		context.getGlobalContext().getResourceManager().getResourceForInstance(input).destroy();
		ProcessTree tree = input.tree;
		return editBeginner(context, tree);
	}
	
	@Plugin(name = " XEditor Process Tree", 
			returnLabels = { "Editor" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"Process tree" }, 
			userAccessible = true)
	@Visualizer
	public JComponent edit(UIPluginContext context, ProcessTree input){
		ProcessTree tree = new ProcessTreeImpl(input);
		JPanel ret = new JPanel();
		createLayout(context, tree, ret, false);
		JFrame frame = new JFrame();
		frame.getContentPane().add(new JScrollPane(ret));
		return (JComponent) frame.getContentPane();
	}
	
	@Plugin(name = " XEditor Process Tree Beginner", 
			returnLabels = { "Editor" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = {	"Process tree" }, 
			userAccessible = true)
	@Visualizer
	public JComponent editBeginner(UIPluginContext context, ProcessTree input){
		ProcessTree tree = new ProcessTreeImpl(input);
		JPanel ret = new JPanel();
		createLayout(context, tree, ret, true);
		JFrame frame = new JFrame();
		frame.getContentPane().add(new JScrollPane(ret));
		return (JComponent) frame.getContentPane();
	}
	
	private void createLayout(UIPluginContext context, ProcessTree tree, JPanel parent, boolean beginner){
		Rectangle screenRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
		Dimension screenSize = screenRect.getSize();
		int maxWidth = (int) screenSize.getWidth() - 30;
		// scaling due to ProM overhead
		int maxHeight = (int) screenSize.getHeight() - 130;
		Dimension dimGraphPanel = new Dimension((int)(0.7 * maxWidth), (int)(0.7 * maxHeight));
		Dimension dimPropertyPanel = new Dimension((int)(0.7 * maxWidth), (int)(0.3 * maxHeight));
		Dimension dimEditorPanel = new Dimension((int)(0.3 * maxWidth), maxHeight);
		// create the set of folded nodes
		Set<UUID> foldedNodes = new HashSet<>();
		// first make the visualiser for the process tree itself
		GraphPanel graphPanel = new GraphPanel(tree, dimGraphPanel, foldedNodes);
		// add the properties panel
		PropertyPanel propertyPanel = new PropertyPanel(tree, dimPropertyPanel);
		propertyPanel.setGraphPanel(graphPanel);
		// now make a mouse listener to connect all
		GraphMouseListener graphMouseListener = new GraphMouseListener(tree, graphPanel.getGraph(), foldedNodes);
		graphMouseListener.addPropertyPanel(propertyPanel);
		graphMouseListener.addGraphPanel(graphPanel);
		// add the mouse listener to the graph panel
		graphPanel.addMouseListener(graphMouseListener);
		// create the editor panel
		EditorPanel editorPanel = new EditorPanel(context, tree, graphPanel, graphMouseListener, dimEditorPanel, beginner);
		
		graphMouseListener.addEditorPanel(editorPanel);
		graphPanel.setEditorPanel(editorPanel);

		JSplitPane topBottom = new JSplitPane(JSplitPane.VERTICAL_SPLIT, graphPanel.getComponent(), propertyPanel.getComponent());
		//topBottom.setPreferredSize(new Dimension((int)(0.7*maxWidth), maxHeight));
		topBottom.setDividerLocation((int)(0.7 * maxHeight));
		
		//editorPanel.getEditorPanel().setPreferredSize(new Dimension((int)(0.3*maxWidth), maxHeight));
		
		JSplitPane leftRight = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, topBottom, editorPanel.getEditorPanel());
		topBottom.validate();
		leftRight.validate();
		parent.add(leftRight);
		// now do some magic with the size of the stuff
		int widthEditorPanel = editorPanel.getWidth();
		// we still have the maxWidth
		int widthOfTopBottom = (maxWidth - widthEditorPanel);
		topBottom.setPreferredSize(new Dimension(widthOfTopBottom, maxHeight - 2));
		
	}
	
}
