package org.processmining.processtree.editor.editorpanel;

import java.awt.event.ActionEvent;
import java.util.UUID;

import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorActionListenerMoveNode implements EditorActionListener{
	ProcessTree tree;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	boolean isLeft;
	
	public EditorActionListenerMoveNode(ProcessTree tree, GraphMouseListener graphMouseListener, GraphPanel graphPanel, boolean isLeft){
		this.tree = tree;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.isLeft = isLeft;
	}

	public void actionPerformed(ActionEvent e) {
		if(!isEnabled()){
			return;
		}
		UUID selectedID = graphMouseListener.getSelectedID();
		// special case if it is a node and it has multiple parents, unknown what to do so don't, else move around
		if(tree.getElement(selectedID) instanceof Node && ((Node)tree.getElement(selectedID)).getIncomingEdges().size() > 1){
			// skippy
			return;
		}
		Edge toMoveEdge;
		if(tree.getElement(selectedID) instanceof Node){
			toMoveEdge = ((Node)tree.getElement(selectedID)).getIncomingEdges().get(0);
		}
		else if(tree.getElement(selectedID) instanceof Edge){
			toMoveEdge = (Edge) tree.getElement(selectedID);
		}
		else{
			// unknown
			return;
		}
		// now move this edge
		Block parent = toMoveEdge.getSource();
		int indexToMoveEdge = parent.getOutgoingEdges().indexOf(toMoveEdge);
		// first remove it
		parent.removeOutgoingEdge(toMoveEdge);
		// now insert it accordingly
		if(isLeft){
			// add some check it does not go too far
			parent.addOutgoingEdgeAt(toMoveEdge, Math.max(indexToMoveEdge - 1, 0));
		}
		if(!isLeft){
			if(parent.getOutgoingEdges().size() <= indexToMoveEdge + 1){
				// just add it
				parent.addOutgoingEdge(toMoveEdge);
			}
			else{
				parent.addOutgoingEdgeAt(toMoveEdge, indexToMoveEdge + 1);
			}
		}
		graphPanel.redraw();
	}
	
	public boolean isEnabled(){
		UUID selectedID = graphMouseListener.getSelectedID();
		if(selectedID == null){
			return false;
		}
		// special case if it is a node and it has multiple parents, unknown what to do so don't, else move around
		if(tree.getElement(selectedID) instanceof Node && (((Node)tree.getElement(selectedID)).getIncomingEdges().size() > 1 || ((Node)tree.getElement(selectedID)).getIncomingEdges().size() < 1)){
			// skippy
			return false;
		}
		Edge toMoveEdge;
		if(tree.getElement(selectedID) instanceof Node){
			toMoveEdge = ((Node)tree.getElement(selectedID)).getIncomingEdges().get(0);
		}
		else if(tree.getElement(selectedID) instanceof Edge){
			toMoveEdge = (Edge) tree.getElement(selectedID);
		}
		else{
			// unknown
			return false;
		}
		Block parent = toMoveEdge.getSource();
		int indexToMoveEdge = parent.getOutgoingEdges().indexOf(toMoveEdge);
		if(isLeft){
			// add some check it does not go too far
			return !(indexToMoveEdge - 1 < 0);
		}
		else{
			return !(parent.getOutgoingEdges().size() <= indexToMoveEdge + 1);
		}
	}
}
