package org.processmining.processtree.editor.editorpanel;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorPanel {
	UIPluginContext context;
	JComponent editorPanel;
	ProcessTree tree;
	GraphPanel graphPanel;
	GraphMouseListener graphMouseListener;
	Dimension dimEditorPanel;
	Collection<JComponent> frozenComponents;
	Collection<JComponent> components;
	EditorPanelControlFlow editorPanelControlFlow;
	EditorPanelOriginator editorPanelOriginator;
	EditorPanelData editorPanelData;
	EditorPanelProperties editorPanelProperties;
	EditorPanelCommon editorPanelCommon;
	
	// for beginner mode
	EditorPanelProperties editorPanelPropertiesCF;
	EditorPanelProperties editorPanelPropertiesD;
	EditorPanelProperties editorPanelPropertiesO;
	Boolean beginner;
	private int spacing = 15;
	int width = 420;
	
	public EditorPanel(UIPluginContext context, ProcessTree tree, GraphPanel graphPanel, GraphMouseListener graphMouseListener, Dimension dimEditorPanel, Boolean beginner){
		this.context = context;
		this.tree = tree;
		this.graphPanel = graphPanel;
		this.graphMouseListener = graphMouseListener;
		this.dimEditorPanel = dimEditorPanel;
		this.components = new HashSet<>();
		this.width = Math.min(width, (int) dimEditorPanel.getWidth() - 4);
		this.editorPanel = editorPanel();
		this.beginner = beginner;
	}
	
	public JComponent getEditorPanel(){
		if(beginner){
			return editorPanelBeginner();
		}
		return editorPanel;
	}
	
	public int getWidth(){
		return width;
	}
	
	public void mousePressed(){
		if(beginner){
			editorPanelPropertiesCF.update();
			editorPanelPropertiesD.update();
			editorPanelPropertiesO.update();
		}
		editorPanelControlFlow.update();
		editorPanelOriginator.update();
		editorPanelData.update();
		editorPanelProperties.update();
	}
	
	public void freezePanel(){
		frozenComponents = new HashSet<JComponent>();
		for(JComponent component: components){
			component.setEnabled(false);
			frozenComponents.add(component);
		}
	}
	
	public void defrostPanel(){
		for(JComponent component: frozenComponents){
			component.setEnabled(true);
		}
	}
	
	private JComponent editorPanel(){
		// we have the following rows
		editorPanelControlFlow = new EditorPanelControlFlow(tree, width - 4, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent cf = editorPanelControlFlow.createControlFlowPerspective();
		editorPanelData = new EditorPanelData(tree, width - 4, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent d = editorPanelData.createDataPerspective();
		editorPanelOriginator = new EditorPanelOriginator(tree, width - 4, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent o = editorPanelOriginator.createResourcePerspective();
		editorPanelProperties = new EditorPanelProperties(context, tree, width - 4, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent p = editorPanelProperties.createProperties();
		editorPanelCommon = new EditorPanelCommon(context, tree, width - 4, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent c = editorPanelCommon.createCommon();
		
		int columns = 1;
		int rows = 5 + 4;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		layout[1][0] = editorPanelControlFlow.getHeight() + 1;
		layout[1][1] = spacing;
		layout[1][2] = editorPanelData.getHeight() + 1;
		layout[1][3] = spacing;
		layout[1][4] = editorPanelOriginator.getHeight() + 1;
		layout[1][5] = spacing;
		layout[1][6] = editorPanelProperties.getHeight() + 1;
		layout[1][7] = spacing;
		layout[1][8] = editorPanelCommon.getHeight() + 1;
		
		
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new TableLayout(layout));
		//mainPanel.setPreferredSize(new Dimension(dimEditorPanel));
		mainPanel.add(cf, "0, 0, 0, 0, l, c");
		mainPanel.add(d, "0, 2, 0, 2, l, c");
		mainPanel.add(o, "0, 4, 0, 4, l, c");
		mainPanel.add(p, "0, 6, 0, 6, l, c");
		mainPanel.add(c, "0, 8, 0, 8, l, c");
		
		return new JScrollPane(mainPanel);
	}
	
	private JComponent editorPanelBeginner(){
		// we have the following rows
		editorPanelControlFlow = new EditorPanelControlFlow(tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent cf = editorPanelControlFlow.createControlFlowPerspective();
		editorPanelData = new EditorPanelData(tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent d = editorPanelData.createDataPerspective();
		editorPanelOriginator = new EditorPanelOriginator(tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent o = editorPanelOriginator.createResourcePerspective();
		
		editorPanelPropertiesCF = new EditorPanelProperties(context, tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent pCF = editorPanelPropertiesCF.createProperties();
		editorPanelCommon = new EditorPanelCommon(context, tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent cCF = editorPanelCommon.createCommon();
		
		editorPanelPropertiesD = new EditorPanelProperties(context, tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent pD = editorPanelPropertiesD.createProperties();
		JComponent cD = editorPanelCommon.createCommon();
		
		editorPanelPropertiesO = new EditorPanelProperties(context, tree, width - 8, (int) dimEditorPanel.getHeight(), graphMouseListener, graphPanel, components);
		JComponent pO = editorPanelPropertiesO.createProperties();
		JComponent cO = editorPanelCommon.createCommon();
		
		JTabbedPane tabPane = new JTabbedPane();
		
		int columns = 1;
		int rows = 5;
		double[][] layoutCF = new double[2][Math.max(columns, rows)];
		double[][] layoutD = new double[2][Math.max(columns, rows)];
		double[][] layoutO = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layoutCF[0][i] = width - 4;
			layoutD[0][i] = width - 4;
			layoutO[0][i] = width - 4;
		}
		layoutCF[1][0] = editorPanelControlFlow.getHeight() + 1;
		layoutCF[1][1] = spacing;
		layoutCF[1][2] = editorPanelProperties.getHeight() + 1;
		layoutCF[1][3] = spacing;
		layoutCF[1][4] = editorPanelCommon.getHeight() + 1;
		
		layoutD[1][0] = editorPanelData.getHeight() + 1;
		layoutD[1][1] = spacing;
		layoutD[1][2] = editorPanelProperties.getHeight() + 1;
		layoutD[1][3] = spacing;
		layoutD[1][4] = editorPanelCommon.getHeight() + 1;
		
		layoutO[1][0] = editorPanelOriginator.getHeight() + 1;
		layoutO[1][1] = spacing;
		layoutO[1][2] = editorPanelProperties.getHeight() + 1;
		layoutO[1][3] = spacing;
		layoutO[1][4] = editorPanelCommon.getHeight() + 1;
		
		JPanel cfPanel = new JPanel();
		cfPanel.setLayout(new TableLayout(layoutCF));
		cfPanel.add(cf, "0, 0, 0, 0, l, c");
		cfPanel.add(pCF, "0, 2, 0, 2, l, c");
		cfPanel.add(cCF, "0, 4, 0, 4, l, c");
		
		JPanel dPanel = new JPanel();
		dPanel.setLayout(new TableLayout(layoutD));
		dPanel.add(d, "0, 0, 0, 0, l, c");
		dPanel.add(pD, "0, 2, 0, 2, l, c");
		dPanel.add(cD, "0, 4, 0, 4, l, c");
		
		JPanel oPanel = new JPanel();
		oPanel.setLayout(new TableLayout(layoutO));
		oPanel.add(o, "0, 0, 0, 0, l, c");
		oPanel.add(pO, "0, 2, 0, 2, l, c");
		oPanel.add(cO, "0, 4, 0, 4, l, c");
		
		tabPane.addTab("Control-Flow", cfPanel);
		tabPane.addTab("Data", dPanel);
		tabPane.addTab("Originators", oPanel);
				
		return new JScrollPane(tabPane);
		
	}
	
}
