package org.processmining.processtree.editor.editorpanel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.enumerations.Constructs;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Event;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.Task;
import org.processmining.processtree.Variable;

public class EditorPanelAuxiliary {

	@SuppressWarnings("unchecked")
	public static List<Property<?>> getAllowedProperties(PluginContext context, ProcessTreeElement element){
		Set<Property<?>> currentProperties = element.getIndependentProperties().keySet();
		Set<Class<?>> potentialPropertiesUnknownTyped = context.getPluginManager().getKnownClassesAnnotatedWith(PropertableElementProperty.class);
		Set<Property<?>> potentialProperties = new HashSet<>();
		for(Class<?> clazz: potentialPropertiesUnknownTyped){
			try {
				potentialProperties.add(((Class<? extends Property<?>>)clazz).newInstance());
			} catch (InstantiationException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		List<Property<?>> possibleProperties = new ArrayList<Property<?>>();
		// now we have a set of classes
		for(Property<?> prop: potentialProperties){
			Collection<Class<? extends Property<?>>> dependencies = prop.getDependencies();
			boolean possible = true;
			possible &= adheresToDependency(currentProperties, dependencies);
			possible &= isInstance(element, prop.getMeaningfulTypeConstructs());
			if(possible){
				possibleProperties.add(prop);
			}
		}
		possibleProperties.removeAll(currentProperties);
		return possibleProperties;
	}
	
	private static boolean adheresToDependency(Collection<Property<?>> inElement, Collection<Class<? extends Property<?>>> dependencies){
		for(Class<? extends Property<?>> dependency: dependencies){
			try {
				if(!inElement.contains(dependency.newInstance())){
					return false;
				}
			} catch (InstantiationException | IllegalAccessException e) {
				e.printStackTrace();
				return false;
			}
		}
		return true;
	}
	
	private static boolean isInstance(ProcessTreeElement ele, Set<Constructs> constructs){
		Boolean ret = false;
		for(Constructs constr: constructs){
			ret |= isInstance(ele, constr);
		}
		return ret;
	}
	
	private static boolean isInstance(ProcessTreeElement ele, Constructs constr) {
		switch (constr){
			case AutomaticTask:
				return ele instanceof Task.Automatic;
			case DeferredChoice:
				return ele instanceof Block.Def;
			case DeferredChoiceLoop:
				return ele instanceof Block.DefLoop;
			case Edge:
				return ele instanceof Edge;
			case Event:
				return ele instanceof Event;
			case ExclusiveChoice:
				return ele instanceof Block.Xor;
			case ExclusiveChoiceLoop:
				return ele instanceof Block.XorLoop;
			case ManualTask:
				return ele instanceof Task.Manual;
			case Model:
				return ele instanceof ProcessTree;
			case Node:
				return ele instanceof Node;
			case OR:
				return ele instanceof Block.Or;
			case Originator:
				return ele instanceof Originator;
			case Parallelism:
				return ele instanceof Block.And;
			case Sequence:
				return ele instanceof Block.Seq;
			case Variable:
				return ele instanceof Variable;
			default:
				return false;
		}
	}
	
}
