package org.processmining.processtree.editor.editorpanel;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import org.processmining.processtree.Expression;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.Variable;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorPanelData {
	ProcessTree tree;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	JTextField nameField;
	JComboBox<ProcessTreeElement> variableCombo;
	//JComboBox<Variable> expressionVariableCombo;
	JComboBox<ProcessTreeElement> expressionCombo; 
	//JComboBox<String> comparator;
	//JComboBox<String> variableValues;
	//JComboBox<String> operator;
	//JComboBox<String> clauseCombo1;
	
	//JComboBox<String> clauseCombo2;
	Collection<JComponent> components;
	private int rows = 6;
	private int heightOfRows = 25;
	private Collection<JButton> localComponents;
	
//	/private List<String> clauses;
	//private Set<Variable> varsOfClauses;
	
	
	public EditorPanelData(ProcessTree tree, int maxWidth, int maxHeight, GraphMouseListener graphMouseListener, GraphPanel graphPanel, Collection<JComponent> components){
		this.tree = tree;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.components = components;
		this.localComponents = new HashSet<>();
	}
	
	public int getHeight(){
		// return the height in terms of elements
		// added some for the border
		return rows * heightOfRows + 24;
	}
	
	public JComponent createDataPerspective(){
		int columns = 4;
		// scaling for the border
		int width = (maxWidth - 8) / columns;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfRows;
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Data Perspective"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		
		int offset = 0;
		JLabel nodeNameLabel = new JLabel("Name:");
		nodeNameLabel.setPreferredSize(new Dimension(1 * width, heightOfRows));
		nameField = new JTextField();
		nameField.setPreferredSize(new Dimension(3 * width, heightOfRows));
		List<Class<? extends ProcessTreeElement>> validElements = new ArrayList<>();
		validElements.add(Variable.class);
		validElements.add(Expression.class);
		nameField.addKeyListener(new EditorKeyListenerNameField(tree, graphMouseListener, graphPanel, nameField, validElements));
		panel.add(nodeNameLabel, "0, " + offset + ", 0, " + offset + ", l, c");
		panel.add(nameField, "1, " + offset + ", 3, " + offset + ", l, c");
		offset++;
		JLabel varNameLabel = new JLabel("Variable name:");
		JTextField varName = new JTextField();
		varName.setPreferredSize(new Dimension(3 * width, heightOfRows));
		panel.add(varNameLabel, "0, " + offset + ", 0, " + offset + ", l, c");
		panel.add(varName, "1, " + offset + ", 2, " + offset + ", l, c");
		createAddVarButton(panel, varName, offset, 3, width, heightOfRows);
		createSelectionVariables(panel, offset + 1, 0, width, heightOfRows);
		createDeleteButton(panel, offset + 1, 3, width, heightOfRows, Variable.class);
		createExpressionPanel(panel, offset + 2, 0, width, heightOfRows);
		update();
		return panel;
	}
	
	public void update(){
		updateDataCombo();
		checkEnablement();
		updateNameField();
	}
	
	private void updateDataCombo(){
		List<ProcessTreeElement> variables = new ArrayList<ProcessTreeElement>(tree.getVariables());
		//if(variableCombo == null || expressionVariableCombo == null || expressionCombo == null){
		if(variableCombo == null || expressionCombo == null){
			// not initialised yet so skip
			return;
		}
		// first remove the action listeners
		ActionListener[] variableComboListeners = variableCombo.getActionListeners();
		for(ActionListener actListen: variableComboListeners){
			variableCombo.removeActionListener(actListen);
		}
		/*
		ActionListener[] variableExpressionComboListeners = expressionVariableCombo.getActionListeners();
		for(ActionListener actListen: variableExpressionComboListeners){
			expressionVariableCombo.removeActionListener(actListen);
		}
		*/
		ActionListener[] expressionComboListeners = expressionCombo.getActionListeners();
		for(ActionListener actListen: expressionComboListeners){
			expressionCombo.removeActionListener(actListen);
		}
		ProcessTreeElement selectedVariable = (ProcessTreeElement) variableCombo.getSelectedItem();
		ProcessTreeElement selectedExpression = (ProcessTreeElement) expressionCombo.getSelectedItem();
		//Variable selectedExpressionVar = (Variable) expressionVariableCombo.getSelectedItem();
		//String selectedClause1 = (String) clauseCombo1.getSelectedItem();
		//String selectedClause2 = (String) clauseCombo2.getSelectedItem();
		variableCombo.removeAllItems();
		expressionCombo.removeAllItems();
		//expressionVariableCombo.removeAllItems();
		//variableValues.removeAllItems();
		//clauseCombo1.removeAllItems();
		//clauseCombo2.removeAllItems();
		for(ProcessTreeElement var: variables){
			variableCombo.addItem(var);
			//expressionVariableCombo.addItem((Variable)var);
		}
		for(ProcessTreeElement expr: tree.getExpressions()){
			expressionCombo.addItem(expr);
		}
		// now set the selected item
		variableCombo.setSelectedItem(selectedVariable);
		//expressionVariableCombo.setSelectedItem(selectedExpressionVar);
		expressionCombo.setSelectedItem(selectedExpression);
		/*
		Variable var = (Variable) expressionVariableCombo.getSelectedItem();
		try {
			if(var != null && var.getIndependentProperty(DataValues.class) != null){
				// we can provision the combo box
				for(String value: (List<String>)var.getIndependentProperty(DataValues.class)){
					variableValues.addItem(value);
				}
			}
		} catch (IllegalAccessException | InstantiationException e) {
			e.printStackTrace();
		}
		*/
		/*
		for(String clause: clauses){
			clauseCombo1.addItem(clause);
			clauseCombo2.addItem(clause);
		}
		clauseCombo1.setSelectedItem(selectedClause1);
		clauseCombo2.setSelectedItem(selectedClause2);
		*/
		// now add the actionlisteners again
		for(ActionListener actListen: variableComboListeners){
			variableCombo.addActionListener(actListen);
		}
		/*
		for(ActionListener actListen: variableExpressionComboListeners){
			expressionVariableCombo.addActionListener(actListen);
		}
		*/
		for(ActionListener actListen: expressionComboListeners){
			expressionCombo.addActionListener(actListen);
		}
	}
	
	private void checkEnablement(){
		for(JButton component: localComponents){
			for(ActionListener actListen: component.getActionListeners()){
				if(actListen instanceof EditorActionListener){
					component.setEnabled(((EditorActionListener) actListen).isEnabled());
				}
			}
		}
	}
	
	private void updateNameField(){
		UUID selectedID = graphMouseListener.getSelectedID();
		if(tree.getElement(selectedID) != null && (tree.getElement(selectedID) instanceof Variable ||
				tree.getElement(selectedID) instanceof Expression)){
			nameField.setEnabled(true);
			nameField.setText(tree.getElement(selectedID).getName());
		}
		else{
			nameField.setText("");
			nameField.setEnabled(false);
		}
	}
	
	private void createAddVarButton(JPanel panel, JTextField nameField, int row, int column, int width, int height){
		JButton addVariable = new JButton("Add Variable");
		addVariable.setPreferredSize(new Dimension(width, height));
		addVariable.addActionListener(new EditorActionListenerAddVariable(tree, graphMouseListener, graphPanel, nameField));
		panel.add(addVariable, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(addVariable);
		localComponents.add(addVariable);
	}
	
	private void createSelectionVariables(JPanel panel, int row, int column, int width, int height){
		JLabel varLabel = new JLabel("Variables");
		varLabel.setPreferredSize(new Dimension(width, height));
		variableCombo = new JComboBox<ProcessTreeElement>();
		variableCombo.setPreferredSize(new Dimension(2 * width, height));
		updateDataCombo();
		variableCombo.addActionListener(new EditorActionListenerComboBox(variableCombo, graphMouseListener));
		panel.add(varLabel, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		panel.add(variableCombo, "" + (column + 1) + ", " + row + ", " + (column + 2) + ", " + row + ", l, c");
		components.add(variableCombo);
	}
	
	private void createDeleteButton(JPanel panel, int row, int column, int width, int height, Class<? extends ProcessTreeElement> elementType){
		JButton deleteNode = new JButton("Delete");
		deleteNode.setPreferredSize(new Dimension(4 * width, height));
		deleteNode.addActionListener(new EditorActionListenerDeleteElement(tree, elementType, null, graphMouseListener, graphPanel));
		panel.add(deleteNode, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(deleteNode);
		localComponents.add(deleteNode);
	}
	
	private void createExpressionPanel(JPanel panel, int row, int column, int width, int height){
		JLabel expressionLabel = new JLabel("Expression:");
		final JTextField expressionField = new JTextField();
		expressionField.setPreferredSize(new Dimension(3 * width, height));
		
		panel.add(expressionLabel, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		panel.add(expressionField, "" + (column + 1) + ", " + row + ", " + (column + 3) + ", " + row + ", l, c");
		
		JButton addExpression = new JButton("Add Expression");
		addExpression.setPreferredSize(new Dimension(4 * width, height));
		addExpression.addActionListener(new EditorActionListenerAddExpression(tree, graphMouseListener, expressionField));
		panel.add(addExpression, "" + column + ", " + (row + 1) + ", " + (column + 3) + ", " + (row + 1) + ", l, c");
		
		// now make something to delete expressions
		JLabel exprLabel = new JLabel("Expressions");
		exprLabel.setPreferredSize(new Dimension(width, height));
		expressionCombo = new JComboBox<ProcessTreeElement>();
		expressionCombo.setPreferredSize(new Dimension(2 * width, height));
		updateDataCombo();
		expressionCombo.addActionListener(new EditorActionListenerComboBox(expressionCombo, graphMouseListener));
		panel.add(exprLabel, "" + column + ", " + (row + 2) + ", " + column + ", " + (row + 2) + ", l, c");
		panel.add(expressionCombo, "" + (column + 1) + ", " + (row + 2) + ", " + (column + 2) + ", " + (row + 2) + ", l, c");
		components.add(expressionCombo);
		createDeleteButton(panel, row + 2, column + 3, width, height, Expression.class);
		
	}
	/*
	private void createExpressionPanelBeginner(JPanel panel, int row, int column, int width, int height){
		int localColumns = 6;
		int localRows = 4;
		clauses = new ArrayList<String>();
		varsOfClauses = new HashSet<Variable>();
		// scaling for the border
		int localWidth = (4 * width - 8) / localColumns;
		double[][] layout = new double[2][Math.max(localColumns, localRows)];
		for(int i = 0; i < localColumns; i++){
			layout[0][i] = localWidth;
		}
		for(int i = 0; i < localRows; i++){
			layout[1][i] = heightOfRows;
		}		
		JPanel expressionPanel = new JPanel();
		expressionPanel.setLayout(new TableLayout(layout));
		expressionPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Expressions"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		JLabel expressionLabel = new JLabel("Expression:");
		final JTextField expressionField = new JTextField();
		expressionField.setPreferredSize(new Dimension(5 * localWidth, height));
		
		expressionPanel.add(expressionLabel, "0, 0, 0, 0, l, c");
		expressionPanel.add(expressionField, "1, 0, 5, 0, l, c");
		
		expressionVariableCombo = new JComboBox<>();
		expressionVariableCombo.setPreferredSize(new Dimension(2*localWidth, height));
		expressionVariableCombo.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				updateDataCombo();
			}
		});
		comparator = new JComboBox<>();
		comparator.setPreferredSize(new Dimension(localWidth, height));
		variableValues = new JComboBox<>();
		variableValues.setPreferredSize(new Dimension(2*localWidth, height));
		JButton addClauseSimple = new JButton("Add Clause");
		addClauseSimple.setPreferredSize(new Dimension(localWidth, height));
		addClauseSimple.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				Variable selectedVar = (Variable) expressionVariableCombo.getSelectedItem();
				if(selectedVar != null && variableValues.getSelectedItem() != null){
					varsOfClauses.add(selectedVar);
					clauses.add("(" + selectedVar + " " + comparator.getSelectedItem() + " \"" + variableValues.getSelectedItem() + "\")");
					updateDataCombo();
				}
			}
		});
		
		comparator.addItem("==");
		comparator.addItem("!=");
		
		expressionPanel.add(expressionVariableCombo, "0, 1, 1, 1, l, c");
		expressionPanel.add(comparator, "2, 1, 2, 1, l, c");
		expressionPanel.add(variableValues, "3, 1, 4, 1, l, c");
		expressionPanel.add(addClauseSimple, "5, 1, 5, 1, l, c");
		
		clauseCombo1 = new JComboBox<>();
		clauseCombo1.setPreferredSize(new Dimension(2*localWidth, height));
		operator = new JComboBox<>();
		operator.setPreferredSize(new Dimension(localWidth, height));
		clauseCombo2 = new JComboBox<>();
		clauseCombo2.setPreferredSize(new Dimension(2*localWidth, height));
		
		JButton addClauseComplex = new JButton("Add Clause");
		addClauseComplex.setPreferredSize(new Dimension(localWidth, height));
		addClauseComplex.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				if((clauseCombo1.getSelectedItem() != null || operator.getSelectedItem().equals("!")) && clauseCombo2.getSelectedItem() != null){
					clauses.remove(clauseCombo1.getSelectedItem());
					clauses.remove(clauseCombo2.getSelectedItem());
					if(clauseCombo1.getSelectedItem() != null){
						clauses.add("(" + clauseCombo1.getSelectedItem() + " " + operator.getSelectedItem() + " " + clauseCombo2.getSelectedItem() + ")");
					}
					else{
						clauses.add("(" + operator.getSelectedItem() + clauseCombo2.getSelectedItem() + ")");
					}
					updateDataCombo();
				}
			}
		});
		
		JButton addExpression = new JButton("Add expression");
		addExpression.setPreferredSize(new Dimension(6 * localWidth, height));
		addExpression.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				if(!expressionField.getText().equalsIgnoreCase("")){
					Variable[] usedVars = new Variable[varsOfClauses.size()];
					int index = 0;
					for(Variable var: varsOfClauses){
						usedVars[index] = var;
						index++;
					}
					Expression expr = null;
					if(clauseCombo1.getSelectedItem() != null){
						expr = new ExpressionImpl("(" + clauseCombo1.getSelectedItem() + " " + operator.getSelectedItem() + " " + clauseCombo2.getSelectedItem() + ")", usedVars);
					}
					else{
						expr = new ExpressionImpl("(" + operator.getSelectedItem() + clauseCombo2.getSelectedItem() + ")", usedVars);
					}
					expressionField.setText(expr.getExpression());
					clauses.clear();
					varsOfClauses.clear();
				}
				
			}
		});
		
		operator.addItem("&&");
		operator.addItem("||");
		operator.addItem("!");
		
		expressionPanel.add(clauseCombo1, "0, 2, 1, 2, l, c");
		expressionPanel.add(operator, "2, 2, 2, 2, l, c");
		expressionPanel.add(clauseCombo2, "3, 2, 4, 2, l, c");
		expressionPanel.add(addClauseComplex, "5, 2, 5, 2, l, c");
		expressionPanel.add(addExpression, "0, 3, 5, 3, l, c");
		
		panel.add(expressionPanel, "" + column + ", " + row + ", " + (column + 3) + ", " + (row + 4) + ", l, c");
	}
	*/
}
