package org.processmining.processtree.editor.editorpanel;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorPanelOriginator {
	ProcessTree tree;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	JComboBox<ProcessTreeElement> originatorCombo;
	Collection<JComponent> components;
	JTextField nameField;
	
	private int rows = 4;
	private int heightOfRows = 25;
	Collection<JButton> localComponents;
	
	public EditorPanelOriginator(ProcessTree tree, int maxWidth, int maxHeight, GraphMouseListener graphMouseListener, GraphPanel graphPanel, Collection<JComponent> components){
		this.tree = tree;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.components = components;
		this.localComponents = new HashSet<>();
	}
	
	public int getHeight(){
		// return the height in terms of elements
		// added some for the border
		return rows * heightOfRows + 24;
	}
	
	public JComponent createResourcePerspective(){
		int columns = 4;
		// scaling for the border
		int width = (maxWidth - 8) / columns;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfRows;
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Originator Perspective"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		
		int offset = 0;
		JLabel nodeNameLabel = new JLabel("Name:");
		nodeNameLabel.setPreferredSize(new Dimension(1 * width, heightOfRows));
		nameField = new JTextField();
		nameField.setPreferredSize(new Dimension(3 * width, heightOfRows));
		List<Class<? extends ProcessTreeElement>> validElements = new ArrayList<>();
		validElements.add(Originator.class);
		nameField.addKeyListener(new EditorKeyListenerNameField(tree, graphMouseListener, graphPanel, nameField, validElements));
		panel.add(nodeNameLabel, "0, " + offset + ", 0, " + offset + ", l, c");
		panel.add(nameField, "1, " + offset + ", 3, " + offset + ", l, c");
		offset++;		
		JLabel resNameLabel = new JLabel("Originator name:");
		JTextField resName = new JTextField();
		resName.setPreferredSize(new Dimension(3 * width, heightOfRows));
		panel.add(resNameLabel, "0, " + offset + ", 0, " + offset + ", l, c");
		panel.add(resName, "1, " + offset + ", 3, " + offset + ", l, c");
		createResourceButtons(panel, resName, offset + 1, 1, width, heightOfRows);
		createSelectionOriginators(panel, offset + 2, 0, width, heightOfRows);
		createDeleteButton(panel, offset + 2, 3, width, heightOfRows);
		update();
		return panel;
	}
	
	private void createResourceButtons(JPanel panel, JTextField nameField, int row, int column, int width, int height){
		JButton addResource = new JButton("Add Resource");
		addResource.setPreferredSize(new Dimension(width, height));
		addResource.addActionListener(new EditorActionListenerAddOriginator(tree, graphMouseListener, graphPanel, nameField, true, false, false));
		panel.add(addResource, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(addResource);
		localComponents.add(addResource);
		
		JButton addRole = new JButton("Add Role");
		addRole.setPreferredSize(new Dimension(width, height));
		addRole.addActionListener(new EditorActionListenerAddOriginator(tree, graphMouseListener, graphPanel, nameField, false, true, false));
		panel.add(addRole, "" + (column + 1) + ", " + row + ", " + (column + 1) + ", " + row + ", l, c");
		components.add(addRole);
		localComponents.add(addRole);
		
		JButton addGroup = new JButton("Add Group");
		addGroup.setPreferredSize(new Dimension(width, height));
		addGroup.addActionListener(new EditorActionListenerAddOriginator(tree, graphMouseListener, graphPanel, nameField, false, false, true));
		panel.add(addGroup, "" + (column + 2) + ", " + row + ", " + (column + 2) + ", " + row + ", l, c");
		components.add(addGroup);
		localComponents.add(addGroup);
	}
	
	public void update(){
		updateResourceCombo();
		checkEnablement();
		updateNameField();
	}
	
	private void updateResourceCombo(){
		List<ProcessTreeElement> originators = new ArrayList<ProcessTreeElement>(tree.getOriginators());
		if(originatorCombo == null){
			// not initialised yet so skip
			return;
		}
		// first remove the action listeners
		ActionListener[] originatorComboListeners = originatorCombo.getActionListeners();
		for(ActionListener actListen: originatorComboListeners){
			originatorCombo.removeActionListener(actListen);
		}
		ProcessTreeElement selectedOriginator = (ProcessTreeElement) originatorCombo.getSelectedItem();
		originatorCombo.removeAllItems();
		for(ProcessTreeElement org: originators){
			originatorCombo.addItem(org);
		}
		// now set the selected item
		originatorCombo.setSelectedItem(selectedOriginator);
		
		// now add the actionlisteners again
		for(ActionListener actListen: originatorComboListeners){
			originatorCombo.addActionListener(actListen);
		}
	}
	
	private void checkEnablement(){
		for(JButton component: localComponents){
			for(ActionListener actListen: component.getActionListeners()){
				if(actListen instanceof EditorActionListener){
					component.setEnabled(((EditorActionListener) actListen).isEnabled());
				}
			}
		}
	}
	
	private void updateNameField(){
		UUID selectedID = graphMouseListener.getSelectedID();
		if(tree.getElement(selectedID) != null && tree.getElement(selectedID) instanceof Originator){
			nameField.setEnabled(true);
			nameField.setText(tree.getElement(selectedID).getName());
		}
		else{
			nameField.setText("");
			nameField.setEnabled(false);
		}
	}
	
	private void createSelectionOriginators(JPanel panel, int row, int column, int width, int height){
		JLabel resLabel = new JLabel("Originators");
		resLabel.setPreferredSize(new Dimension(width, height));
		
		originatorCombo = new JComboBox<ProcessTreeElement>();
		originatorCombo.setPreferredSize(new Dimension(2 * width, height));
		updateResourceCombo();
		originatorCombo.addActionListener(new EditorActionListenerComboBox(originatorCombo, graphMouseListener));
		
		panel.add(resLabel, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		panel.add(originatorCombo, "" + (column + 1) + ", " + row + ", " + (column + 2) + ", " + row + ", l, c");
		components.add(originatorCombo);
	}
	
	private void createDeleteButton(JPanel panel, int row, int column, int width, int height){
		JButton deleteNode = new JButton("Delete");
		deleteNode.setPreferredSize(new Dimension(width, height));
		deleteNode.addActionListener(new EditorActionListenerDeleteElement(tree, Originator.class, null, graphMouseListener, graphPanel));
		panel.add(deleteNode, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(deleteNode);
		localComponents.add(deleteNode);
	}
}
