package org.processmining.processtree.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.UUID;

import org.processmining.processtree.Originator;
import org.processmining.processtree.Task;
import org.processmining.processtree.Variable;

public abstract class AbstractTask extends AbstractNode {

	public AbstractTask(String name) {
		super(name);
	}
	
	public AbstractTask(UUID id, String name) {
		super(id, name);
	}
	
	public AbstractTask(Task t){
		this(t.getID(), t);
	}
	
	public AbstractTask(UUID id, Task t){
		super(id, t);
	}
	
	public Collection<Variable> getReadVariablesRecursive(){
		Collection<Variable> vars = new HashSet<Variable>();
		vars.addAll(readVars);
		return vars;
	}
	
	public Collection<Variable> getWrittenVariablesRecursive() {
		Collection<Variable> vars = new HashSet<Variable>();
		vars.addAll(writeVars);
		return vars;
	}

	/**
	 * returns a short string representation of the subclass
	 * 
	 * @return
	 */
	public String toStringShort() {
		return name;
	}
	
	public boolean isLeaf() {
		return true;
	}

	/**
	 * Automatic task
	 * 
	 * @author bfvdonge
	 * 
	 */
	public static class Automatic extends AbstractTask implements Task.Automatic {

		public Automatic(String name) {
			super(name);
		}
		
		public Automatic(UUID id, String name) {
			super(id, name);
		}
		
		public Automatic(Task.Automatic t){
			this(t.getID(), t);
		}
		
		public Automatic(UUID id, Task.Automatic t){
			super(id, t);
		}

	}

	/**
	 * Manual task
	 * 
	 * @author bfvdonge
	 * 
	 */
	public static class Manual extends AbstractTask implements Task.Manual {

		protected Collection<Originator> originators;
		protected Collection<Originator> remOriginators;

		public Manual(String name, Collection<Originator> originators) {
			super(name);
			this.originators = originators;
			this.remOriginators = new HashSet<Originator>();
		}
		
		public Manual(UUID id, String name, Collection<Originator> originators) {
			super(id, name);
			this.originators = originators;
			this.remOriginators = new HashSet<Originator>();
		}

		public Manual(String name, Originator... originators) {
			super(name);
			this.originators = new HashSet<Originator>();
			for (Originator originator : originators) {
				this.originators.add(originator);
			}
			this.remOriginators = new HashSet<Originator>();
		}
		
		public Manual(UUID id, String name, Originator... originators) {
			super(id, name);
			this.originators = new HashSet<Originator>();
			for (Originator originator : originators) {
				this.originators.add(originator);
			}
			this.remOriginators = new HashSet<Originator>();
		}
		
		public Manual(Task.Manual t){
			super(t);
			this.originators = new HashSet<Originator>();
			this.remOriginators = new HashSet<Originator>();
		}

		public boolean addOriginator(Originator originator) {
			return originators.add(originator);
		}
		
		public boolean addRemovableOriginator(Originator removableOriginator) {
			return remOriginators.add(removableOriginator);
		}

		public boolean removeOriginator(Originator originator) {
			return originators.remove(originator);
		}

		/**
		 * Returns a non-empty collection of originators attached to this manual
		 * task
		 * 
		 * @return
		 */
		public Collection<Originator> getOriginators() {
			return originators;
		}
		
		public void addOriginators(Collection<Originator> originators) {
			this.originators = originators;
		}
		
		public Collection<Originator> getRemovableOriginators(){
			return remOriginators;
		}
		
		public void addRemovableOriginators(Collection<Originator> removableOriginators){
			this.remOriginators = removableOriginators;
		}
	}

}
