package org.processmining.processtree.ptml;

import java.util.UUID;

import org.xmlpull.v1.XmlPullParser;

public class PtmlEdge extends PtmlId {

	protected UUID sourceId;
	protected UUID targetId;

	public PtmlEdge(String tag, UUID id) {
		super(tag, id);
		sourceId = null;
		targetId = null;
	}

	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importFromId(xpp, ptml);
		importToId(xpp, ptml);
	}

	private void importFromId(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "sourceId");
		if (value != null) {
			sourceId = UUID.fromString(value);
		}
		ptml.log(sourceId != null ? value : "<no sourceId>");
	}

	private void importToId(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "targetId");
		if (value != null) {
			targetId = UUID.fromString(value);
		}
		ptml.log(targetId != null ? value : "<no targetId>");
	}

	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportFromId(ptml) + exportToId(ptml);
	}

	private String exportFromId(Ptml ptml) {
		if (sourceId != null) {
			return exportAttribute("sourceId", sourceId.toString(), ptml);
		}
		return "";
	}
	
	private String exportToId(Ptml ptml) {
		if (targetId != null) {
			return exportAttribute("targetId", targetId.toString(), ptml);
		}
		return "";
	}
}
