package org.processmining.processtree.ptml;

import java.util.Map;
import java.util.UUID;

import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task;
import org.processmining.processtree.impl.AbstractTask;
import org.xmlpull.v1.XmlPullParser;

public class PtmlIsOriginator extends PtmlEdge {

	public final static String TAG = "performsTask";
	boolean removable = false;
	
	public PtmlIsOriginator(UUID id) {
		super(TAG, id);
		removable = false;
	}
	
	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importRemovable(xpp, ptml);
	}

	private void importRemovable(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "removable");
		if (value != null) {
			//expression = new String(value);
			removable = Boolean.parseBoolean(value);
		}
		ptml.log(value != null ? value : "<no value>");
	}
	
	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportRemovable(ptml);
	}

	private String exportRemovable(Ptml ptml) {
		if (removable) {
			return exportAttribute("removable", "true", ptml);
		}
		return "";
	}

	public void unmarshall(ProcessTree tree, Map<UUID, Node> nodeMap, Map<UUID, Originator> originatorMap) {
		Node manual = nodeMap.get(sourceId);
		Originator originator = originatorMap.get(targetId);
		if (manual != null && manual instanceof AbstractTask.Manual && originator != null) {
			((AbstractTask.Manual) manual).addOriginator(originator);
			if(removable){
				((AbstractTask.Manual) manual).addRemovableOriginator(originator);
			}
		}
	}
	
	public void marshall(ProcessTree tree, Task.Manual manual, Originator originator, Map<Originator, UUID> originatorMap) {
		sourceId = manual.getID();
		targetId = originatorMap.get(originator);
		if(manual.getRemovableOriginators() != null){
			removable = manual.getRemovableOriginators().contains(originator);
		}
	}
}
