package org.processmining.processtree.visualization.bpmn;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.PortView;
import org.processmining.processtree.visualization.bpmn.TreeBPMNLayoutBuilder.EdgeRouting;

public class OrthogonalRouting extends DefaultEdge.DefaultRouting {

	private static final long serialVersionUID = -1806452010692814216L;

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public List route(GraphLayoutCache cache, EdgeView edge) {
		EdgeRouting startHorizontal = (EdgeRouting) edge.getAllAttributes().get(TreeBPMNLayoutBuilder.EDGEROUTING);

		if (startHorizontal == EdgeRouting.NORMAL) {
			return super.route(cache, edge);
		}

		List newPoints = new ArrayList();
		int n = edge.getPointCount();
		Point2D from = edge.getPoint(0);
		newPoints.add(from);
		if (edge.getSource() instanceof PortView) {
			newPoints.set(0, edge.getSource());
			from = ((PortView) edge.getSource()).getLocation();
		} else if (edge.getSource() != null) {
			Rectangle2D b = edge.getSource().getBounds();
			from = edge.getAttributes().createPoint(b.getCenterX(), b.getCenterY());
		}
		Point2D to = edge.getPoint(n - 1);
		CellView target = edge.getTarget();
		if (target instanceof PortView)
			to = ((PortView) target).getLocation();
		else if (target != null) {
			Rectangle2D b = target.getBounds();
			to = edge.getAttributes().createPoint(b.getCenterX(), b.getCenterY());
		}
		if (from != null && to != null) {

			Point2D routed1, routed2;

			if (startHorizontal == EdgeRouting.HORIZONTAL_FIRST) {
				routed1 = edge.getAttributes().createPoint(to.getX(), from.getY());
				routed2 = null;
			} else if (startHorizontal == EdgeRouting.VERTICAL_FIRST) {
				routed1 = edge.getAttributes().createPoint(from.getX(), to.getY());
				routed2 = null;
			} else /* (startHorizontal == EdgeRouting.SKIP) */{
				double y = (Double) edge.getAllAttributes().get(TreeBPMNLayoutBuilder.ROUTESKIPY);
				routed1 = edge.getAttributes().createPoint(from.getX(), from.getY() + y);
				routed2 = edge.getAttributes().createPoint(to.getX(), to.getY() + y);
			}

			Rectangle2D targetBounds = null;
			Rectangle2D sourceBounds = null;
			if ((edge.getTarget() != null && edge.getTarget().getParentView() != null)
					&& (edge.getSource() != null && edge.getSource().getParentView() != null)) {
				targetBounds = edge.getTarget().getParentView().getBounds();
				sourceBounds = edge.getSource().getParentView().getBounds();
				if (!targetBounds.contains(routed1) && (!sourceBounds.contains(routed1))) {
					newPoints.add(routed1);
				}
				if (routed2 != null && !targetBounds.contains(routed2) && (!sourceBounds.contains(routed2))) {
					newPoints.add(routed2);
				}

			}
			// Add target point
			if (target != null)
				newPoints.add(target);
			else
				newPoints.add(to);
			return newPoints;
		}
		return null;
	}

	public int getPreferredLineStyle(EdgeView edge) {
		return GraphConstants.STYLE_ORTHOGONAL;
	}

}
