package org.processmining.processtree.visualization.data;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.visualisation.VisualiseProperties;
import org.processmining.processtree.Block;
import org.processmining.processtree.Block.Or;
import org.processmining.processtree.Block.Xor;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Expression;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Variable;
import org.processmining.processtree.editor.graphpanel.GraphPanel;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class DataVisualiser {
	ProcessTree tree;
	Boolean editable;
	Map<Pair<Node, Variable>, VariableCell> variableCells;
	Node node;
	Edge edge;
	private GraphPanel graphPanel;
	
	public DataVisualiser(ProcessTree tree){
		this(tree, false);
	}
	
	public DataVisualiser(ProcessTree tree, Boolean editable){
		this(tree, null, null, editable, null);
	}
	
	public DataVisualiser(ProcessTree tree, Edge edge){
		this(tree, edge, false);
	}
	
	public DataVisualiser(ProcessTree tree, Edge edge, Boolean editable){
		this(tree, null, edge, editable, null);
	}
	
	public DataVisualiser(ProcessTree tree, Node node, Edge edge, Boolean editable, GraphPanel graphPanel){
		this.tree = tree;
		this.node = node;
		this.edge = edge;
		this.editable = editable;
		this.graphPanel = graphPanel;
		this.variableCells = new HashMap<>();
	}
	
	public List<JComponent> dataVisualiser() throws InstantiationException, IllegalAccessException{
		List<JComponent> comps = new ArrayList<JComponent>();
		// add a check whether the tree is null
		if(edge == null && node == null){
			comps.add(variables());
			comps.add(variableProperties());
			comps.add(expressions());
			comps.add(expressionProperties());
		}
		else if(node != null){
			comps.add(variables(node));
			comps.add(variableProperties(node));
		}
		else{
			comps.add(expression(edge));
			comps.add(expressionProperty(edge));
		}
		return comps;
	}
	
	private JComponent variables(){
		return variables(null);
	}
	
	private JComponent variables(Node node){
		// now let's do it properly
		JPanel panel = new JPanel();
		int width = 2 * tree.getVariables().size() + 1;
		int height = 0;
		Collection<Node> nodes;
		if(node != null){
			nodes = new ArrayList<Node>();
			nodes.add(node);
		}
		else{
			nodes = tree.getNodes();
		}
		height = 2 * nodes.size() + 1;
		double[][] layout = new double[2][Math.max(width, height)];
		for(int i = 0; i < width; i++){
			layout[0][i] = 100;
		}
		for(int i = 0; i < height; i++){
			layout[1][i] = 30;
		}
		panel.setLayout(new TableLayout(layout));
		// collect the elements as the ordering is not given
		List<Variable> vars = new ArrayList<>();
		for(Variable var: tree.getVariables()){
			vars.add(var);
		}
		
		// generate the overview
		int row = 0;
		int offset = 1;
		if(!vars.isEmpty()){
			// visual aid for not having any variables
			for(Node n: nodes){
				// put this node on the panel
				JLabel label = new JLabel(n.getName());
				panel.add(label, "0, " + (row + offset) + ", 0, " + (row + 1 + offset) + ", c, c");
				row += 2;
			}
			int column = 0;
			offset = 1;
			for(Variable var: vars){
				JLabel label = new JLabel(var.getName());
				panel.add(label, "" + (column + offset) + ", 0, " + (column + 1 + offset) + " 0 , c, c");
				column += 2;
			}
			// now render the combinations
			int offsetRow = 1;
			int offsetColumn = 1;
			row = 0;
			column = 0;
			for(Node n: nodes){
				for(Variable var: vars){
					renderCellVariable(panel, n, var, offsetRow + row, offsetColumn + column);
					column += 2;
				}
				row += 2;
				column = 0;
			}
		}
		JScrollPane scroll = new JScrollPane(panel);
		scroll.setName("Variables");
		return scroll;
	}
	
	private void renderCellVariable(JComponent component, Node n, Variable var, int lowerRow, int lowerColumn){
		JCheckBox read = new JCheckBox("r", n.getReadVariables().contains(var));
		read.setToolTipText("Read Variable");
		read.setHorizontalTextPosition(SwingConstants.LEFT);
		JCheckBox write = new JCheckBox("w", n.getWrittenVariables().contains(var));
		write.setToolTipText("Write Variable");
		JCheckBox remRead = new JCheckBox("rr", n.getRemovableReadVariables().contains(var));
		remRead.setToolTipText("Removable Read Variable");
		remRead.setHorizontalTextPosition(SwingConstants.LEFT);
		JCheckBox remWrite = new JCheckBox("rw", n.getRemovableWrittenVariables().contains(var));
		remWrite.setToolTipText("Removable Write Variable");
		read.setEnabled(editable);
		read.addActionListener(new VariableApplyActionListener(variableCells));
		write.setEnabled(editable);
		write.addActionListener(new VariableApplyActionListener(variableCells));
		remRead.setEnabled(read.isSelected() && editable);
		remRead.addActionListener(new VariableApplyActionListener(variableCells));
		remWrite.setEnabled(write.isSelected() && editable);
		remWrite.addActionListener(new VariableApplyActionListener(variableCells));
		VariableCell varCell = new VariableCell(read, write, remRead, remWrite);
		variableCells.put(new Pair<Node, Variable>(n, var), varCell);
		// now add the nodes to the component
		component.add(read, "" + lowerColumn + ", " + lowerRow + ", " + lowerColumn + ", " + lowerRow + ", r, c");
		component.add(write, "" + (lowerColumn + 1) + ", " + lowerRow + ", " + (lowerColumn + 1) + ", " + lowerRow + ", l, c");
		component.add(remRead, "" + lowerColumn + ", " + (lowerRow + 1) + ", " + lowerColumn + ", " + (lowerRow + 1) + ", r, c");
		component.add(remWrite, "" + (lowerColumn + 1) + ", " + (lowerRow + 1) + ", " + (lowerColumn + 1) + ", " + (lowerRow + 1) + ", l, c");
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private JComponent expressions() {
		SlickerTabbedPane tabPane = new SlickerTabbedPane("Expression", Color.BLACK, Color.WHITE, Color.RED);
		int nrRows = 1;
		for(Edge e: tree.getEdges()){
			if(e.hasExpression()){
				nrRows++;
			}
		}
		int nrColomns = 5;
		double[][] layout = new double[2][Math.max(nrColomns, nrRows)];
		SlickerFactory slickerFactory = SlickerFactory.instance();
		for(int i = 0; i < Math.max(nrColomns, nrRows); i++){
			if(i < nrColomns){
				layout[0][i] = 100;
				if(i == 2){
					layout[0][i] = 300;
				}
				if(i == 4){
					layout[0][i] = 300;
				}
			}
			else{
				layout[0][i] = 0;
			}
			if(i < nrRows){
				layout[1][i] = 30;
			}
			else{
				layout[1][i] = 0;
			}
		}
		tabPane.setLayout(new TableLayout(layout));
		HashMap<Edge, JTextField> etext = new HashMap<Edge, JTextField>();
		HashMap<Edge, SelectionManager> eVars = new HashMap<Edge, SelectionManager>();
		int row = 0;
		JLabel sourceL = slickerFactory.createLabel("Source");
		JLabel targetL = slickerFactory.createLabel("Target");
		JLabel exprL = slickerFactory.createLabel("Expression");
		JLabel variablesL = slickerFactory.createLabel("Variables");
		JLabel variableList = slickerFactory.createLabel("Variables");
		tabPane.add(sourceL, "0, " + row + ", 0, " + row + ", l, c");
		tabPane.add(targetL, "1, " + row + ", 1, " + row + ", l, c");
		tabPane.add(exprL, "2, " + row + ", 2, " + row + ", l, c");
		tabPane.add(variablesL, "3, " + row + ", 3, " + row + ", l, c");
		tabPane.add(variableList, "4, " + row + ", 4, " + row + ", l, c");
		row = 1;
		for(Edge e: tree.getEdges()){
			if(e.hasExpression()){
				JLabel source = slickerFactory.createLabel(e.getSource().getName());
				JLabel target = slickerFactory.createLabel(e.getTarget().getName());
				JTextField expr = new JTextField(e.getExpression().getExpression(), 60);
				expr.setEditable(false);
				JLabel variables = slickerFactory.createLabel("");
				String label = "";
				boolean first = true;
				for(Variable var: e.getExpression().getVariables()){
		        	if(first){
		        		first = !first;
		        	}
		        	else {
		        		label += ", ";
		        	}	        	
		        	label += var.getName();
		        }
				variables.setText(label);
				SelectionManager manager = new SelectionManager(variables, e.getExpression());
				MultiRenderer renderer = new MultiRenderer(manager);
				DefaultComboBoxModel model = new DefaultComboBoxModel(tree.getVariables().toArray());
				JComboBox vars = new JComboBox(model);
				vars.addActionListener(manager);
				vars.setRenderer(renderer);
				vars.setEnabled(false);
				tabPane.add(source, "0, " + row + ", 0, " + row + ", l, c");
				tabPane.add(target, "1, " + row + ", 1, " + row + ", l, c");
				tabPane.add(expr, "2, " + row + ", 2, " + row + ", l, c");
				tabPane.add(vars, "3, " + row + ", 3, " + row + ", l, c");
				tabPane.add(variables, "4, " + row + ", 4, " + row + ", l, c");
				etext.put(e, expr);
				eVars.put(e, manager);
				row++;
			}
		}
		tabPane.setName("Expressions");
		JScrollPane scrollPane = new JScrollPane(tabPane);
		scrollPane.setName("Expressions");
		return scrollPane;
	}
	
	private JComponent expression(Edge edge){
		SlickerTabbedPane tabPane = new SlickerTabbedPane("Expression", Color.BLACK, Color.WHITE, Color.RED);
		int nrColumns = 3;
		int nrRows = tree.getExpressions().size() + 1;
		double[][] layout = new double[2][Math.max(nrColumns, nrRows)];
		SlickerFactory slickerFactory = SlickerFactory.instance();
		for(int i = 0; i < Math.max(nrColumns, nrRows); i++){
			if(i < nrColumns){
				layout[0][0] = 500;
				layout[0][1] = 50;
				layout[0][2] = 50;
			}
			else{
				layout[0][i] = 0;
			}
			if(i < nrRows){
				layout[1][i] = 30;
			}
			else{
				layout[1][i] = 0;
			}
		}
		tabPane.setLayout(new TableLayout(layout));
		Map<Expression, JRadioButton> defaultExpression = new HashMap<>();
		Map<Expression, JCheckBox> expressionList = new HashMap<>();
		int row = 0;
		JLabel expressionLabel = slickerFactory.createLabel("Expression: ");
		JLabel defaultLabel = slickerFactory.createLabel("Default: ");
		JLabel configurableLabel = slickerFactory.createLabel("Configurable: ");
		tabPane.add(expressionLabel, "0, " + row + ", 0, " + row + ", l, c");
		tabPane.add(defaultLabel, "1, " + row + ", 1, " + row + ", l, c");
		tabPane.add(configurableLabel, "2, " + row + ", 2, " + row + ", l, c");
		row = 1;
		// now do the other expressions (from the tree)
		for(Expression expr: tree.getExpressions()){
			renderExpression(tabPane, expr, row, defaultExpression, expressionList);
			row++;
		}
		// now do some magic in combining the radio buttons
		ButtonGroup bg = new ButtonGroup();
		for(Expression expr: defaultExpression.keySet()){
			bg.add(defaultExpression.get(expr));
			if(edge.hasExpression() && expr.equals(edge.getExpression())){
				defaultExpression.get(expr).setSelected(true);
			}
			if(edge.getExpressions().contains(expr)){
				expressionList.get(expr).setSelected(true);
			}
		}
		// now do the action listeners
		for(Expression expr: defaultExpression.keySet()){
			JRadioButton radioButton = defaultExpression.get(expr);
			radioButton.addActionListener(new ExpressionActionListener(edge, graphPanel, radioButton, null, expr));
			JCheckBox checkBox = expressionList.get(expr);
			checkBox.addActionListener(new ExpressionActionListener(edge, graphPanel, null, checkBox, expr));
		}
		tabPane.setName("Expressions");
		JScrollPane scrollPane = new JScrollPane(tabPane);
		scrollPane.setName("Expressions");
		return scrollPane;
	}
	
	private void renderExpression(SlickerTabbedPane slickerTabPane, Expression expr, int row, Map<Expression, JRadioButton> defaultExpression, Map<Expression, JCheckBox> expressionList){
		Block source = edge.getSource();
		Boolean enabled = false;
		if(source instanceof Xor || source instanceof Or || (source instanceof XorLoop && source.getOutgoingEdges().indexOf(edge) > 0)){
			enabled = true;
		}
		JLabel expressionLabel = new JLabel(expr.getExpression());
		slickerTabPane.add(expressionLabel, "0, " + row + ", 0, " + row + ", l, c");
		JRadioButton radioButton = new JRadioButton();
		radioButton.setEnabled(enabled);
		slickerTabPane.add(radioButton, "1, " + row + ", 1, " + row + ", l, c");
		JCheckBox checkBox = new JCheckBox();
		checkBox.setEnabled(enabled);
		slickerTabPane.add(checkBox, "2, " + row + ", 2, " + row + ", l, c");
		defaultExpression.put(expr, radioButton);
		expressionList.put(expr, checkBox);
	}
	
	private JComponent expressionProperty(Edge edge){
		JPanel panel = new JPanel();
		panel.setName("Expression Properties");
		return panel;
	}
	
	private JComponent variableProperties() throws InstantiationException, IllegalAccessException{
		return variableProperties(null);
	}
	
	private JComponent variableProperties(Node node) throws InstantiationException, IllegalAccessException{
		// set the variable properties
		JTabbedPane tabPaneSuper = new JTabbedPane();
		tabPaneSuper.setName("VariableProperties");
		for(Variable var: tree.getVariables()){
			if(node != null && !node.getReadVariables().contains(var) && !node.getWrittenVariables().contains(var) &&
					!node.getRemovableReadVariables().contains(var) && !node.getRemovableWrittenVariables().contains(var)){
				// never mind
			}
			else{
				JTabbedPane tabpane = new JTabbedPane();
				tabPaneSuper.addTab(var.getName(), tabpane);
				List<JComponent> components = VisualiseProperties.visualiseProperties(var, editable); 
				for(JComponent comp: components){
					tabpane.add(comp);
				}
			}
		}
		return tabPaneSuper;
	}
	
	private JComponent expressionProperties() throws InstantiationException, IllegalAccessException{
		// set the variable properties
		JTabbedPane tabPaneSuper = new JTabbedPane();
		tabPaneSuper.setName("ExpressionProperties");
		for(Expression expr: tree.getExpressions()){
			JTabbedPane tabpane = new JTabbedPane();
			tabPaneSuper.addTab(expr.getName(), tabpane);
			List<JComponent> components = VisualiseProperties.visualiseProperties(expr); 
			for(JComponent comp: components){
				tabpane.add(comp);
			}
		}
		return tabPaneSuper;
	}
	@SuppressWarnings("rawtypes")
	class SelectionManager implements ActionListener {   
	    JComboBox combo = null;   
	    List<Object> selectedItems = new ArrayList<Object>();  // j2se 1.5+   
	    // List selectedItems = new ArrayList();               // j2se 1.4-   
	    List<Object> nonSelectables = new ArrayList<Object>();   
	    JLabel selectedVars;
	    
	    public SelectionManager(JLabel vars, Expression expression){
	    	this.selectedVars = vars;
	    	this.selectedItems.addAll(expression.getVariables());
	    }
	    
	    public void actionPerformed(ActionEvent e) {   
	        if(combo == null) {   
	            combo = (JComboBox)e.getSource();   
	        }
	        if(!combo.isEnabled()){
	        	return;
	        }
	        Object item = combo.getSelectedItem();   
	        // Toggle the selection state for item.   
	        if(selectedItems.contains(item)) {   
	            selectedItems.remove(item);   
	        } else if(!nonSelectables.contains(item)) {   
	            selectedItems.add(item);   
	        }
	        // now repaint the JLabel
	        String label = "";
	        boolean first = true;
	        for(Object o: selectedItems){
	        	if(first){
	        		first = !first;
	        	}
	        	else {
	        		label += ", ";
	        	}	        	
	        	label += o.toString();
	        }
	        selectedVars.setText(label);
	    }   
	    
	    /**  
	     * The varargs feature (Object... args) is new in j2se 1.5  
	     * You can replace the argument with an array.  
	     */  
	    public void setNonSelectables(Object... args) {   
	        for(int j = 0; j < args.length; j++) {   
	            nonSelectables.add(args[j]);   
	        }   
	    }   
	    
	    public boolean isSelected(Object item) {   
	        return selectedItems.contains(item);   
	    }   
	}
	
	class MultiRenderer extends BasicComboBoxRenderer {   
		private static final long serialVersionUID = 1L;
		SelectionManager selectionManager;   
	    
	    public MultiRenderer(SelectionManager sm) {   
	        selectionManager = sm;   
	    }   
	    
	    @SuppressWarnings("rawtypes")
		public Component getListCellRendererComponent(JList list,   
	                                                  Object value,   
	                                                  int index,   
	                                                  boolean isSelected,   
	                                                  boolean cellHasFocus) {   
	        if (selectionManager.isSelected(value)) {   
	            setBackground(list.getSelectionBackground());   
	            setForeground(list.getSelectionForeground());   
	        } else {   
	            setBackground(list.getBackground());   
	            setForeground(list.getForeground());   
	        }   
	    
	        setFont(list.getFont());   
	    
	        if (value instanceof Icon) {   
	            setIcon((Icon)value);   
	        } else {   
	            setText((value == null) ? "" : value.toString());   
	        }   
	        return this;   
	    }   
	}  
}
