package org.processmining.processtree.visualization.extrainforenderer.colourcoding.impl;

import java.awt.Color;

import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.visualization.extrainforenderer.colourcoding.ColourCoding;

public abstract class ColourCodingMap implements ColourCoding {

	Property<?> property;
	boolean independent;
	double min;
	double max;
	
	public ColourCodingMap(Property<?> property, boolean independent) {
		this(property, independent, 0d, 1d);
	}

	public ColourCodingMap(Property<?> property, boolean independent, double min, double max) {
		this.property = property;
		if (property == null) {
			throw new NullPointerException();
		}
		this.independent = independent;
		this.min = min;
		this.max = max;
	}
	
	/*
	 * compute a colour from a value between 0 and 1, both inclusive
	 */
	protected abstract Color giveColour(double value);

	public Color giveColour(ProcessTree tree, Node n) throws InstantiationException, IllegalAccessException {
		if (tree == null || n == null) {
			throw new NullPointerException();
		}

		//fetch the value of the property
		double value = 0;
		Color defaultColour = new Color(1.0f, 1.0f, 1.0f);

		if (independent) {
			if (n.getIndependentProperty(property) != null) {
				value = property.getValueDouble(n.getIndependentProperty(property));
			} else {
				return defaultColour;
			}
		} else {
			if (n.getDependentProperty(property) != null) {
				value = property.getValueDouble(n.getDependentProperty(property));
			} else {
				return defaultColour;
			}
		}

		//normalise the value to a value between 0 and 1
		double maxdiff = max - min;
		double diff = Math.min(Math.max(0, value - min), maxdiff);
		double v = diff / maxdiff;

		return giveColour(v);
	}

	public Property<?> coloursProperty() {
		return property;
	}

	public boolean isIndependent() {
		return independent;
	}
}
