package org.processmining.processtree.visualization.extrainforenderer.impl;

import java.awt.Color;
import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.SwingConstants;

import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.visualization.extrainforenderer.ExtraInfoRenderer;
import org.processmining.processtree.visualization.extrainforenderer.colourcoding.ColourCoding;

import com.jgraph.components.labels.CellConstants;

public class ExtraInfoRendererImpl implements ExtraInfoRenderer {

	@SuppressWarnings("unused")
	private static final long serialVersionUID = 5311999373009693286L;
	private final ProcessTree tree;
	private final Class<? extends Property<?>> property;
	private final ArrayList<ColourCoding> colours;
	private final double minWidth = 5d;
	private final double minHeight = 5d;
	private final double prefWidthIcon = 20d;
	private final double prefHeightIcon = 20d;
	private final double prefWidthBar = 100d;
	private final double prefHeightBar = 20d;
	private final double subDivision = 0.4d;
	
	
	public ExtraInfoRendererImpl(ProcessTree tree, Class<? extends Property<?>> property, ArrayList<ColourCoding> colours){
		this.tree = tree;
		this.property = property;
		this.colours = colours;
		if(tree == null || property == null || colours.isEmpty()){
			throw new NullPointerException();
		}
	}
	
	public HashSet<DefaultGraphCell> drawComponent(GraphLayoutCache g, UUID id, double x, double y, double maxWidth, double maxHeight) throws InstantiationException, IllegalAccessException {
		// we need to make a division to what extend we want to have an icon and to what extend we want to have a bar
		HashSet<DefaultGraphCell> ret = new HashSet<DefaultGraphCell>();
		
		ImageIcon icon = property.newInstance().getIcon();
		
		DefaultGraphCell graphCellIcon;
		graphCellIcon = new DefaultGraphCell();
		GraphConstants.setBounds(graphCellIcon.getAttributes(), new Rectangle2D.Double(x, y, Math.min(prefWidthIcon, subDivision*maxWidth), Math.min(prefHeightIcon, maxHeight)));
		GraphConstants.setConnectable(graphCellIcon.getAttributes(), false);
		GraphConstants.setDisconnectable(graphCellIcon.getAttributes(), false);
		GraphConstants.setVerticalAlignment(graphCellIcon.getAttributes(), SwingConstants.TOP);
		if(icon != null){
			Image img = icon.getImage();
			Image img2 = img.getScaledInstance((int)Math.min(prefWidthIcon, subDivision*maxWidth), (int)Math.min(prefHeightIcon, maxHeight), java.awt.Image.SCALE_SMOOTH);
			GraphConstants.setIcon(graphCellIcon.getAttributes(), new ImageIcon(img2));
			CellConstants.setLineWidth(graphCellIcon.getAttributes(), 0);
			GraphConstants.setBorder(graphCellIcon.getAttributes(), BorderFactory.createLineBorder(Color.BLACK, 0));
		}
		else{
			CellConstants.setLineWidth(graphCellIcon.getAttributes(), 1);
			GraphConstants.setBorder(graphCellIcon.getAttributes(), BorderFactory.createLineBorder(Color.BLACK, 1));
		}
		
		GraphConstants.setOpaque(graphCellIcon.getAttributes(), true);
		CellConstants.setBackground(graphCellIcon.getAttributes(), Color.WHITE);
		CellConstants.setLineColor(graphCellIcon.getAttributes(), Color.BLACK);
		
		ret.add(graphCellIcon);
		
		// here we have the bar, so we need to cut it into little pieces
		for(ColourCoding colour: colours){
			double xBar = x + Math.min(prefWidthIcon, subDivision*maxWidth) + 10;
			
			DefaultGraphCell graphCellBar;
			graphCellBar = new DefaultGraphCell();
			GraphConstants.setBounds(graphCellBar.getAttributes(), new Rectangle2D.Double(xBar, y, Math.min(prefWidthBar, (1 - subDivision) * maxWidth), Math.min(prefHeightBar, maxHeight)));
			GraphConstants.setBorder(graphCellBar.getAttributes(), BorderFactory.createLineBorder(Color.BLACK, 1));
			GraphConstants.setConnectable(graphCellBar.getAttributes(), false);
			GraphConstants.setDisconnectable(graphCellBar.getAttributes(), false);
			GraphConstants.setVerticalAlignment(graphCellBar.getAttributes(), SwingConstants.TOP);
			CellConstants.setLineWidth(graphCellBar.getAttributes(), 1);
			
			GraphConstants.setOpaque(graphCellBar.getAttributes(), true);
			CellConstants.setBackground(graphCellBar.getAttributes(), colour.giveColour(tree, tree.getNode(id)));
			CellConstants.setLineColor(graphCellBar.getAttributes(), colour.giveColour(tree, tree.getNode(id)));
			ret.add(graphCellBar);
		}	
		return ret;
	}
	
	public double getMinWidth(){
		return minWidth;
	}
	
	public double getMinHeight(){
		return minHeight;
	}

	public String printFriendly() throws InstantiationException, IllegalAccessException{
		return "Visualisation for " + property.newInstance().getName();
	}
	
}
