package org.processmining.processtree.visualization.tree;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import org.jgraph.JGraph;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.impl.Fitness;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.visualization.extrainforenderer.ExtraInfoRenderer;
import org.processmining.processtree.visualization.extrainforenderer.colourcoding.ColourCoding;
import org.processmining.processtree.visualization.extrainforenderer.colourcoding.impl.ColourMapBlackBodyRadiation;
import org.processmining.processtree.visualization.extrainforenderer.impl.ExtraInfoRendererImpl;
import org.processmining.processtree.visualization.interaction.MouseInteraction;

public class TreeVisualization {
	@Plugin(name = "Visualize Process tree as Tree", level = PluginLevel.PeerReviewed, returnLabels = { "Visualized Process tree" }, returnTypes = { JComponent.class }, parameterLabels = {
	"Process tree" }, userAccessible = true)
	@Visualizer
	public JComponent visualize(PluginContext context, ProcessTree tree) {
		TreeLayoutBuilder builder = new TreeLayoutBuilder(tree);
		JGraph graph = builder.getJGraph();
		JFrame frame = new JFrame();
		frame.getContentPane().add(new JScrollPane(graph));
		return (JComponent) frame.getContentPane();
	}
	
	@Plugin(name = "Visualize Process tree with Fitness", returnLabels = { "Visualized Process tree" }, returnTypes = { JComponent.class }, parameterLabels = {
	"Process tree" }, userAccessible = true)
	@Visualizer
	public JComponent visualizePerformance(PluginContext context, ProcessTree tree) {
		
		//set up extra info renderers
		ArrayList<ExtraInfoRenderer> renderers = new ArrayList<ExtraInfoRenderer>();
		ArrayList<ColourCoding> colours = new ArrayList<ColourCoding>();
		
		Property<?> property = new Fitness();
		ColourCoding cc = new ColourMapBlackBodyRadiation(property, true, 0, 1);
		colours.add(cc);
		renderers.add(new ExtraInfoRendererImpl(tree, Fitness.class, colours));
		
		TreeLayoutBuilder builder = new TreeLayoutBuilder(tree, renderers, null);
		JGraph graph = builder.getJGraph();
		JFrame frame = new JFrame();
		frame.getContentPane().add(new JScrollPane(graph));
		return (JComponent) frame.getContentPane();
	}
	
	
	@Plugin(name = "Visualize Process tree as Tree With Properties", returnLabels = { "Visualized Process tree with properties" }, returnTypes = { JComponent.class }, parameterLabels = {
	"Process tree" }, userAccessible = true)
	@Visualizer
	public JComponent visualizeProperties(PluginContext context, ProcessTree tree) {
		Set<UUID> foldedElements = new HashSet<UUID>();
		TreeLayoutBuilder builder = new TreeLayoutBuilder(tree, null, null, foldedElements);
		JGraph graph = builder.getJGraph();
		JTabbedPane tabPane = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
		JScrollPane scrollPane = new JScrollPane(graph);
		JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		graph.addMouseListener(new MouseInteraction(tree, graph, builder, sp, scrollPane, tabPane, foldedElements));
		JFrame frame = new JFrame();
		sp.add(scrollPane);
		sp.add(tabPane);
		frame.getContentPane().add(sp);
		return (JComponent) frame.getContentPane();
	}
}
