package org.processmining.projectedrecallandprecision.helperclasses.automata;

import java.util.BitSet;

import org.processmining.projectedrecallandprecision.helperclasses.automata.DFAImplEdgeList.EdgeIterableOutgoing;

import gnu.trove.list.TIntList;
import gnu.trove.list.TShortList;

public interface DFA {

	public int getInitialState();

	public void setAccepting(int state, boolean accepting);

	public boolean isAccepting(int state);

	/**
	 * 
	 * @return A copy of the accepting states in a bitset.
	 */
	public BitSet getAcceptingStates();

	/**
	 * Add an edge to the graph. If there is already an activity-edge outgoing
	 * of source, its target is replaced by the given target.
	 * 
	 * @param source
	 * @param activity
	 * @param target
	 */
	public void addEdge(int source, short activity, int target);

	/**
	 * Adds an edge to the graph. Returns the (possibly new) target state.
	 * 
	 * @param source
	 * @param activity
	 * @return
	 */
	public int addEdge(int source, short activity);

	/**
	 * Replace all incoming edges of oldTarget and redirect them to newTarget,
	 * and delete all outgoing edges of oldTarget. It is required that oldTarget
	 * > newTarget. Works in O(n).
	 * 
	 * @param remove
	 * @param redirectEdgesTo
	 */
	public void removeStateAndRedirectEdgesTo(int remove, int redirectEdgesTo);

	/**
	 * Remove all the outgoing edges of a state.
	 * 
	 * @param state
	 */
	public void removeOutgoingEdgesOf(int state);

	public void removeStatesExcept(BitSet keepStates);

	public String toString();

	/**
	 * Returns a dot representation of the automaton.
	 */
	public String toDot();

	/**
	 * Get an iterator over the outgoing edges of a state. This iterator returns
	 * the outgoing edges in order of the activities.
	 * 
	 * @param state
	 * @return
	 */
	public EdgeIterableOutgoing getOutgoingEdgesIterator(int state);

	/**
	 * 
	 * @return The highest index of any state in the automaton + 1. Does not
	 *         necessarily count the number of states, but gives an upper bound.
	 */
	public int getNumberOfStates();

	public TIntList getTargets();

	public TIntList getSources();

	public TShortList getActivities();

	/**
	 * 
	 * @param automaton2State
	 * @param activity
	 * @return the state reached after taking the activity step from state, or
	 *         -1 if this is not possible, or if state was -1.
	 */
	public int step(int state, short activity);

}