package org.processmining.projectedrecallandprecision.helperclasses.automata.operations;

import java.util.ArrayDeque;
import java.util.Iterator;
import java.util.List;

import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFA;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFAImplEdgeList;

import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TObjectIntHashMap;

public class Automaton2Union {
	/**
	 * Returns an automaton that accepts the union of the given automata. Does
	 * not modify the automata.
	 * 
	 * @param canceller
	 * @param automata
	 * @return the first automaton for convenience.
	 */
	public static DFA union(List<DFA> automata, ProMCanceller canceller) {
		Iterator<DFA> it = automata.iterator();
		if (!it.hasNext()) {
			return null;
		}

		DFA result = it.next();

		while (it.hasNext()) {
			result = union(result, it.next(), canceller);
			
			if (result == null || canceller.isCancelled()) {
				return null;
			}
		}

		return result;
	}

	public static DFA union(DFA automatonA, DFA automatonB, ProMCanceller canceller) {
		DFA union = new DFAImplEdgeList();

		ArrayDeque<StatePair> worklist = new ArrayDeque<>();
		TObjectIntMap<StatePair> statePair2unionState = new TObjectIntHashMap<>(10, 0.5f, -1);
		{
			StatePair statePair = new StatePair(automatonA.getInitialState(), automatonB.getInitialState());
			worklist.add(statePair);
			statePair2unionState.put(statePair, union.getInitialState());
		}

		StatePairEdgesIterator it = new StatePairEdgesIterator(automatonA, automatonB);
		while (!worklist.isEmpty()) {
			StatePair sourcePair = worklist.removeFirst();
			int unionSourceState = statePair2unionState.get(sourcePair);

			//set the state accepting if necessary
			union.setAccepting(unionSourceState,
					(sourcePair.stateA == -1 ? false : automatonA.isAccepting(sourcePair.stateA))
							|| (sourcePair.stateB == -1 ? false : automatonB.isAccepting(sourcePair.stateB)));

			it.reset(sourcePair);

			while (it.hasNext()) {
				it.next();
				short activity = it.getActivity();
				StatePair statePairTarget = new StatePair(it.getTargetA(), it.getTargetB());
				int unionTargetState = statePair2unionState.get(statePairTarget);
				if (unionTargetState == statePair2unionState.getNoEntryValue()) {
					//this state pair did not exist yet
					unionTargetState = union.addEdge(unionSourceState, activity);
					statePair2unionState.put(statePairTarget, unionTargetState);

					worklist.add(statePairTarget);
				} else {
					//this state pair did not exist yet
					union.addEdge(unionSourceState, activity, unionTargetState);
				}
			}

		}

		return union;
	}
}
