package org.processmining.projectedrecallandprecision.helperclasses.automata;

import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.concurrent;
import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.leaf;
import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.loop;
import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.or;
import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.sequence;
import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.tau;
import static org.processmining.plugins.InductiveMiner.efficienttree.InlineTree.xor;
import static org.processmining.projectedrecallandprecision.helperclasses.automata.DFAOperations.accepts;
import static org.processmining.projectedrecallandprecision.helperclasses.automata.testAutomaton.check;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.plugins.InductiveMiner.Triple;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTree;
import org.processmining.projectedrecallandprecision.helperclasses.Tree2Automaton;

public class testAutomatonFromTree {

	public static final List<Triple<EfficientTree, short[][], short[][]>> trees;
	//A: tree
	//B: accepted traces
	//C: not-accepted traces
	static {
		trees = new ArrayList<>();
		trees.add(Triple.of( //
				(EfficientTree) tau(), //
				new short[][] { {} }, //
				new short[][] { { 1, 2 }, { 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) leaf("a"), //
				new short[][] { { 0 } }, //
				new short[][] { {}, { 0, 0 }, { 0, 1 }, { 1 } })); //

		trees.add(Triple.of( //
				(EfficientTree) xor(leaf("a"), leaf("b")), //
				new short[][] { { 0 }, { 1 } }, //
				new short[][] { {}, { 0, 0 }, { 0, 1 }, { 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) xor(leaf("a"), leaf("b"), leaf("a")), //
				new short[][] { { 0 }, { 1 } }, //
				new short[][] { {}, { 0, 0 }, { 0, 1 }, { 1, 1 }, { 2 } })); //

		trees.add(Triple.of( //
				(EfficientTree) xor(leaf("a"), leaf("b"), leaf("a"), tau()), //
				new short[][] { { 0 }, { 1 }, {} }, //
				new short[][] { { 0, 0 }, { 0, 1 }, { 1, 1 }, { 2 } })); //

		trees.add(Triple.of( //
				(EfficientTree) sequence(leaf("a"), leaf("b"), leaf("a"), tau()), //
				new short[][] { { 0, 1, 0 } }, //
				new short[][] { { 0, 0 }, { 0, 1 }, { 1, 1 }, { 2 }, {} })); //

		trees.add(Triple.of( //
				(EfficientTree) sequence(xor(leaf("a"), leaf("b")), xor(leaf("a"), tau())), //
				new short[][] { { 0, 0 }, { 0 }, { 1, 0 }, { 1 } }, //
				new short[][] { {}, { 0, 1 }, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) concurrent(leaf("a"), leaf("b"), leaf("a"), tau()), //
				new short[][] { { 0, 1, 0 }, { 0, 0, 1 }, { 1, 0, 0 } }, //
				new short[][] { {}, { 0, 1 }, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) concurrent(xor(leaf("a"), leaf("b")), xor(leaf("a"), tau())), //
				new short[][] { { 0, 0 }, { 0 }, { 1, 0 }, { 1 }, { 0, 1 } }, //
				new short[][] { {}, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) concurrent(sequence(leaf("a"), leaf("b")), xor(leaf("a"), tau())), //
				new short[][] { { 0, 1, 0 }, { 0, 1 }, { 0, 0, 1 } }, //
				new short[][] { {}, { 1, 1 }, { 2 }, { 0, 0, 0 }, { 1, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) xor(sequence(leaf("a"), leaf("b")), tau()), //
				new short[][] { {}, { 0, 1 } }, //
				new short[][] { { 1 }, { 0 }, { 1, 0 }, { 0, 0, 0 }, { 1, 0, 0 }, { 0, 1, 0 }, { 0, 0, 1 } })); //

		trees.add(Triple.of( //
				(EfficientTree) concurrent(sequence(leaf("a"), leaf("b")), xor(sequence(leaf("a"), leaf("b")), tau())), //
				new short[][] { { 0, 1 }, { 0, 1, 0, 1 }, { 0, 0, 1, 1 } }, //
				new short[][] { {}, { 1, 1 }, { 2 }, { 0, 0, 0 }, { 1, 0, 0 }, { 0, 1, 0 }, { 0, 0, 1 } })); //

		trees.add(Triple.of( //
				(EfficientTree) sequence(xor(leaf("a"), tau()), xor(leaf("a"), leaf("b"))), //
				new short[][] { { 0, 0 }, { 0, 1 }, { 0 }, { 1 } }, //
				new short[][] { {}, { 1, 0 }, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) loop(leaf("a"), tau(), tau()), //
				new short[][] { { 0 }, { 0, 0 }, { 0, 0, 0 } }, //
				new short[][] { {}, { 0, 1 }, { 1, 1 }, { 2 } })); //

		trees.add(Triple.of( //
				(EfficientTree) loop(xor(leaf("a"), leaf("b")), xor(leaf("a"), tau()), leaf("c")), //
				new short[][] { { 0, 0, 2 }, { 0, 2 }, { 1, 0, 2 }, { 1, 2 } }, //
				new short[][] { {}, { 0, 1 }, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) or(leaf("a"), leaf("b")), //
				new short[][] { { 0 }, { 1 }, { 0, 1 }, { 1, 0 } }, //
				new short[][] { {}, { 0, 0 }, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //

		trees.add(Triple.of( //
				(EfficientTree) or(xor(tau(), leaf("a")), xor(tau(), leaf("b"))), //
				new short[][] { {}, { 0 }, { 1 }, { 0, 1 }, { 1, 0 } }, //
				new short[][] { { 0, 0 }, { 1, 1 }, { 2 }, { 0, 0, 0 } })); //
	}

	private static ProMCanceller canceller = new ProMCanceller() {
		public boolean isCancelled() {
			return false;
		}
	};

	public static void test() throws Exception {
		for (Triple<EfficientTree, short[][], short[][]> triple : trees) {
			EfficientTree tree = triple.getA();
			short[][] acceptedTraces = triple.getB();
			short[][] rejectedTraces = triple.getC();

			System.out.println(" test tree: " + tree);

			DFA automaton = Tree2Automaton.convert(tree, canceller);

			//System.out.println(automaton);

			for (short[] trace : acceptedTraces) {
				check(accepts(automaton, trace), "  trace acceptance " + Arrays.toString(trace));
			}

			for (short[] trace : rejectedTraces) {
				check(!accepts(automaton, trace), "  trace rejection " + Arrays.toString(trace));
			}

		}
	}
}
