package org.processmining.projectedrecallandprecision.plugins;

import java.util.Arrays;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTree;
import org.processmining.plugins.InductiveMiner.efficienttree.ProcessTree2EfficientTree;
import org.processmining.plugins.InductiveMiner.plugins.dialogs.IMMiningDialog;
import org.processmining.plugins.flowerMiner.FlowerMinerDfg;
import org.processmining.processtree.ProcessTree;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.framework.CompareParameters.RecallName;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.helperclasses.EfficientLog;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResultImplBasic;

public class CompareLog2ProcessTreePluginScaledLogPrecision {
	@Plugin(name = "Compute projected fitness and scaled precision (log and process tree)", returnLabels = {
			"fitness and scaled precision" }, returnTypes = {
					ProjectedRecallPrecisionResult.class }, parameterLabels = { "Log", "Model" }, userAccessible = true)
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Mine a Process Tree, dialog", requiredParameterLabels = { 0, 1 })
	public ProjectedRecallPrecisionResult measure(final UIPluginContext context, XLog log, ProcessTree model)
			throws AutomatonFailedException, InterruptedException, ProjectedMeasuresFailedException {
		CompareParametersDialog dialog = new CompareParametersDialog(log,
				ProcessTree2EfficientTree.convert(model).getInt2activity().length, RecallName.fitness);
		InteractionResult dialogResult = context.showWizard("Projected Conformance Checking", true, true, dialog);
		if (dialogResult != InteractionResult.FINISHED) {
			context.getFutureResult(0).cancel(false);
			return null;
		}

		context.log("Comparing...");

		ProjectedRecallPrecisionResult result = measure(log, model, dialog.getMiningParameters(), new ProMCanceller() {
			public boolean isCancelled() {
				return context.getProgress().isCancelled();
			}
		});

		if (result == null || context.getProgress().isCancelled()) {
			context.getFutureResult(0).cancel(false);
			return null;
		}

		return result;
	}

	public static ProjectedRecallPrecisionResult measure(XLog log, ProcessTree model, CompareParameters parameters,
			ProMCanceller canceller)
			throws AutomatonFailedException, InterruptedException, ProjectedMeasuresFailedException {
		EfficientLog efficientLog = new EfficientLog(log, parameters.getClassifier());
		ProjectedRecallPrecisionResult resultPetriNet = CompareLog2EfficientTreePlugin.measure(efficientLog,
				ProcessTree2EfficientTree.convert(model), parameters, canceller);

		//compute log-precision of flower model
		EfficientTree flowerModel = FlowerMinerDfg.mine(efficientLog.getActivities());
		CompareParameters flowerCompareParameters = parameters.clone();
		flowerCompareParameters.setComputeRecallFitness(false);
		ProjectedRecallPrecisionResult resultFlower = CompareLog2EfficientTreePlugin.measure(efficientLog, flowerModel,
				flowerCompareParameters, canceller);

		if (resultPetriNet.getNumberOfFailedMeasures() > 0 || resultFlower.getNumberOfFailedMeasures() > 0) {
			throw new ProjectedMeasuresFailedException();
		}

		return normalise(resultPetriNet, resultFlower);
	}

	public static ProjectedRecallPrecisionResult normalise(ProjectedRecallPrecisionResult modelResult,
			ProjectedRecallPrecisionResult flowerResult)
			throws AutomatonFailedException, ProjectedMeasuresFailedException {
		//perform the comparison
		ProjectedRecallPrecisionResultImplBasic result = new ProjectedRecallPrecisionResultImplBasic(1,
				Arrays.asList(modelResult.getActivities()), modelResult.getRecallName(), modelResult.getSystem(),
				modelResult.getModel(), modelResult.isRecallComputed(), modelResult.isPrecisionComputed());
		for (String activity : modelResult.getActivities()) {
			double precision = 1 - (1 - modelResult.getPrecision(activity)) / (1 - flowerResult.getPrecision(activity));
			result.set(modelResult.getRecall(activity), precision, activity);
		}
		return result;
	}
}
