package org.processmining.projectedrecallandprecision.plugins;

import java.util.Set;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.plugins.InductiveMiner.plugins.dialogs.IMMiningDialog;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.framework.CompareParameters.RecallName;
import org.processmining.projectedrecallandprecision.framework.ComparePetriNet2PetriNet;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

import gnu.trove.set.hash.THashSet;

public class ComparePetriNet2PetriNetPlugin {

	@Plugin(name = "Compute projected recall and precision (accepting Petri net and accepting Petri net)", returnLabels = {
			"fitness and precision" }, returnTypes = { ProjectedRecallPrecisionResult.class }, parameterLabels = {
					"System", "Model" }, userAccessible = true)
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Mine a Process Tree, dialog", requiredParameterLabels = { 0, 1 })
	public ProjectedRecallPrecisionResult measure(final UIPluginContext context, AcceptingPetriNet system,
			AcceptingPetriNet model) throws AutomatonFailedException, ReductionFailedException, InterruptedException,
			ProjectedMeasuresFailedException {
		Set<String> labels = new THashSet<>();
		for (Transition t : system.getNet().getTransitions()) {
			if (!t.isInvisible()) {
				labels.add(t.getLabel());
			}
		}
		for (Transition t : model.getNet().getTransitions()) {
			if (!t.isInvisible()) {
				labels.add(t.getLabel());
			}
		}
		CompareParametersDialog dialog = new CompareParametersDialog(null, labels.size(), RecallName.recall);
		InteractionResult result = context.showWizard("Projected Conformance Checking", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			context.getFutureResult(0).cancel(false);
			return null;
		}

		context.log("Comparing...");

		return measure(system, model, dialog.getMiningParameters(), new ProMCanceller() {
			public boolean isCancelled() {
				return context.getProgress().isCancelled();
			}
		});
	}

	/**
	 * 
	 * @param system
	 * @param systemInitialMarking
	 * @param systemFinalMarkings
	 * @param system
	 * @param canceller 
	 * @param modelInitialMarking
	 * @param modelFinalMarkings
	 * @param data.k
	 *            the size of subsets of activities to which projection is done
	 * @return
	 * @throws InterruptedException
	 * @throws ReductionFailedException
	 * @throws AutomatonFailedException
	 * @throws ProjectedMeasuresFailedException
	 */
	public static ProjectedRecallPrecisionResult measure(AcceptingPetriNet system, AcceptingPetriNet model,
			CompareParameters parameters, ProMCanceller canceller) throws AutomatonFailedException, ReductionFailedException,
			InterruptedException, ProjectedMeasuresFailedException {
		ComparePetriNet2PetriNet m = new ComparePetriNet2PetriNet();
		return m.apply(system, model, parameters, canceller);
	}
}
