package org.processmining.projectedrecallandprecision.plugins;

import java.util.Set;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTree;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduce.ReductionFailedException;
import org.processmining.plugins.InductiveMiner.efficienttree.ProcessTree2EfficientTree;
import org.processmining.plugins.InductiveMiner.efficienttree.UnknownTreeNodeException;
import org.processmining.plugins.InductiveMiner.plugins.dialogs.IMMiningDialog;
import org.processmining.processtree.ProcessTree;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.framework.CompareParameters.RecallName;
import org.processmining.projectedrecallandprecision.framework.CompareProcessTree2ProcessTree;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

import gnu.trove.set.hash.THashSet;

public class CompareProcessTree2ProcessTreePlugin {
	@Plugin(name = "Compute projected recall and precision (process tree and process tree)", returnLabels = {
			"recall and precision" }, returnTypes = { ProjectedRecallPrecisionResult.class }, parameterLabels = {
					"System", "Model" }, userAccessible = true, level = PluginLevel.Regular)
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Mine a Process Tree, dialog", requiredParameterLabels = { 0, 1 })
	public ProjectedRecallPrecisionResult measure1automata(final UIPluginContext context, ProcessTree system,
			ProcessTree model) throws AutomatonFailedException, ReductionFailedException, InterruptedException,
			UnknownTreeNodeException, ProjectedMeasuresFailedException {
		Set<String> labels = new THashSet<>();
		labels.addAll(ProcessTree2EfficientTree.convert(system).getActivity2int().keySet());
		labels.addAll(ProcessTree2EfficientTree.convert(model).getActivity2int().keySet());

		CompareParametersDialog dialog = new CompareParametersDialog(null, labels.size(), RecallName.recall);
		InteractionResult result = context.showWizard("Projected Conformance Checking", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			context.getFutureResult(0).cancel(false);
			return null;
		}

		context.log("Comparing...");

		return measure(system, model, dialog.getMiningParameters(), new ProMCanceller() {
			public boolean isCancelled() {
				return context.getProgress().isCancelled();
			}
		});
	}

	/**
	 * 
	 * @param system
	 * @param model
	 * @param canceller
	 * @param data.k
	 *            the size of subsets of activities to which projection is done
	 * @return
	 * @throws InterruptedException
	 * @throws ReductionFailedException
	 * @throws AutomatonFailedException
	 * @throws UnknownTreeNodeException
	 * @throws ProjectedMeasuresFailedException
	 */
	public static ProjectedRecallPrecisionResult measure(ProcessTree system, ProcessTree model,
			CompareParameters parameters, ProMCanceller canceller) throws AutomatonFailedException,
			ReductionFailedException, InterruptedException, UnknownTreeNodeException, ProjectedMeasuresFailedException {
		CompareProcessTree2ProcessTree m = new CompareProcessTree2ProcessTree();
		EfficientTree efficientSystem = ProcessTree2EfficientTree.convert(system);
		EfficientTree efficientModel = ProcessTree2EfficientTree.convert(model);
		return m.apply(efficientSystem, efficientModel, parameters, canceller);
	}

	public static ProjectedRecallPrecisionResult measure(EfficientTree system, EfficientTree model,
			CompareParameters parameters, ProMCanceller canceller)
			throws AutomatonFailedException, InterruptedException, ProjectedMeasuresFailedException {
		CompareProcessTree2ProcessTree m = new CompareProcessTree2ProcessTree();
		return m.apply(system, model, parameters, canceller);
	}
}
