package org.processmining.projectedrecallandprecision.result;

import java.text.DecimalFormat;

import org.processmining.plugins.InductiveMiner.Pair;
import org.processmining.plugins.InductiveMiner.Triple;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTree;
import org.processmining.plugins.InductiveMiner.efficienttree.UnknownTreeNodeException;
import org.processmining.plugins.inductiveVisualMiner.alignedLogVisualisation.data.AlignedLogVisualisationDataImplEmpty;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

public class ProjectedRecallPrecisionAlignedLogVisualisation extends AlignedLogVisualisationDataImplEmpty {

	private final EfficientTree tree;
	private final ProjectedRecallPrecisionResult result;
	private final DecimalFormat df;

	public ProjectedRecallPrecisionAlignedLogVisualisation(EfficientTree tree, ProjectedRecallPrecisionResult result) {
		this.tree = tree;
		this.result = result;
		df = new DecimalFormat("#0.000");
	}

	public Pair<Long, Long> getExtremeCardinalities() throws UnknownTreeNodeException {
		return Pair.of(0l, Long.MAX_VALUE);
	}

	public Triple<String, Long, Long> getNodeLabel(int node, boolean includeModelMoves)
			throws UnknownTreeNodeException {
		String activity = tree.getActivityName(node);
		try {
			if (result.isRecallComputed() && result.isPrecisionComputed()) {
				return Triple.of(
						result.getRecallName() + ": " + df.format(result.getRecall(activity)) + "\nprecision: "
								+ df.format(result.getPrecision(activity)),
						(long) (Long.MAX_VALUE * result.getRecall(activity)),
						(long) (Long.MAX_VALUE * result.getPrecision(activity)));
			} else if (result.isRecallComputed()) {
				return Triple.of(result.getRecallName() + ": " + df.format(result.getRecall(activity)),
						(long) (Long.MAX_VALUE * result.getRecall(activity)),
						(long) (Long.MAX_VALUE * result.getRecall(activity)));
			} else if (result.isPrecisionComputed()) {
				return Triple.of("precision: " + df.format(result.getPrecision(activity)),
						(long) (Long.MAX_VALUE * result.getPrecision(activity)),
						(long) (Long.MAX_VALUE * result.getPrecision(activity)));
			} else {
				return Triple.of("", 0l, 0l);
			}
		} catch (ProjectedMeasuresFailedException e) {
			e.printStackTrace();
			return null;
		}
	}

}
