package org.processmining.projectedrecallandprecision.result;

import org.processmining.framework.util.collection.AlphanumComparator;
import org.processmining.projectedrecallandprecision.framework.CompareParameters.RecallName;

public interface ProjectedRecallPrecisionResult {

	/**
	 * 
	 * @param recall
	 * @param precision
	 * @param activities
	 *            Thread safe on unique activities
	 */
	public void set(double recall, double precision, String... activities);

	/**
	 * 
	 * @param activities
	 *            Can be of any size <= k.
	 * @return the (average) recall over all measurements performed for the
	 *         given number of activities.
	 * @throws ProjectedMeasuresFailedException
	 */
	public double getRecall(String... activities) throws ProjectedMeasuresFailedException;

	/**
	 * 
	 * @param activities
	 *            Can be of any size <= k.
	 * @return the (average) precision over all measurements performed for the
	 *         given number of activities.
	 * @throws ProjectedMeasuresFailedException
	 */
	public double getPrecision(String... activities) throws ProjectedMeasuresFailedException;

	/**
	 * 
	 * @return the number of measures that failed
	 */
	public int getNumberOfFailedMeasures();

	public int getK();

	public RecallName getRecallName();

	public String[] getActivities();

	public Object getSystem();

	public Object getModel();

	public boolean isRecallComputed();

	public boolean isPrecisionComputed();
	
	public static AlphanumComparator comparator = new AlphanumComparator();

	public static class ProjectedMeasuresFailedException extends Exception {
		private static final long serialVersionUID = 5061712138546581049L;
	}
}
