package org.processmining.projectedrecallandprecision.result;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.contexts.util.HtmlPanel;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTree;
import org.processmining.plugins.InductiveMiner.plugins.dialogs.IMMiningDialog;
import org.processmining.plugins.graphviz.colourMaps.ColourMapWhiteRed;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.visualisation.DotPanel;
import org.processmining.plugins.inductiveVisualMiner.helperClasses.IvMEfficientTree;
import org.processmining.plugins.inductiveVisualMiner.visualisation.ProcessTreeVisualisation;
import org.processmining.plugins.inductiveVisualMiner.visualisation.ProcessTreeVisualisationParameters;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

public class ProjectedRecallPrecisionVisualiserModel {
	@Plugin(level = PluginLevel.Regular, name = "Projected recall/precision model", returnLabels = {
			"Projected recall/precision model" }, returnTypes = {
					JComponent.class }, parameterLabels = { "Projected recall/precision" }, userAccessible = true)
	@Visualizer
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Show projected recall and precision", requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, ProjectedRecallPrecisionResult recallPrecisionResult)
			throws ProjectedMeasuresFailedException {

		if (recallPrecisionResult.getSystem() != null && recallPrecisionResult.getSystem() instanceof EfficientTree) {
			//visualise model
			ProcessTreeVisualisation visualisation = new ProcessTreeVisualisation();
			ProcessTreeVisualisationParameters parameters = new ProcessTreeVisualisationParameters();
			parameters.setColourNodes(new ColourMapWhiteRed());
			parameters.setColourNodesGradient(new ColourMapWhiteRed());
			Dot dot = visualisation.fancy(new IvMEfficientTree((EfficientTree) recallPrecisionResult.getSystem()),
					new ProjectedRecallPrecisionAlignedLogVisualisation(
							(EfficientTree) recallPrecisionResult.getSystem(),
							recallPrecisionResult),
					parameters).getA();
			return new DotPanel(dot);
		}

		return new HtmlPanel(ProjectedRecallPrecisionVisualiserTable.generateHtml(recallPrecisionResult, true));
	}
}
